/* valadostatement.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include "valadostatement.h"
#include "valaexpression.h"
#include "valasourcereference.h"
#include "valacodevisitor.h"

struct _ValaDoStatementPrivate {
	ValaStatement* _body;
	ValaExpression* _condition;
};
#define VALA_DO_STATEMENT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_DO_STATEMENT, ValaDoStatementPrivate))
enum  {
	VALA_DO_STATEMENT_DUMMY_PROPERTY,
	VALA_DO_STATEMENT_BODY,
	VALA_DO_STATEMENT_CONDITION,
};
static void vala_do_statement_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static void vala_do_statement_dispose (GObject * obj);

/**
 * Creates a new do statement.
 *
 * @param cond   loop condition
 * @param body   loop body
 * @param source reference to source code
 * @return       newly created do statement
 */
ValaDoStatement* vala_do_statement_new (ValaStatement* _body, ValaExpression* cond, ValaSourceReference* source)
{
	g_return_val_if_fail (VALA_IS_STATEMENT (_body), NULL);
	g_return_val_if_fail (VALA_IS_EXPRESSION (cond), NULL);
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	GParameter * __params = g_new0 (GParameter, 3);
	GParameter * __params_it = __params;
	(__params_it->name = "body", g_value_init (&__params_it->value, VALA_TYPE_STATEMENT), g_value_set_object (&__params_it->value, _body), __params_it++);
	(__params_it->name = "condition", g_value_init (&__params_it->value, VALA_TYPE_EXPRESSION), g_value_set_object (&__params_it->value, cond), __params_it++);
	(__params_it->name = "source-reference", g_value_init (&__params_it->value, VALA_TYPE_SOURCE_REFERENCE), g_value_set_object (&__params_it->value, source), __params_it++);
	ValaDoStatement * self = g_object_newv (VALA_TYPE_DO_STATEMENT, __params_it - __params, __params);
	return self;
}


static void vala_do_statement_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
{
	ValaDoStatement * self = VALA_DO_STATEMENT (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	vala_code_node_accept (VALA_CODE_NODE (vala_do_statement_get_body (self)), visitor);
	vala_code_node_accept (VALA_CODE_NODE (vala_do_statement_get_condition (self)), visitor);
	vala_code_visitor_visit_end_full_expression (visitor, vala_do_statement_get_condition (self));
	vala_code_visitor_visit_do_statement (visitor, self);
}


ValaStatement* vala_do_statement_get_body (ValaDoStatement* self)
{
	g_return_val_if_fail (VALA_IS_DO_STATEMENT (self), NULL);
	return self->priv->_body;
}


void vala_do_statement_set_body (ValaDoStatement* self, ValaStatement* value)
{
	g_return_if_fail (VALA_IS_DO_STATEMENT (self));
	ValaStatement* __temp2 = NULL;
	ValaStatement* __temp1 = NULL;
	self->priv->_body = (__temp2 = (__temp1 = value, (__temp1 == NULL ? __temp1 : g_object_ref (__temp1))), (self->priv->_body == NULL ? NULL : (self->priv->_body = (g_object_unref (self->priv->_body), NULL))), __temp2);
}


ValaExpression* vala_do_statement_get_condition (ValaDoStatement* self)
{
	g_return_val_if_fail (VALA_IS_DO_STATEMENT (self), NULL);
	return self->priv->_condition;
}


void vala_do_statement_set_condition (ValaDoStatement* self, ValaExpression* value)
{
	g_return_if_fail (VALA_IS_DO_STATEMENT (self));
	ValaExpression* __temp5 = NULL;
	ValaExpression* __temp4 = NULL;
	self->priv->_condition = (__temp5 = (__temp4 = value, (__temp4 == NULL ? __temp4 : g_object_ref (__temp4))), (self->priv->_condition == NULL ? NULL : (self->priv->_condition = (g_object_unref (self->priv->_condition), NULL))), __temp5);
}


void vala_do_statement_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaDoStatement * self = VALA_DO_STATEMENT (object);
	switch (property_id) {
		case VALA_DO_STATEMENT_BODY:
		g_value_set_object (value, vala_do_statement_get_body (self));
		break;
		case VALA_DO_STATEMENT_CONDITION:
		g_value_set_object (value, vala_do_statement_get_condition (self));
		break;
	}
}


void vala_do_statement_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaDoStatement * self = VALA_DO_STATEMENT (object);
	switch (property_id) {
		case VALA_DO_STATEMENT_BODY:
		vala_do_statement_set_body (self, g_value_get_object (value));
		break;
		case VALA_DO_STATEMENT_CONDITION:
		vala_do_statement_set_condition (self, g_value_get_object (value));
		break;
	}
}


static void vala_do_statement_class_init (ValaDoStatementClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaDoStatementPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_do_statement_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_do_statement_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_do_statement_dispose;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_do_statement_real_accept;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_DO_STATEMENT_BODY, g_param_spec_object ("body", "foo", "bar", VALA_TYPE_STATEMENT, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_DO_STATEMENT_CONDITION, g_param_spec_object ("condition", "foo", "bar", VALA_TYPE_EXPRESSION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void vala_do_statement_init (ValaDoStatement * self)
{
	self->priv = VALA_DO_STATEMENT_GET_PRIVATE (self);
}


static void vala_do_statement_dispose (GObject * obj)
{
	ValaDoStatement * self = VALA_DO_STATEMENT (obj);
	(self->priv->_body == NULL ? NULL : (self->priv->_body = (g_object_unref (self->priv->_body), NULL)));
	(self->priv->_condition == NULL ? NULL : (self->priv->_condition = (g_object_unref (self->priv->_condition), NULL)));
	ValaDoStatementClass * klass;
	GObjectClass * parent_class;
	klass = VALA_DO_STATEMENT_CLASS (g_type_class_peek (VALA_TYPE_DO_STATEMENT));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_do_statement_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaDoStatementClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_do_statement_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaDoStatement), 0, (GInstanceInitFunc) vala_do_statement_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_STATEMENT, "ValaDoStatement", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



