/* valacodegenerator.vala
 *
 * Copyright (C) 2006  Jürg Billeter, Raffaele Sandrini
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 *	Raffaele Sandrini <rasa@gmx.ch>
 */

#include "valacodegenerator.h"
#include <valaccodestruct.h>
#include <valaccodeenum.h>
#include <valaccodefunction.h>
#include <valaccodeincludedirective.h>
#include <valaccodenode.h>
#include <valaccodetypedefinition.h>
#include <valaccodevariabledeclarator.h>
#include <valaccodedeclarator.h>
#include <valaccodecomment.h>
#include <valaccodewriter.h>
#include <valaccodeoncesection.h>
#include <valaccodenewline.h>
#include <valaccodemacroreplacement.h>
#include <valaccodeformalparameter.h>
#include <valaccodemodifiers.h>
#include <valaccodefunctioncall.h>
#include <valaccodeconstant.h>
#include <valaccodeexpressionstatement.h>
#include <valaccodeassignment.h>
#include <valaccodememberaccess.h>
#include <valaccodedeclaration.h>
#include <valaccodeswitchstatement.h>
#include <valaccodecasestatement.h>
#include <valaccodebreakstatement.h>
#include <valaccodefunctiondeclarator.h>
#include <valaccodebinaryexpression.h>
#include <valaccodereturnstatement.h>
#include <valaccodeemptystatement.h>
#include <valaccodeinitializerlist.h>
#include <valaccodeconditionalexpression.h>
#include <valaccodeparenthesizedexpression.h>
#include <valaccodeifstatement.h>
#include <valaccodewhilestatement.h>
#include <valaccodeforstatement.h>
#include <valaccodecontinuestatement.h>
#include <valaccodeunaryexpression.h>
#include <valaccodeelementaccess.h>
#include <valaccodecastexpression.h>
#include "valasourcefilecycle.h"
#include "valanamespace.h"
#include "valasourcereference.h"
#include "valaclassregisterfunction.h"
#include "valatyperegisterfunction.h"
#include "valasignal.h"
#include "valamemberaccessibility.h"
#include "valainterfaceregisterfunction.h"
#include "valaarray.h"
#include "valaflags.h"
#include "valareport.h"
#include "valalocalvariabledeclaration.h"
#include "valainstancecast.h"

struct _ValaCodeGeneratorPrivate {
	gboolean _memory_management;
	ValaSymbol* root_symbol;
	ValaSymbol* current_symbol;
	ValaSymbol* current_type_symbol;
	ValaCCodeFragment* header_begin;
	ValaCCodeFragment* header_type_declaration;
	ValaCCodeFragment* header_type_definition;
	ValaCCodeFragment* header_type_member_declaration;
	ValaCCodeFragment* source_begin;
	ValaCCodeFragment* source_include_directives;
	ValaCCodeFragment* source_type_member_declaration;
	ValaCCodeFragment* source_type_member_definition;
	ValaCCodeStruct* instance_struct;
	ValaCCodeStruct* type_struct;
	ValaCCodeStruct* instance_priv_struct;
	ValaCCodeEnum* prop_enum;
	ValaCCodeEnum* cenum;
	ValaCCodeFunction* function;
	ValaCCodeBlock* block;
	GList* temp_vars;
	GList* temp_ref_vars;
	int next_temp_var_id;
	ValaTypeReference* bool_type;
	ValaTypeReference* int_type;
	ValaTypeReference* string_type;
};
#define VALA_CODE_GENERATOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_CODE_GENERATOR, ValaCodeGeneratorPrivate))
enum  {
	VALA_CODE_GENERATOR_DUMMY_PROPERTY,
	VALA_CODE_GENERATOR_MEMORY_MANAGEMENT,
};
static void vala_code_generator_real_visit_begin_source_file (ValaCodeVisitor* base, ValaSourceFile* source_file);
static char* vala_code_generator_get_define_for_filename (char* filename);
static void vala_code_generator_real_visit_end_source_file (ValaCodeVisitor* base, ValaSourceFile* source_file);
static void vala_code_generator_real_visit_begin_class (ValaCodeVisitor* base, ValaClass* cl);
static void vala_code_generator_real_visit_end_class (ValaCodeVisitor* base, ValaClass* cl);
static void vala_code_generator_add_class_init_function (ValaCodeGenerator* self, ValaClass* cl);
static void vala_code_generator_add_interface_init_function (ValaCodeGenerator* self, ValaClass* cl, ValaInterface* iface);
static void vala_code_generator_add_instance_init_function (ValaCodeGenerator* self, ValaClass* cl);
static void vala_code_generator_add_dispose_function (ValaCodeGenerator* self, ValaClass* cl);
static ValaCCodeIdentifier* vala_code_generator_get_value_setter_function (ValaCodeGenerator* self, ValaTypeReference* type_reference);
static void vala_code_generator_add_get_property_function (ValaCodeGenerator* self, ValaClass* cl);
static void vala_code_generator_add_set_property_function (ValaCodeGenerator* self, ValaClass* cl);
static void vala_code_generator_real_visit_begin_struct (ValaCodeVisitor* base, ValaStruct* st);
static void vala_code_generator_real_visit_begin_interface (ValaCodeVisitor* base, ValaInterface* iface);
static void vala_code_generator_real_visit_end_interface (ValaCodeVisitor* base, ValaInterface* iface);
static void vala_code_generator_real_visit_begin_enum (ValaCodeVisitor* base, ValaEnum* en);
static void vala_code_generator_real_visit_enum_value (ValaCodeVisitor* base, ValaEnumValue* ev);
static void vala_code_generator_real_visit_end_callback (ValaCodeVisitor* base, ValaCallback* cb);
static void vala_code_generator_real_visit_constant (ValaCodeVisitor* base, ValaConstant* c);
static void vala_code_generator_real_visit_field (ValaCodeVisitor* base, ValaField* f);
static void vala_code_generator_real_visit_begin_method (ValaCodeVisitor* base, ValaMethod* m);
static ValaCCodeStatement* vala_code_generator_create_method_type_check_statement (ValaCodeGenerator* self, ValaMethod* m, ValaDataType* t, gboolean non_null, char* var_name);
static ValaCCodeStatement* vala_code_generator_create_property_type_check_statement (ValaCodeGenerator* self, ValaProperty* prop, gboolean getter, ValaDataType* t, gboolean non_null, char* var_name);
static ValaCCodeStatement* vala_code_generator_create_type_check_statement (ValaCodeGenerator* self, ValaCodeNode* method_node, ValaDataType* ret_type, ValaDataType* t, gboolean non_null, char* var_name);
static ValaDataType* vala_code_generator_find_parent_type (ValaCodeGenerator* self, ValaCodeNode* node);
static void vala_code_generator_real_visit_end_method (ValaCodeVisitor* base, ValaMethod* m);
static void vala_code_generator_real_visit_formal_parameter (ValaCodeVisitor* base, ValaFormalParameter* p);
static void vala_code_generator_real_visit_end_property (ValaCodeVisitor* base, ValaProperty* prop);
static void vala_code_generator_real_visit_end_property_accessor (ValaCodeVisitor* base, ValaPropertyAccessor* acc);
static void vala_code_generator_real_visit_end_constructor (ValaCodeVisitor* base, ValaConstructor* c);
static void vala_code_generator_real_visit_begin_block (ValaCodeVisitor* base, ValaBlock* b);
static void vala_code_generator_add_object_creation (ValaCodeGenerator* self, ValaCCodeBlock* b);
static void vala_code_generator_real_visit_end_block (ValaCodeVisitor* base, ValaBlock* b);
static void vala_code_generator_real_visit_empty_statement (ValaCodeVisitor* base, ValaEmptyStatement* stmt);
static void vala_code_generator_real_visit_declaration_statement (ValaCodeVisitor* base, ValaDeclarationStatement* stmt);
static void vala_code_generator_real_visit_variable_declarator (ValaCodeVisitor* base, ValaVariableDeclarator* decl);
static void vala_code_generator_real_visit_initializer_list (ValaCodeVisitor* base, ValaInitializerList* list);
static ValaVariableDeclarator* vala_code_generator_get_temp_variable_declarator (ValaCodeGenerator* self, ValaTypeReference* type);
static ValaCCodeExpression* vala_code_generator_get_unref_expression (ValaCodeGenerator* self, ValaCCodeExpression* cvar, ValaTypeReference* type);
static void vala_code_generator_real_visit_end_full_expression (ValaCodeVisitor* base, ValaExpression* expr);
static void vala_code_generator_append_temp_decl (ValaCodeGenerator* self, ValaCCodeFragment* cfrag, GList* temp_vars);
static void vala_code_generator_real_visit_expression_statement (ValaCodeVisitor* base, ValaExpressionStatement* stmt);
static void vala_code_generator_create_temp_decl (ValaCodeGenerator* self, ValaStatement* stmt, GList* temp_vars);
static void vala_code_generator_real_visit_if_statement (ValaCodeVisitor* base, ValaIfStatement* stmt);
static void vala_code_generator_real_visit_while_statement (ValaCodeVisitor* base, ValaWhileStatement* stmt);
static void vala_code_generator_real_visit_for_statement (ValaCodeVisitor* base, ValaForStatement* stmt);
static void vala_code_generator_real_visit_end_foreach_statement (ValaCodeVisitor* base, ValaForeachStatement* stmt);
static void vala_code_generator_real_visit_break_statement (ValaCodeVisitor* base, ValaBreakStatement* stmt);
static void vala_code_generator_real_visit_continue_statement (ValaCodeVisitor* base, ValaContinueStatement* stmt);
static void vala_code_generator_append_local_free (ValaCodeGenerator* self, ValaSymbol* sym, ValaCCodeFragment* cfrag, gboolean stop_at_loop);
static void vala_code_generator_create_local_free (ValaCodeGenerator* self, ValaStatement* stmt);
static gboolean vala_code_generator_append_local_free_expr (ValaCodeGenerator* self, ValaSymbol* sym, ValaCCodeCommaExpression* ccomma, gboolean stop_at_loop);
static void vala_code_generator_create_local_free_expr (ValaCodeGenerator* self, ValaExpression* expr);
static void vala_code_generator_real_visit_return_statement (ValaCodeVisitor* base, ValaReturnStatement* stmt);
static void vala_code_generator_real_visit_boolean_literal (ValaCodeVisitor* base, ValaBooleanLiteral* expr);
static void vala_code_generator_real_visit_character_literal (ValaCodeVisitor* base, ValaCharacterLiteral* expr);
static void vala_code_generator_real_visit_integer_literal (ValaCodeVisitor* base, ValaIntegerLiteral* expr);
static void vala_code_generator_real_visit_real_literal (ValaCodeVisitor* base, ValaRealLiteral* expr);
static void vala_code_generator_real_visit_string_literal (ValaCodeVisitor* base, ValaStringLiteral* expr);
static void vala_code_generator_real_visit_null_literal (ValaCodeVisitor* base, ValaNullLiteral* expr);
static void vala_code_generator_real_visit_literal_expression (ValaCodeVisitor* base, ValaLiteralExpression* expr);
static void vala_code_generator_process_cmember (ValaCodeGenerator* self, ValaMemberAccess* expr, ValaCCodeExpression* pub_inst, ValaDataType* base_type);
static void vala_code_generator_real_visit_parenthesized_expression (ValaCodeVisitor* base, ValaParenthesizedExpression* expr);
static void vala_code_generator_real_visit_member_access (ValaCodeVisitor* base, ValaMemberAccess* expr);
static void vala_code_generator_real_visit_end_invocation_expression (ValaCodeVisitor* base, ValaInvocationExpression* expr);
static void vala_code_generator_real_visit_element_access (ValaCodeVisitor* base, ValaElementAccess* expr);
static void vala_code_generator_real_visit_postfix_expression (ValaCodeVisitor* base, ValaPostfixExpression* expr);
static ValaCCodeExpression* vala_code_generator_get_ref_expression (ValaCodeGenerator* self, ValaExpression* expr);
static void vala_code_generator_visit_expression (ValaCodeGenerator* self, ValaExpression* expr);
static void vala_code_generator_real_visit_end_object_creation_expression (ValaCodeVisitor* base, ValaObjectCreationExpression* expr);
static void vala_code_generator_real_visit_typeof_expression (ValaCodeVisitor* base, ValaTypeofExpression* expr);
static void vala_code_generator_real_visit_unary_expression (ValaCodeVisitor* base, ValaUnaryExpression* expr);
static void vala_code_generator_real_visit_cast_expression (ValaCodeVisitor* base, ValaCastExpression* expr);
static void vala_code_generator_real_visit_binary_expression (ValaCodeVisitor* base, ValaBinaryExpression* expr);
static void vala_code_generator_real_visit_type_check (ValaCodeVisitor* base, ValaTypeCheck* expr);
static void vala_code_generator_real_visit_conditional_expression (ValaCodeVisitor* base, ValaConditionalExpression* expr);
static void vala_code_generator_real_visit_end_lambda_expression (ValaCodeVisitor* base, ValaLambdaExpression* l);
static void vala_code_generator_real_visit_end_assignment (ValaCodeVisitor* base, ValaAssignment* a);
static void vala_code_generator_dispose (GObject * obj);

ValaCodeGenerator* vala_code_generator_new (gboolean manage_memory)
{
	GParameter * __params = g_new0 (GParameter, 1);
	GParameter * __params_it = __params;
	(__params_it->name = "memory-management", g_value_init (&__params_it->value, G_TYPE_BOOLEAN), g_value_set_boolean (&__params_it->value, manage_memory), __params_it++);
	ValaCodeGenerator * self = g_object_newv (VALA_TYPE_CODE_GENERATOR, __params_it - __params, __params);
	return self;
}


/**
 * Generate and emit C code for the specified code context.
 *
 * @param context a code context
 */
void vala_code_generator_emit (ValaCodeGenerator* self, ValaCodeContext* context)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (VALA_IS_CODE_CONTEXT (context));
	vala_code_context_find_header_cycles (context);
	ValaSymbol* __temp0 = NULL;
	self->priv->root_symbol = (__temp0 = g_object_ref (vala_code_context_get_root (context)), (self->priv->root_symbol == NULL ? NULL : (self->priv->root_symbol = (g_object_unref (self->priv->root_symbol), NULL))), __temp0);
	ValaTypeReference* __temp1 = NULL;
	self->priv->bool_type = (__temp1 = vala_type_reference_new (), (self->priv->bool_type == NULL ? NULL : (self->priv->bool_type = (g_object_unref (self->priv->bool_type), NULL))), __temp1);
	vala_type_reference_set_data_type (self->priv->bool_type, VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "bool"))));
	ValaTypeReference* __temp2 = NULL;
	self->priv->int_type = (__temp2 = vala_type_reference_new (), (self->priv->int_type == NULL ? NULL : (self->priv->int_type = (g_object_unref (self->priv->int_type), NULL))), __temp2);
	vala_type_reference_set_data_type (self->priv->int_type, VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "int"))));
	ValaTypeReference* __temp3 = NULL;
	self->priv->string_type = (__temp3 = vala_type_reference_new (), (self->priv->string_type == NULL ? NULL : (self->priv->string_type = (g_object_unref (self->priv->string_type), NULL))), __temp3);
	vala_type_reference_set_data_type (self->priv->string_type, VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "string"))));
	/* we're only interested in non-pkg source files */
	GList* source_files = vala_code_context_get_source_files (context);
	{
		GList* file_it;
		for (file_it = source_files; file_it != NULL; file_it = file_it->next) {
			ValaSourceFile* file = file_it->data;
			{
				if (!vala_source_file_get_pkg (file)) {
					vala_source_file_accept (file, VALA_CODE_VISITOR (self));
				}
			}
		}
	}
	(source_files == NULL ? NULL : (source_files = (g_list_free (source_files), NULL)));
}


static void vala_code_generator_real_visit_begin_source_file (ValaCodeVisitor* base, ValaSourceFile* source_file)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_SOURCE_FILE (source_file));
	ValaCCodeFragment* __temp4 = NULL;
	self->priv->header_begin = (__temp4 = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL), (self->priv->header_begin == NULL ? NULL : (self->priv->header_begin = (g_object_unref (self->priv->header_begin), NULL))), __temp4);
	ValaCCodeFragment* __temp5 = NULL;
	self->priv->header_type_declaration = (__temp5 = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL), (self->priv->header_type_declaration == NULL ? NULL : (self->priv->header_type_declaration = (g_object_unref (self->priv->header_type_declaration), NULL))), __temp5);
	ValaCCodeFragment* __temp6 = NULL;
	self->priv->header_type_definition = (__temp6 = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL), (self->priv->header_type_definition == NULL ? NULL : (self->priv->header_type_definition = (g_object_unref (self->priv->header_type_definition), NULL))), __temp6);
	ValaCCodeFragment* __temp7 = NULL;
	self->priv->header_type_member_declaration = (__temp7 = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL), (self->priv->header_type_member_declaration == NULL ? NULL : (self->priv->header_type_member_declaration = (g_object_unref (self->priv->header_type_member_declaration), NULL))), __temp7);
	ValaCCodeFragment* __temp8 = NULL;
	self->priv->source_begin = (__temp8 = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL), (self->priv->source_begin == NULL ? NULL : (self->priv->source_begin = (g_object_unref (self->priv->source_begin), NULL))), __temp8);
	ValaCCodeFragment* __temp9 = NULL;
	self->priv->source_include_directives = (__temp9 = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL), (self->priv->source_include_directives == NULL ? NULL : (self->priv->source_include_directives = (g_object_unref (self->priv->source_include_directives), NULL))), __temp9);
	ValaCCodeFragment* __temp10 = NULL;
	self->priv->source_type_member_declaration = (__temp10 = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL), (self->priv->source_type_member_declaration == NULL ? NULL : (self->priv->source_type_member_declaration = (g_object_unref (self->priv->source_type_member_declaration), NULL))), __temp10);
	ValaCCodeFragment* __temp11 = NULL;
	self->priv->source_type_member_definition = (__temp11 = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL), (self->priv->source_type_member_definition == NULL ? NULL : (self->priv->source_type_member_definition = (g_object_unref (self->priv->source_type_member_definition), NULL))), __temp11);
	self->priv->next_temp_var_id = 0;
	ValaCCodeIncludeDirective* __temp12 = NULL;
	vala_ccode_fragment_append (self->priv->header_begin, VALA_CCODE_NODE ((__temp12 = vala_ccode_include_directive_new ("glib.h", FALSE))));
	(__temp12 == NULL ? NULL : (__temp12 = (g_object_unref (__temp12), NULL)));
	ValaCCodeIncludeDirective* __temp13 = NULL;
	vala_ccode_fragment_append (self->priv->source_include_directives, VALA_CCODE_NODE ((__temp13 = vala_ccode_include_directive_new (vala_source_file_get_cheader_filename (source_file), TRUE))));
	(__temp13 == NULL ? NULL : (__temp13 = (g_object_unref (__temp13), NULL)));
	GList* used_includes = NULL;
	used_includes = g_list_append (used_includes, "glib.h");
	used_includes = g_list_append (used_includes, vala_source_file_get_cheader_filename (source_file));
	{
		GList* filename1_it;
		for (filename1_it = vala_source_file_get_header_external_includes (source_file); filename1_it != NULL; filename1_it = filename1_it->next) {
			char* filename1 = filename1_it->data;
			{
				if (g_list_find_custom (used_includes, filename1, strcmp) == NULL) {
					ValaCCodeIncludeDirective* __temp14 = NULL;
					vala_ccode_fragment_append (self->priv->header_begin, VALA_CCODE_NODE ((__temp14 = vala_ccode_include_directive_new (filename1, FALSE))));
					(__temp14 == NULL ? NULL : (__temp14 = (g_object_unref (__temp14), NULL)));
					used_includes = g_list_append (used_includes, filename1);
				}
			}
		}
	}
	{
		GList* filename2_it;
		for (filename2_it = vala_source_file_get_header_internal_includes (source_file); filename2_it != NULL; filename2_it = filename2_it->next) {
			char* filename2 = filename2_it->data;
			{
				if (g_list_find_custom (used_includes, filename2, strcmp) == NULL) {
					ValaCCodeIncludeDirective* __temp15 = NULL;
					vala_ccode_fragment_append (self->priv->header_begin, VALA_CCODE_NODE ((__temp15 = vala_ccode_include_directive_new (filename2, TRUE))));
					(__temp15 == NULL ? NULL : (__temp15 = (g_object_unref (__temp15), NULL)));
					used_includes = g_list_append (used_includes, filename2);
				}
			}
		}
	}
	{
		GList* filename3_it;
		for (filename3_it = vala_source_file_get_source_external_includes (source_file); filename3_it != NULL; filename3_it = filename3_it->next) {
			char* filename3 = filename3_it->data;
			{
				if (g_list_find_custom (used_includes, filename3, strcmp) == NULL) {
					ValaCCodeIncludeDirective* __temp16 = NULL;
					vala_ccode_fragment_append (self->priv->source_include_directives, VALA_CCODE_NODE ((__temp16 = vala_ccode_include_directive_new (filename3, FALSE))));
					(__temp16 == NULL ? NULL : (__temp16 = (g_object_unref (__temp16), NULL)));
					used_includes = g_list_append (used_includes, filename3);
				}
			}
		}
	}
	{
		GList* filename4_it;
		for (filename4_it = vala_source_file_get_source_internal_includes (source_file); filename4_it != NULL; filename4_it = filename4_it->next) {
			char* filename4 = filename4_it->data;
			{
				if (g_list_find_custom (used_includes, filename4, strcmp) == NULL) {
					ValaCCodeIncludeDirective* __temp17 = NULL;
					vala_ccode_fragment_append (self->priv->source_include_directives, VALA_CCODE_NODE ((__temp17 = vala_ccode_include_directive_new (filename4, TRUE))));
					(__temp17 == NULL ? NULL : (__temp17 = (g_object_unref (__temp17), NULL)));
					used_includes = g_list_append (used_includes, filename4);
				}
			}
		}
	}
	if (vala_source_file_get_is_cycle_head (source_file)) {
		{
			GList* cycle_file_it;
			for (cycle_file_it = vala_source_file_get_cycle (source_file)->files; cycle_file_it != NULL; cycle_file_it = cycle_file_it->next) {
				ValaSourceFile* cycle_file = cycle_file_it->data;
				{
					GList* namespaces = vala_source_file_get_namespaces (cycle_file);
					{
						GList* ns_it;
						for (ns_it = namespaces; ns_it != NULL; ns_it = ns_it->next) {
							ValaNamespace* ns = ns_it->data;
							{
								GList* structs = vala_namespace_get_structs (ns);
								{
									GList* st_it;
									for (st_it = structs; st_it != NULL; st_it = st_it->next) {
										ValaStruct* st = st_it->data;
										{
											ValaCCodeTypeDefinition* __temp18 = NULL;
											vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp18 = vala_ccode_type_definition_new (g_strdup_printf ("struct _%s", vala_data_type_get_cname (VALA_DATA_TYPE (st))), VALA_CCODE_DECLARATOR (vala_ccode_variable_declarator_new (vala_data_type_get_cname (VALA_DATA_TYPE (st))))))));
											(__temp18 == NULL ? NULL : (__temp18 = (g_object_unref (__temp18), NULL)));
										}
									}
								}
								GList* classes = vala_namespace_get_classes (ns);
								{
									GList* cl_it;
									for (cl_it = classes; cl_it != NULL; cl_it = cl_it->next) {
										ValaClass* cl = cl_it->data;
										{
											ValaCCodeTypeDefinition* __temp19 = NULL;
											vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp19 = vala_ccode_type_definition_new (g_strdup_printf ("struct _%s", vala_data_type_get_cname (VALA_DATA_TYPE (cl))), VALA_CCODE_DECLARATOR (vala_ccode_variable_declarator_new (vala_data_type_get_cname (VALA_DATA_TYPE (cl))))))));
											(__temp19 == NULL ? NULL : (__temp19 = (g_object_unref (__temp19), NULL)));
											ValaCCodeTypeDefinition* __temp20 = NULL;
											vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp20 = vala_ccode_type_definition_new (g_strdup_printf ("struct _%sClass", vala_data_type_get_cname (VALA_DATA_TYPE (cl))), VALA_CCODE_DECLARATOR (vala_ccode_variable_declarator_new (g_strdup_printf ("%sClass", vala_data_type_get_cname (VALA_DATA_TYPE (cl)))))))));
											(__temp20 == NULL ? NULL : (__temp20 = (g_object_unref (__temp20), NULL)));
										}
									}
								}
								(structs == NULL ? NULL : (structs = (g_list_free (structs), NULL)));
								(classes == NULL ? NULL : (classes = (g_list_free (classes), NULL)));
							}
						}
					}
					(namespaces == NULL ? NULL : (namespaces = (g_list_free (namespaces), NULL)));
				}
			}
		}
	}
	(used_includes == NULL ? NULL : (used_includes = (g_list_free (used_includes), NULL)));
}


static char* vala_code_generator_get_define_for_filename (char* filename)
{
	g_return_val_if_fail (filename != NULL, NULL);
	GString* define = g_string_new ("__");
	char* i = filename;
	while (g_utf8_strlen (i, -1) > 0) {
		gunichar c = g_utf8_get_char (i);
		/* FIXME: remove explicit cast when implicit cast works */
		if (g_unichar_isalnum (c) && c < 128) {
			g_string_append_unichar (define, g_unichar_toupper (c));
		} else {
			g_string_append_c (define, '_');
		}
		i = g_utf8_next_char (i);
	}
	g_string_append (define, "__");
	char* __temp21 = NULL;
	char* __temp22 = NULL;
	return (__temp22 = (__temp21 = define->str, (__temp21 == NULL ? __temp21 : g_strdup (__temp21))), (define == NULL ? NULL : (define = (g_string_free (define, TRUE), NULL))), __temp22);
	(define == NULL ? NULL : (define = (g_string_free (define, TRUE), NULL)));
}


static void vala_code_generator_real_visit_end_source_file (ValaCodeVisitor* base, ValaSourceFile* source_file)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_SOURCE_FILE (source_file));
	char* header_define = vala_code_generator_get_define_for_filename (vala_source_file_get_cheader_filename (source_file));
	ValaCCodeComment* comment = NULL;
	if (vala_source_file_get_comment (source_file) != NULL) {
		ValaCCodeComment* __temp23 = NULL;
		comment = (__temp23 = vala_ccode_comment_new (vala_source_file_get_comment (source_file)), (comment == NULL ? NULL : (comment = (g_object_unref (comment), NULL))), __temp23);
	}
	ValaCCodeWriter* writer = vala_ccode_writer_new (vala_source_file_get_cheader_filename (source_file));
	if (comment != NULL) {
		vala_ccode_node_write (VALA_CCODE_NODE (comment), writer);
	}
	vala_ccode_writer_write_newline (writer);
	ValaCCodeOnceSection* once = vala_ccode_once_section_new (header_define);
	ValaCCodeNewline* __temp24 = NULL;
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE ((__temp24 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp24 == NULL ? NULL : (__temp24 = (g_object_unref (__temp24), NULL)));
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE (self->priv->header_begin));
	ValaCCodeNewline* __temp25 = NULL;
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE ((__temp25 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp25 == NULL ? NULL : (__temp25 = (g_object_unref (__temp25), NULL)));
	ValaCCodeIdentifier* __temp26 = NULL;
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE ((__temp26 = vala_ccode_identifier_new ("G_BEGIN_DECLS"))));
	(__temp26 == NULL ? NULL : (__temp26 = (g_object_unref (__temp26), NULL)));
	ValaCCodeNewline* __temp27 = NULL;
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE ((__temp27 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp27 == NULL ? NULL : (__temp27 = (g_object_unref (__temp27), NULL)));
	ValaCCodeNewline* __temp28 = NULL;
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE ((__temp28 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp28 == NULL ? NULL : (__temp28 = (g_object_unref (__temp28), NULL)));
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE (self->priv->header_type_declaration));
	ValaCCodeNewline* __temp29 = NULL;
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE ((__temp29 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp29 == NULL ? NULL : (__temp29 = (g_object_unref (__temp29), NULL)));
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE (self->priv->header_type_definition));
	ValaCCodeNewline* __temp30 = NULL;
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE ((__temp30 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp30 == NULL ? NULL : (__temp30 = (g_object_unref (__temp30), NULL)));
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE (self->priv->header_type_member_declaration));
	ValaCCodeNewline* __temp31 = NULL;
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE ((__temp31 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp31 == NULL ? NULL : (__temp31 = (g_object_unref (__temp31), NULL)));
	ValaCCodeIdentifier* __temp32 = NULL;
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE ((__temp32 = vala_ccode_identifier_new ("G_END_DECLS"))));
	(__temp32 == NULL ? NULL : (__temp32 = (g_object_unref (__temp32), NULL)));
	ValaCCodeNewline* __temp33 = NULL;
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE ((__temp33 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp33 == NULL ? NULL : (__temp33 = (g_object_unref (__temp33), NULL)));
	ValaCCodeNewline* __temp34 = NULL;
	vala_ccode_fragment_append (VALA_CCODE_FRAGMENT (once), VALA_CCODE_NODE ((__temp34 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp34 == NULL ? NULL : (__temp34 = (g_object_unref (__temp34), NULL)));
	vala_ccode_node_write (VALA_CCODE_NODE (once), writer);
	vala_ccode_writer_close (writer);
	ValaCCodeWriter* __temp35 = NULL;
	writer = (__temp35 = vala_ccode_writer_new (vala_source_file_get_csource_filename (source_file)), (writer == NULL ? NULL : (writer = (g_object_unref (writer), NULL))), __temp35);
	if (comment != NULL) {
		vala_ccode_node_write (VALA_CCODE_NODE (comment), writer);
	}
	vala_ccode_node_write (VALA_CCODE_NODE (self->priv->source_begin), writer);
	vala_ccode_writer_write_newline (writer);
	vala_ccode_node_write (VALA_CCODE_NODE (self->priv->source_include_directives), writer);
	vala_ccode_writer_write_newline (writer);
	vala_ccode_node_write (VALA_CCODE_NODE (self->priv->source_type_member_declaration), writer);
	vala_ccode_writer_write_newline (writer);
	vala_ccode_node_write (VALA_CCODE_NODE (self->priv->source_type_member_definition), writer);
	vala_ccode_writer_write_newline (writer);
	vala_ccode_writer_close (writer);
	ValaCCodeFragment* __temp36 = NULL;
	self->priv->header_begin = (__temp36 = NULL, (self->priv->header_begin == NULL ? NULL : (self->priv->header_begin = (g_object_unref (self->priv->header_begin), NULL))), __temp36);
	ValaCCodeFragment* __temp37 = NULL;
	self->priv->header_type_declaration = (__temp37 = NULL, (self->priv->header_type_declaration == NULL ? NULL : (self->priv->header_type_declaration = (g_object_unref (self->priv->header_type_declaration), NULL))), __temp37);
	ValaCCodeFragment* __temp38 = NULL;
	self->priv->header_type_definition = (__temp38 = NULL, (self->priv->header_type_definition == NULL ? NULL : (self->priv->header_type_definition = (g_object_unref (self->priv->header_type_definition), NULL))), __temp38);
	ValaCCodeFragment* __temp39 = NULL;
	self->priv->header_type_member_declaration = (__temp39 = NULL, (self->priv->header_type_member_declaration == NULL ? NULL : (self->priv->header_type_member_declaration = (g_object_unref (self->priv->header_type_member_declaration), NULL))), __temp39);
	ValaCCodeFragment* __temp40 = NULL;
	self->priv->source_begin = (__temp40 = NULL, (self->priv->source_begin == NULL ? NULL : (self->priv->source_begin = (g_object_unref (self->priv->source_begin), NULL))), __temp40);
	ValaCCodeFragment* __temp41 = NULL;
	self->priv->source_include_directives = (__temp41 = NULL, (self->priv->source_include_directives == NULL ? NULL : (self->priv->source_include_directives = (g_object_unref (self->priv->source_include_directives), NULL))), __temp41);
	ValaCCodeFragment* __temp42 = NULL;
	self->priv->source_type_member_declaration = (__temp42 = NULL, (self->priv->source_type_member_declaration == NULL ? NULL : (self->priv->source_type_member_declaration = (g_object_unref (self->priv->source_type_member_declaration), NULL))), __temp42);
	ValaCCodeFragment* __temp43 = NULL;
	self->priv->source_type_member_definition = (__temp43 = NULL, (self->priv->source_type_member_definition == NULL ? NULL : (self->priv->source_type_member_definition = (g_object_unref (self->priv->source_type_member_definition), NULL))), __temp43);
	(header_define == NULL ? NULL : (header_define = (g_free (header_define), NULL)));
	(comment == NULL ? NULL : (comment = (g_object_unref (comment), NULL)));
	(writer == NULL ? NULL : (writer = (g_object_unref (writer), NULL)));
	(once == NULL ? NULL : (once = (g_object_unref (once), NULL)));
}


static void vala_code_generator_real_visit_begin_class (ValaCodeVisitor* base, ValaClass* cl)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_CLASS (cl));
	ValaSymbol* __temp45 = NULL;
	ValaSymbol* __temp44 = NULL;
	self->priv->current_symbol = (__temp45 = (__temp44 = vala_code_node_get_symbol (VALA_CODE_NODE (cl)), (__temp44 == NULL ? __temp44 : g_object_ref (__temp44))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp45);
	ValaSymbol* __temp47 = NULL;
	ValaSymbol* __temp46 = NULL;
	self->priv->current_type_symbol = (__temp47 = (__temp46 = vala_code_node_get_symbol (VALA_CODE_NODE (cl)), (__temp46 == NULL ? __temp46 : g_object_ref (__temp46))), (self->priv->current_type_symbol == NULL ? NULL : (self->priv->current_type_symbol = (g_object_unref (self->priv->current_type_symbol), NULL))), __temp47);
	ValaCCodeStruct* __temp48 = NULL;
	self->priv->instance_struct = (__temp48 = vala_ccode_struct_new (g_strdup_printf ("_%s", vala_data_type_get_cname (VALA_DATA_TYPE (cl)))), (self->priv->instance_struct == NULL ? NULL : (self->priv->instance_struct = (g_object_unref (self->priv->instance_struct), NULL))), __temp48);
	ValaCCodeStruct* __temp49 = NULL;
	self->priv->type_struct = (__temp49 = vala_ccode_struct_new (g_strdup_printf ("_%sClass", vala_data_type_get_cname (VALA_DATA_TYPE (cl)))), (self->priv->type_struct == NULL ? NULL : (self->priv->type_struct = (g_object_unref (self->priv->type_struct), NULL))), __temp49);
	ValaCCodeStruct* __temp50 = NULL;
	self->priv->instance_priv_struct = (__temp50 = vala_ccode_struct_new (g_strdup_printf ("_%sPrivate", vala_data_type_get_cname (VALA_DATA_TYPE (cl)))), (self->priv->instance_priv_struct == NULL ? NULL : (self->priv->instance_priv_struct = (g_object_unref (self->priv->instance_priv_struct), NULL))), __temp50);
	ValaCCodeEnum* __temp51 = NULL;
	self->priv->prop_enum = (__temp51 = vala_ccode_enum_new (NULL), (self->priv->prop_enum == NULL ? NULL : (self->priv->prop_enum = (g_object_unref (self->priv->prop_enum), NULL))), __temp51);
	char* __temp53 = NULL;
	char* __temp52 = NULL;
	vala_ccode_enum_add_value (self->priv->prop_enum, (__temp53 = g_strdup_printf ("%s_DUMMY_PROPERTY", (__temp52 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL)))), NULL);
	(__temp53 == NULL ? NULL : (__temp53 = (g_free (__temp53), NULL)));
	(__temp52 == NULL ? NULL : (__temp52 = (g_free (__temp52), NULL)));
	ValaCCodeNewline* __temp54 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp54 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp54 == NULL ? NULL : (__temp54 = (g_object_unref (__temp54), NULL)));
	char* __temp55 = NULL;
	char* __temp56 = NULL;
	char* macro = (__temp56 = g_strdup_printf ("(%s_get_type ())", (__temp55 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL))), (__temp55 == NULL ? NULL : (__temp55 = (g_free (__temp55), NULL))), __temp56);
	ValaCCodeMacroReplacement* __temp57 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp57 = vala_ccode_macro_replacement_new (vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), "TYPE_"), macro))));
	(__temp57 == NULL ? NULL : (__temp57 = (g_object_unref (__temp57), NULL)));
	char* __temp59 = NULL;
	char* __temp58 = NULL;
	macro = (__temp59 = g_strdup_printf ("(G_TYPE_CHECK_INSTANCE_CAST ((obj), %s, %s))", (__temp58 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), "TYPE_")), vala_data_type_get_cname (VALA_DATA_TYPE (cl))), (macro == NULL ? NULL : (macro = (g_free (macro), NULL))), __temp59);
	(__temp58 == NULL ? NULL : (__temp58 = (g_free (__temp58), NULL)));
	ValaCCodeMacroReplacement* __temp61 = NULL;
	char* __temp60 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp61 = vala_ccode_macro_replacement_new (g_strdup_printf ("%s(obj)", (__temp60 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL))), macro))));
	(__temp61 == NULL ? NULL : (__temp61 = (g_object_unref (__temp61), NULL)));
	(__temp60 == NULL ? NULL : (__temp60 = (g_free (__temp60), NULL)));
	char* __temp63 = NULL;
	char* __temp62 = NULL;
	macro = (__temp63 = g_strdup_printf ("(G_TYPE_CHECK_CLASS_CAST ((klass), %s, %sClass))", (__temp62 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), "TYPE_")), vala_data_type_get_cname (VALA_DATA_TYPE (cl))), (macro == NULL ? NULL : (macro = (g_free (macro), NULL))), __temp63);
	(__temp62 == NULL ? NULL : (__temp62 = (g_free (__temp62), NULL)));
	ValaCCodeMacroReplacement* __temp65 = NULL;
	char* __temp64 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp65 = vala_ccode_macro_replacement_new (g_strdup_printf ("%s_CLASS(klass)", (__temp64 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL))), macro))));
	(__temp65 == NULL ? NULL : (__temp65 = (g_object_unref (__temp65), NULL)));
	(__temp64 == NULL ? NULL : (__temp64 = (g_free (__temp64), NULL)));
	char* __temp67 = NULL;
	char* __temp66 = NULL;
	macro = (__temp67 = g_strdup_printf ("(G_TYPE_CHECK_INSTANCE_TYPE ((obj), %s))", (__temp66 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), "TYPE_"))), (macro == NULL ? NULL : (macro = (g_free (macro), NULL))), __temp67);
	(__temp66 == NULL ? NULL : (__temp66 = (g_free (__temp66), NULL)));
	ValaCCodeMacroReplacement* __temp69 = NULL;
	char* __temp68 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp69 = vala_ccode_macro_replacement_new (g_strdup_printf ("%s(obj)", (__temp68 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), "IS_"))), macro))));
	(__temp69 == NULL ? NULL : (__temp69 = (g_object_unref (__temp69), NULL)));
	(__temp68 == NULL ? NULL : (__temp68 = (g_free (__temp68), NULL)));
	char* __temp71 = NULL;
	char* __temp70 = NULL;
	macro = (__temp71 = g_strdup_printf ("(G_TYPE_CHECK_CLASS_TYPE ((klass), %s))", (__temp70 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), "TYPE_"))), (macro == NULL ? NULL : (macro = (g_free (macro), NULL))), __temp71);
	(__temp70 == NULL ? NULL : (__temp70 = (g_free (__temp70), NULL)));
	ValaCCodeMacroReplacement* __temp73 = NULL;
	char* __temp72 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp73 = vala_ccode_macro_replacement_new (g_strdup_printf ("%s_CLASS(klass)", (__temp72 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), "IS_"))), macro))));
	(__temp73 == NULL ? NULL : (__temp73 = (g_object_unref (__temp73), NULL)));
	(__temp72 == NULL ? NULL : (__temp72 = (g_free (__temp72), NULL)));
	char* __temp75 = NULL;
	char* __temp74 = NULL;
	macro = (__temp75 = g_strdup_printf ("(G_TYPE_INSTANCE_GET_CLASS ((obj), %s, %sClass))", (__temp74 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), "TYPE_")), vala_data_type_get_cname (VALA_DATA_TYPE (cl))), (macro == NULL ? NULL : (macro = (g_free (macro), NULL))), __temp75);
	(__temp74 == NULL ? NULL : (__temp74 = (g_free (__temp74), NULL)));
	ValaCCodeMacroReplacement* __temp77 = NULL;
	char* __temp76 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp77 = vala_ccode_macro_replacement_new (g_strdup_printf ("%s_GET_CLASS(obj)", (__temp76 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL))), macro))));
	(__temp77 == NULL ? NULL : (__temp77 = (g_object_unref (__temp77), NULL)));
	(__temp76 == NULL ? NULL : (__temp76 = (g_free (__temp76), NULL)));
	ValaCCodeNewline* __temp78 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp78 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp78 == NULL ? NULL : (__temp78 = (g_object_unref (__temp78), NULL)));
	if (vala_source_file_get_cycle (vala_source_reference_get_file (vala_code_node_get_source_reference (VALA_CODE_NODE (cl)))) == NULL) {
		ValaCCodeTypeDefinition* __temp79 = NULL;
		vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp79 = vala_ccode_type_definition_new (g_strdup_printf ("struct %s", vala_ccode_struct_get_name (self->priv->instance_struct)), VALA_CCODE_DECLARATOR (vala_ccode_variable_declarator_new (vala_data_type_get_cname (VALA_DATA_TYPE (cl))))))));
		(__temp79 == NULL ? NULL : (__temp79 = (g_object_unref (__temp79), NULL)));
		ValaCCodeTypeDefinition* __temp80 = NULL;
		vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp80 = vala_ccode_type_definition_new (g_strdup_printf ("struct %s", vala_ccode_struct_get_name (self->priv->type_struct)), VALA_CCODE_DECLARATOR (vala_ccode_variable_declarator_new (g_strdup_printf ("%sClass", vala_data_type_get_cname (VALA_DATA_TYPE (cl)))))))));
		(__temp80 == NULL ? NULL : (__temp80 = (g_object_unref (__temp80), NULL)));
	}
	ValaCCodeTypeDefinition* __temp81 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp81 = vala_ccode_type_definition_new (g_strdup_printf ("struct %s", vala_ccode_struct_get_name (self->priv->instance_priv_struct)), VALA_CCODE_DECLARATOR (vala_ccode_variable_declarator_new (g_strdup_printf ("%sPrivate", vala_data_type_get_cname (VALA_DATA_TYPE (cl)))))))));
	(__temp81 == NULL ? NULL : (__temp81 = (g_object_unref (__temp81), NULL)));
	vala_ccode_struct_add_field (self->priv->instance_struct, vala_data_type_get_cname (VALA_DATA_TYPE (vala_class_get_base_class (cl))), "parent");
	char* __temp82 = NULL;
	vala_ccode_struct_add_field (self->priv->instance_struct, (__temp82 = g_strdup_printf ("%sPrivate *", vala_data_type_get_cname (VALA_DATA_TYPE (cl)))), "priv");
	(__temp82 == NULL ? NULL : (__temp82 = (g_free (__temp82), NULL)));
	char* __temp83 = NULL;
	vala_ccode_struct_add_field (self->priv->type_struct, (__temp83 = g_strdup_printf ("%sClass", vala_data_type_get_cname (VALA_DATA_TYPE (vala_class_get_base_class (cl))))), "parent");
	(__temp83 == NULL ? NULL : (__temp83 = (g_free (__temp83), NULL)));
	if (vala_source_reference_get_comment (vala_code_node_get_source_reference (VALA_CODE_NODE (cl))) != NULL) {
		ValaCCodeComment* __temp84 = NULL;
		vala_ccode_fragment_append (self->priv->header_type_definition, VALA_CCODE_NODE ((__temp84 = vala_ccode_comment_new (vala_source_reference_get_comment (vala_code_node_get_source_reference (VALA_CODE_NODE (cl)))))));
		(__temp84 == NULL ? NULL : (__temp84 = (g_object_unref (__temp84), NULL)));
	}
	vala_ccode_fragment_append (self->priv->header_type_definition, VALA_CCODE_NODE (self->priv->instance_struct));
	vala_ccode_fragment_append (self->priv->header_type_definition, VALA_CCODE_NODE (self->priv->type_struct));
	vala_ccode_fragment_append (self->priv->source_type_member_declaration, VALA_CCODE_NODE (self->priv->instance_priv_struct));
	char* __temp86 = NULL;
	char* __temp85 = NULL;
	macro = (__temp86 = g_strdup_printf ("(G_TYPE_INSTANCE_GET_PRIVATE ((o), %s, %sPrivate))", (__temp85 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), "TYPE_")), vala_data_type_get_cname (VALA_DATA_TYPE (cl))), (macro == NULL ? NULL : (macro = (g_free (macro), NULL))), __temp86);
	(__temp85 == NULL ? NULL : (__temp85 = (g_free (__temp85), NULL)));
	ValaCCodeMacroReplacement* __temp88 = NULL;
	char* __temp87 = NULL;
	vala_ccode_fragment_append (self->priv->source_type_member_declaration, VALA_CCODE_NODE ((__temp88 = vala_ccode_macro_replacement_new (g_strdup_printf ("%s_GET_PRIVATE(o)", (__temp87 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL))), macro))));
	(__temp88 == NULL ? NULL : (__temp88 = (g_object_unref (__temp88), NULL)));
	(__temp87 == NULL ? NULL : (__temp87 = (g_free (__temp87), NULL)));
	vala_ccode_fragment_append (self->priv->source_type_member_declaration, VALA_CCODE_NODE (self->priv->prop_enum));
	(macro == NULL ? NULL : (macro = (g_free (macro), NULL)));
}


static void vala_code_generator_real_visit_end_class (ValaCodeVisitor* base, ValaClass* cl)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_CLASS (cl));
	vala_code_generator_add_get_property_function (self, cl);
	vala_code_generator_add_set_property_function (self, cl);
	vala_code_generator_add_class_init_function (self, cl);
	{
		GList* base_type_it;
		for (base_type_it = vala_class_get_base_types (cl); base_type_it != NULL; base_type_it = base_type_it->next) {
			ValaTypeReference* base_type = base_type_it->data;
			{
				if (VALA_IS_INTERFACE (vala_type_reference_get_data_type (base_type))) {
					vala_code_generator_add_interface_init_function (self, cl, VALA_INTERFACE (vala_type_reference_get_data_type (base_type)));
				}
			}
		}
	}
	vala_code_generator_add_instance_init_function (self, cl);
	if (vala_code_generator_get_memory_management (self)) {
		vala_code_generator_add_dispose_function (self, cl);
	}
	ValaClassRegisterFunction* type_fun = vala_class_register_function_new (cl);
	vala_type_register_function_init_from_type (VALA_TYPE_REGISTER_FUNCTION (type_fun));
	ValaCCodeFunction* __temp89 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_member_declaration, VALA_CCODE_NODE ((__temp89 = vala_type_register_function_get_declaration (VALA_TYPE_REGISTER_FUNCTION (type_fun)))));
	(__temp89 == NULL ? NULL : (__temp89 = (g_object_unref (__temp89), NULL)));
	vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (type_fun));
	ValaSymbol* __temp90 = NULL;
	self->priv->current_type_symbol = (__temp90 = NULL, (self->priv->current_type_symbol == NULL ? NULL : (self->priv->current_type_symbol = (g_object_unref (self->priv->current_type_symbol), NULL))), __temp90);
	(type_fun == NULL ? NULL : (type_fun = (g_object_unref (type_fun), NULL)));
}


static void vala_code_generator_add_class_init_function (ValaCodeGenerator* self, ValaClass* cl)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (VALA_IS_CLASS (cl));
	char* __temp91 = NULL;
	ValaCCodeFunction* __temp92 = NULL;
	ValaCCodeFunction* class_init = (__temp92 = vala_ccode_function_new (g_strdup_printf ("%s_class_init", (__temp91 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL))), "void"), (__temp91 == NULL ? NULL : (__temp91 = (g_free (__temp91), NULL))), __temp92);
	ValaCCodeFormalParameter* __temp93 = NULL;
	vala_ccode_function_add_parameter (class_init, (__temp93 = vala_ccode_formal_parameter_new ("klass", g_strdup_printf ("%sClass *", vala_data_type_get_cname (VALA_DATA_TYPE (cl))))));
	(__temp93 == NULL ? NULL : (__temp93 = (g_object_unref (__temp93), NULL)));
	vala_ccode_function_set_modifiers (class_init, VALA_CCODE_MODIFIERS_STATIC);
	ValaCCodeBlock* init_block = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
	vala_ccode_function_set_block (class_init, init_block);
	ValaCCodeFunctionCall* ccall = NULL;
	if (vala_class_get_has_private_fields (cl)) {
		ValaCCodeFunctionCall* __temp94 = NULL;
		ccall = (__temp94 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_type_class_add_private"))), (ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL))), __temp94);
		ValaCCodeIdentifier* __temp95 = NULL;
		vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp95 = vala_ccode_identifier_new ("klass"))));
		(__temp95 == NULL ? NULL : (__temp95 = (g_object_unref (__temp95), NULL)));
		ValaCCodeConstant* __temp96 = NULL;
		vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp96 = vala_ccode_constant_new (g_strdup_printf ("sizeof (%sPrivate)", vala_data_type_get_cname (VALA_DATA_TYPE (cl)))))));
		(__temp96 == NULL ? NULL : (__temp96 = (g_object_unref (__temp96), NULL)));
		ValaCCodeExpressionStatement* __temp97 = NULL;
		vala_ccode_block_add_statement (init_block, VALA_CCODE_NODE ((__temp97 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (ccall)))));
		(__temp97 == NULL ? NULL : (__temp97 = (g_object_unref (__temp97), NULL)));
	}
	ValaCCodeFunctionCall* __temp98 = NULL;
	ccall = (__temp98 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("G_OBJECT_CLASS"))), (ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL))), __temp98);
	ValaCCodeIdentifier* __temp99 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp99 = vala_ccode_identifier_new ("klass"))));
	(__temp99 == NULL ? NULL : (__temp99 = (g_object_unref (__temp99), NULL)));
	ValaCCodeExpressionStatement* __temp101 = NULL;
	char* __temp100 = NULL;
	vala_ccode_block_add_statement (init_block, VALA_CCODE_NODE ((__temp101 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (ccall), "get_property")), VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (g_strdup_printf ("%s_get_property", (__temp100 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL))))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))))));
	(__temp101 == NULL ? NULL : (__temp101 = (g_object_unref (__temp101), NULL)));
	(__temp100 == NULL ? NULL : (__temp100 = (g_free (__temp100), NULL)));
	ValaCCodeExpressionStatement* __temp103 = NULL;
	char* __temp102 = NULL;
	vala_ccode_block_add_statement (init_block, VALA_CCODE_NODE ((__temp103 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (ccall), "set_property")), VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (g_strdup_printf ("%s_set_property", (__temp102 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL))))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))))));
	(__temp103 == NULL ? NULL : (__temp103 = (g_object_unref (__temp103), NULL)));
	(__temp102 == NULL ? NULL : (__temp102 = (g_free (__temp102), NULL)));
	if (vala_class_get_constructor (cl) != NULL) {
		ValaCCodeFunctionCall* ccast = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("G_OBJECT_CLASS")));
		ValaCCodeIdentifier* __temp104 = NULL;
		vala_ccode_function_call_add_argument (ccast, VALA_CCODE_EXPRESSION ((__temp104 = vala_ccode_identifier_new ("klass"))));
		(__temp104 == NULL ? NULL : (__temp104 = (g_object_unref (__temp104), NULL)));
		ValaCCodeExpressionStatement* __temp106 = NULL;
		char* __temp105 = NULL;
		vala_ccode_block_add_statement (init_block, VALA_CCODE_NODE ((__temp106 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (ccast), "constructor")), VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (g_strdup_printf ("%s_constructor", (__temp105 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL))))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))))));
		(__temp106 == NULL ? NULL : (__temp106 = (g_object_unref (__temp106), NULL)));
		(__temp105 == NULL ? NULL : (__temp105 = (g_free (__temp105), NULL)));
		(ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL)));
	}
	GList* __temp107 = NULL;
	gboolean __temp108;
	if ((__temp108 = vala_code_generator_get_memory_management (self) && (__temp107 = vala_class_get_fields (cl)) != NULL, (__temp107 == NULL ? NULL : (__temp107 = (g_list_free (__temp107), NULL))), __temp108)) {
		ValaCCodeFunctionCall* ccast = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("G_OBJECT_CLASS")));
		ValaCCodeIdentifier* __temp109 = NULL;
		vala_ccode_function_call_add_argument (ccast, VALA_CCODE_EXPRESSION ((__temp109 = vala_ccode_identifier_new ("klass"))));
		(__temp109 == NULL ? NULL : (__temp109 = (g_object_unref (__temp109), NULL)));
		ValaCCodeExpressionStatement* __temp111 = NULL;
		char* __temp110 = NULL;
		vala_ccode_block_add_statement (init_block, VALA_CCODE_NODE ((__temp111 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (ccast), "dispose")), VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (g_strdup_printf ("%s_dispose", (__temp110 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL))))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))))));
		(__temp111 == NULL ? NULL : (__temp111 = (g_object_unref (__temp111), NULL)));
		(__temp110 == NULL ? NULL : (__temp110 = (g_free (__temp110), NULL)));
		(ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL)));
	}
	GList* methods = vala_class_get_methods (cl);
	{
		GList* m_it;
		for (m_it = methods; m_it != NULL; m_it = m_it->next) {
			ValaMethod* m = m_it->data;
			{
				if (!vala_method_get_is_virtual (m) && !vala_method_get_overrides (m)) {
					continue;
				}
				ValaCodeNode* base_type = vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (vala_method_get_base_method (m)))));
				if (VALA_IS_INTERFACE (base_type)) {
					continue;
				}
				char* __temp112 = NULL;
				ValaCCodeFunctionCall* __temp113 = NULL;
				ValaCCodeFunctionCall* ccast = (__temp113 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (g_strdup_printf ("%s_CLASS", (__temp112 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE ((VALA_CLASS (base_type))), NULL)))))), (__temp112 == NULL ? NULL : (__temp112 = (g_free (__temp112), NULL))), __temp113);
				ValaCCodeIdentifier* __temp114 = NULL;
				vala_ccode_function_call_add_argument (ccast, VALA_CCODE_EXPRESSION ((__temp114 = vala_ccode_identifier_new ("klass"))));
				(__temp114 == NULL ? NULL : (__temp114 = (g_object_unref (__temp114), NULL)));
				ValaCCodeExpressionStatement* __temp115 = NULL;
				vala_ccode_block_add_statement (init_block, VALA_CCODE_NODE ((__temp115 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (ccast), vala_method_get_name (m))), VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_method_get_real_cname (m))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))))));
				(__temp115 == NULL ? NULL : (__temp115 = (g_object_unref (__temp115), NULL)));
				(ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL)));
			}
		}
	}
	GList* props = vala_class_get_properties (cl);
	{
		GList* prop_it;
		for (prop_it = props; prop_it != NULL; prop_it = prop_it->next) {
			ValaProperty* prop = prop_it->data;
			{
				ValaCCodeFunctionCall* cinst = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_object_class_install_property")));
				vala_ccode_function_call_add_argument (cinst, VALA_CCODE_EXPRESSION (ccall));
				ValaCCodeConstant* __temp116 = NULL;
				vala_ccode_function_call_add_argument (cinst, VALA_CCODE_EXPRESSION ((__temp116 = vala_ccode_constant_new (vala_property_get_upper_case_cname (prop)))));
				(__temp116 == NULL ? NULL : (__temp116 = (g_object_unref (__temp116), NULL)));
				ValaCCodeFunctionCall* cspec = vala_ccode_function_call_new (NULL);
				ValaCCodeConstant* __temp117 = NULL;
				vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp117 = vala_property_get_canonical_cconstant (prop))));
				(__temp117 == NULL ? NULL : (__temp117 = (g_object_unref (__temp117), NULL)));
				ValaCCodeConstant* __temp118 = NULL;
				vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp118 = vala_ccode_constant_new ("\"foo\""))));
				(__temp118 == NULL ? NULL : (__temp118 = (g_object_unref (__temp118), NULL)));
				ValaCCodeConstant* __temp119 = NULL;
				vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp119 = vala_ccode_constant_new ("\"bar\""))));
				(__temp119 == NULL ? NULL : (__temp119 = (g_object_unref (__temp119), NULL)));
				if (VALA_IS_CLASS (vala_type_reference_get_data_type (vala_property_get_type_reference (prop)))) {
					ValaCCodeIdentifier* __temp120 = NULL;
					vala_ccode_function_call_set_call (cspec, VALA_CCODE_EXPRESSION ((__temp120 = vala_ccode_identifier_new ("g_param_spec_object"))));
					(__temp120 == NULL ? NULL : (__temp120 = (g_object_unref (__temp120), NULL)));
					ValaCCodeIdentifier* __temp121 = NULL;
					vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp121 = vala_ccode_identifier_new (vala_data_type_get_upper_case_cname (vala_type_reference_get_data_type (vala_property_get_type_reference (prop)), "TYPE_")))));
					(__temp121 == NULL ? NULL : (__temp121 = (g_object_unref (__temp121), NULL)));
				} else {
					if (vala_type_reference_get_data_type (vala_property_get_type_reference (prop)) == vala_type_reference_get_data_type (self->priv->string_type)) {
						ValaCCodeIdentifier* __temp122 = NULL;
						vala_ccode_function_call_set_call (cspec, VALA_CCODE_EXPRESSION ((__temp122 = vala_ccode_identifier_new ("g_param_spec_string"))));
						(__temp122 == NULL ? NULL : (__temp122 = (g_object_unref (__temp122), NULL)));
						ValaCCodeConstant* __temp123 = NULL;
						vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp123 = vala_ccode_constant_new ("NULL"))));
						(__temp123 == NULL ? NULL : (__temp123 = (g_object_unref (__temp123), NULL)));
					} else {
						if (vala_type_reference_get_data_type (vala_property_get_type_reference (prop)) == vala_type_reference_get_data_type (self->priv->int_type) || VALA_IS_ENUM (vala_type_reference_get_data_type (vala_property_get_type_reference (prop)))) {
							ValaCCodeIdentifier* __temp124 = NULL;
							vala_ccode_function_call_set_call (cspec, VALA_CCODE_EXPRESSION ((__temp124 = vala_ccode_identifier_new ("g_param_spec_int"))));
							(__temp124 == NULL ? NULL : (__temp124 = (g_object_unref (__temp124), NULL)));
							ValaCCodeConstant* __temp125 = NULL;
							vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp125 = vala_ccode_constant_new ("G_MININT"))));
							(__temp125 == NULL ? NULL : (__temp125 = (g_object_unref (__temp125), NULL)));
							ValaCCodeConstant* __temp126 = NULL;
							vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp126 = vala_ccode_constant_new ("G_MAXINT"))));
							(__temp126 == NULL ? NULL : (__temp126 = (g_object_unref (__temp126), NULL)));
							ValaCCodeConstant* __temp127 = NULL;
							vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp127 = vala_ccode_constant_new ("0"))));
							(__temp127 == NULL ? NULL : (__temp127 = (g_object_unref (__temp127), NULL)));
						} else {
							if (vala_type_reference_get_data_type (vala_property_get_type_reference (prop)) == vala_type_reference_get_data_type (self->priv->bool_type)) {
								ValaCCodeIdentifier* __temp128 = NULL;
								vala_ccode_function_call_set_call (cspec, VALA_CCODE_EXPRESSION ((__temp128 = vala_ccode_identifier_new ("g_param_spec_boolean"))));
								(__temp128 == NULL ? NULL : (__temp128 = (g_object_unref (__temp128), NULL)));
								ValaCCodeConstant* __temp129 = NULL;
								vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp129 = vala_ccode_constant_new ("FALSE"))));
								(__temp129 == NULL ? NULL : (__temp129 = (g_object_unref (__temp129), NULL)));
							} else {
								ValaCCodeIdentifier* __temp130 = NULL;
								vala_ccode_function_call_set_call (cspec, VALA_CCODE_EXPRESSION ((__temp130 = vala_ccode_identifier_new ("g_param_spec_pointer"))));
								(__temp130 == NULL ? NULL : (__temp130 = (g_object_unref (__temp130), NULL)));
							}
						}
					}
				}
				char* pflags = g_strdup ("G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB");
				if (vala_property_get_get_accessor (prop) != NULL) {
					char* __temp131 = NULL;
					pflags = (__temp131 = g_strdup_printf ("%s%s", pflags, " | G_PARAM_READABLE"), (pflags == NULL ? NULL : (pflags = (g_free (pflags), NULL))), __temp131);
				}
				if (vala_property_get_set_accessor (prop) != NULL) {
					char* __temp132 = NULL;
					pflags = (__temp132 = g_strdup_printf ("%s%s", pflags, " | G_PARAM_WRITABLE"), (pflags == NULL ? NULL : (pflags = (g_free (pflags), NULL))), __temp132);
					if (vala_property_accessor_get_construction (vala_property_get_set_accessor (prop))) {
						if (vala_property_accessor_get_writable (vala_property_get_set_accessor (prop))) {
							char* __temp133 = NULL;
							pflags = (__temp133 = g_strdup_printf ("%s%s", pflags, " | G_PARAM_CONSTRUCT"), (pflags == NULL ? NULL : (pflags = (g_free (pflags), NULL))), __temp133);
						} else {
							char* __temp134 = NULL;
							pflags = (__temp134 = g_strdup_printf ("%s%s", pflags, " | G_PARAM_CONSTRUCT_ONLY"), (pflags == NULL ? NULL : (pflags = (g_free (pflags), NULL))), __temp134);
						}
					}
				}
				ValaCCodeConstant* __temp135 = NULL;
				vala_ccode_function_call_add_argument (cspec, VALA_CCODE_EXPRESSION ((__temp135 = vala_ccode_constant_new (pflags))));
				(__temp135 == NULL ? NULL : (__temp135 = (g_object_unref (__temp135), NULL)));
				vala_ccode_function_call_add_argument (cinst, VALA_CCODE_EXPRESSION (cspec));
				ValaCCodeExpressionStatement* __temp136 = NULL;
				vala_ccode_block_add_statement (init_block, VALA_CCODE_NODE ((__temp136 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (cinst)))));
				(__temp136 == NULL ? NULL : (__temp136 = (g_object_unref (__temp136), NULL)));
				(cinst == NULL ? NULL : (cinst = (g_object_unref (cinst), NULL)));
				(cspec == NULL ? NULL : (cspec = (g_object_unref (cspec), NULL)));
				(pflags == NULL ? NULL : (pflags = (g_free (pflags), NULL)));
			}
		}
	}
	{
		GList* sig_it;
		for (sig_it = vala_class_get_signals (cl); sig_it != NULL; sig_it = sig_it->next) {
			ValaSignal* sig = sig_it->data;
			{
				ValaCCodeFunctionCall* csignew = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_signal_new")));
				ValaCCodeConstant* __temp137 = NULL;
				vala_ccode_function_call_add_argument (csignew, VALA_CCODE_EXPRESSION ((__temp137 = vala_ccode_constant_new (g_strdup_printf ("\"%s\"", vala_signal_get_name (sig))))));
				(__temp137 == NULL ? NULL : (__temp137 = (g_object_unref (__temp137), NULL)));
				ValaCCodeIdentifier* __temp138 = NULL;
				vala_ccode_function_call_add_argument (csignew, VALA_CCODE_EXPRESSION ((__temp138 = vala_ccode_identifier_new (vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), "TYPE_")))));
				(__temp138 == NULL ? NULL : (__temp138 = (g_object_unref (__temp138), NULL)));
				ValaCCodeConstant* __temp139 = NULL;
				vala_ccode_function_call_add_argument (csignew, VALA_CCODE_EXPRESSION ((__temp139 = vala_ccode_constant_new ("G_SIGNAL_RUN_LAST"))));
				(__temp139 == NULL ? NULL : (__temp139 = (g_object_unref (__temp139), NULL)));
				ValaCCodeConstant* __temp140 = NULL;
				vala_ccode_function_call_add_argument (csignew, VALA_CCODE_EXPRESSION ((__temp140 = vala_ccode_constant_new ("0"))));
				(__temp140 == NULL ? NULL : (__temp140 = (g_object_unref (__temp140), NULL)));
				ValaCCodeConstant* __temp141 = NULL;
				vala_ccode_function_call_add_argument (csignew, VALA_CCODE_EXPRESSION ((__temp141 = vala_ccode_constant_new ("NULL"))));
				(__temp141 == NULL ? NULL : (__temp141 = (g_object_unref (__temp141), NULL)));
				ValaCCodeConstant* __temp142 = NULL;
				vala_ccode_function_call_add_argument (csignew, VALA_CCODE_EXPRESSION ((__temp142 = vala_ccode_constant_new ("NULL"))));
				(__temp142 == NULL ? NULL : (__temp142 = (g_object_unref (__temp142), NULL)));
				/* TODO: generate marshallers */
				char* marshaller = g_strdup ("g_cclosure_marshal");
				ValaCCodeIdentifier* marshal_arg = vala_ccode_identifier_new (marshaller);
				vala_ccode_function_call_add_argument (csignew, VALA_CCODE_EXPRESSION (marshal_arg));
				GList* params = vala_signal_get_parameters (sig);
				unsigned int params_len = g_list_length (params);
				if (vala_type_reference_get_data_type (vala_signal_get_return_type (sig)) == NULL) {
					char* __temp143 = NULL;
					marshaller = (__temp143 = g_strdup_printf ("%s_VOID_", marshaller), (marshaller == NULL ? NULL : (marshaller = (g_free (marshaller), NULL))), __temp143);
					ValaCCodeConstant* __temp144 = NULL;
					vala_ccode_function_call_add_argument (csignew, VALA_CCODE_EXPRESSION ((__temp144 = vala_ccode_constant_new ("G_TYPE_NONE"))));
					(__temp144 == NULL ? NULL : (__temp144 = (g_object_unref (__temp144), NULL)));
				} else {
					char* __temp145 = NULL;
					marshaller = (__temp145 = g_strdup_printf ("%s_%s_", marshaller, vala_data_type_get_marshaller_type_name (vala_type_reference_get_data_type (vala_signal_get_return_type (sig)))), (marshaller == NULL ? NULL : (marshaller = (g_free (marshaller), NULL))), __temp145);
					ValaCCodeConstant* __temp146 = NULL;
					vala_ccode_function_call_add_argument (csignew, VALA_CCODE_EXPRESSION ((__temp146 = vala_ccode_constant_new (vala_data_type_get_type_id (vala_type_reference_get_data_type (vala_signal_get_return_type (sig)))))));
					(__temp146 == NULL ? NULL : (__temp146 = (g_object_unref (__temp146), NULL)));
				}
				ValaCCodeConstant* __temp147 = NULL;
				vala_ccode_function_call_add_argument (csignew, VALA_CCODE_EXPRESSION ((__temp147 = vala_ccode_constant_new (g_strdup_printf ("%d", params_len)))));
				(__temp147 == NULL ? NULL : (__temp147 = (g_object_unref (__temp147), NULL)));
				{
					GList* param_it;
					for (param_it = params; param_it != NULL; param_it = param_it->next) {
						ValaFormalParameter* param = param_it->data;
						{
							char* __temp148 = NULL;
							marshaller = (__temp148 = g_strdup_printf ("%s_%s", marshaller, vala_data_type_get_marshaller_type_name (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param)))), (marshaller == NULL ? NULL : (marshaller = (g_free (marshaller), NULL))), __temp148);
							ValaCCodeConstant* __temp149 = NULL;
							vala_ccode_function_call_add_argument (csignew, VALA_CCODE_EXPRESSION ((__temp149 = vala_ccode_constant_new (vala_data_type_get_type_id (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param)))))));
							(__temp149 == NULL ? NULL : (__temp149 = (g_object_unref (__temp149), NULL)));
						}
					}
				}
				if (params_len == 0) {
					char* __temp150 = NULL;
					marshaller = (__temp150 = g_strdup_printf ("%s_VOID", marshaller), (marshaller == NULL ? NULL : (marshaller = (g_free (marshaller), NULL))), __temp150);
				}
				vala_ccode_identifier_set_name (marshal_arg, marshaller);
				ValaCCodeExpressionStatement* __temp151 = NULL;
				vala_ccode_block_add_statement (init_block, VALA_CCODE_NODE ((__temp151 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (csignew)))));
				(__temp151 == NULL ? NULL : (__temp151 = (g_object_unref (__temp151), NULL)));
				(csignew == NULL ? NULL : (csignew = (g_object_unref (csignew), NULL)));
				(marshaller == NULL ? NULL : (marshaller = (g_free (marshaller), NULL)));
				(marshal_arg == NULL ? NULL : (marshal_arg = (g_object_unref (marshal_arg), NULL)));
				(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
			}
		}
	}
	vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (class_init));
	(class_init == NULL ? NULL : (class_init = (g_object_unref (class_init), NULL)));
	(init_block == NULL ? NULL : (init_block = (g_object_unref (init_block), NULL)));
	(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
	(methods == NULL ? NULL : (methods = (g_list_free (methods), NULL)));
	(props == NULL ? NULL : (props = (g_list_free (props), NULL)));
}


static void vala_code_generator_add_interface_init_function (ValaCodeGenerator* self, ValaClass* cl, ValaInterface* iface)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (VALA_IS_CLASS (cl));
	g_return_if_fail (VALA_IS_INTERFACE (iface));
	char* __temp153 = NULL;
	char* __temp152 = NULL;
	ValaCCodeFunction* __temp154 = NULL;
	ValaCCodeFunction* iface_init = (__temp154 = vala_ccode_function_new (g_strdup_printf ("%s_%s_interface_init", (__temp152 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL)), (__temp153 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (iface), NULL))), "void"), (__temp153 == NULL ? NULL : (__temp153 = (g_free (__temp153), NULL))), (__temp152 == NULL ? NULL : (__temp152 = (g_free (__temp152), NULL))), __temp154);
	ValaCCodeFormalParameter* __temp155 = NULL;
	vala_ccode_function_add_parameter (iface_init, (__temp155 = vala_ccode_formal_parameter_new ("iface", g_strdup_printf ("%sInterface *", vala_data_type_get_cname (VALA_DATA_TYPE (iface))))));
	(__temp155 == NULL ? NULL : (__temp155 = (g_object_unref (__temp155), NULL)));
	vala_ccode_function_set_modifiers (iface_init, VALA_CCODE_MODIFIERS_STATIC);
	ValaCCodeBlock* init_block = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
	vala_ccode_function_set_block (iface_init, init_block);
	ValaCCodeFunctionCall* ccall = NULL;
	GList* methods = vala_class_get_methods (cl);
	{
		GList* m_it;
		for (m_it = methods; m_it != NULL; m_it = m_it->next) {
			ValaMethod* m = m_it->data;
			{
				if (!vala_method_get_overrides (m)) {
					continue;
				}
				ValaCodeNode* base_type = vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (vala_method_get_base_method (m)))));
				if (base_type != iface) {
					continue;
				}
				ValaCCodeIdentifier* ciface = vala_ccode_identifier_new ("iface");
				ValaCCodeExpressionStatement* __temp156 = NULL;
				vala_ccode_block_add_statement (init_block, VALA_CCODE_NODE ((__temp156 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (ciface), vala_method_get_name (m))), VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_method_get_real_cname (m))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))))));
				(__temp156 == NULL ? NULL : (__temp156 = (g_object_unref (__temp156), NULL)));
				(ciface == NULL ? NULL : (ciface = (g_object_unref (ciface), NULL)));
			}
		}
	}
	vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (iface_init));
	(iface_init == NULL ? NULL : (iface_init = (g_object_unref (iface_init), NULL)));
	(init_block == NULL ? NULL : (init_block = (g_object_unref (init_block), NULL)));
	(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
	(methods == NULL ? NULL : (methods = (g_list_free (methods), NULL)));
}


static void vala_code_generator_add_instance_init_function (ValaCodeGenerator* self, ValaClass* cl)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (VALA_IS_CLASS (cl));
	char* __temp157 = NULL;
	ValaCCodeFunction* __temp158 = NULL;
	ValaCCodeFunction* instance_init = (__temp158 = vala_ccode_function_new (g_strdup_printf ("%s_init", (__temp157 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL))), "void"), (__temp157 == NULL ? NULL : (__temp157 = (g_free (__temp157), NULL))), __temp158);
	ValaCCodeFormalParameter* __temp159 = NULL;
	vala_ccode_function_add_parameter (instance_init, (__temp159 = vala_ccode_formal_parameter_new ("self", g_strdup_printf ("%s *", vala_data_type_get_cname (VALA_DATA_TYPE (cl))))));
	(__temp159 == NULL ? NULL : (__temp159 = (g_object_unref (__temp159), NULL)));
	vala_ccode_function_set_modifiers (instance_init, VALA_CCODE_MODIFIERS_STATIC);
	ValaCCodeBlock* init_block = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
	vala_ccode_function_set_block (instance_init, init_block);
	if (vala_class_get_has_private_fields (cl)) {
		char* __temp160 = NULL;
		ValaCCodeFunctionCall* __temp161 = NULL;
		ValaCCodeFunctionCall* ccall = (__temp161 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (g_strdup_printf ("%s_GET_PRIVATE", (__temp160 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL)))))), (__temp160 == NULL ? NULL : (__temp160 = (g_free (__temp160), NULL))), __temp161);
		ValaCCodeIdentifier* __temp162 = NULL;
		vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp162 = vala_ccode_identifier_new ("self"))));
		(__temp162 == NULL ? NULL : (__temp162 = (g_object_unref (__temp162), NULL)));
		ValaCCodeExpressionStatement* __temp163 = NULL;
		vala_ccode_block_add_statement (init_block, VALA_CCODE_NODE ((__temp163 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("self")), "priv")), VALA_CCODE_EXPRESSION (ccall), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))))));
		(__temp163 == NULL ? NULL : (__temp163 = (g_object_unref (__temp163), NULL)));
		(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
	}
	GList* fields = vala_class_get_fields (cl);
	{
		GList* f_it;
		for (f_it = fields; f_it != NULL; f_it = f_it->next) {
			ValaField* f = f_it->data;
			{
				if (vala_field_get_initializer (f) != NULL) {
					ValaCCodeExpression* lhs = NULL;
					if (vala_field_get_instance (f)) {
						if (f->access == VALA_MEMBER_ACCESSIBILITY_PRIVATE) {
							ValaCCodeExpression* __temp164 = NULL;
							lhs = (__temp164 = VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("self")), "priv")), vala_field_get_cname (f))), (lhs == NULL ? NULL : (lhs = (g_object_unref (lhs), NULL))), __temp164);
						} else {
							ValaCCodeExpression* __temp165 = NULL;
							lhs = (__temp165 = VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("self")), vala_field_get_cname (f))), (lhs == NULL ? NULL : (lhs = (g_object_unref (lhs), NULL))), __temp165);
						}
					}
					/* else {
					lhs = new CCodeIdentifier ("%s_%s".printf (cl.get_lower_case_cname (null), f.get_cname ()));
					} */
					if (lhs != NULL) {
						ValaCCodeExpressionStatement* __temp166 = NULL;
						vala_ccode_block_add_statement (init_block, VALA_CCODE_NODE ((__temp166 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_assignment_new (lhs, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_field_get_initializer (f)))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))))));
						(__temp166 == NULL ? NULL : (__temp166 = (g_object_unref (__temp166), NULL)));
					}
					(lhs == NULL ? NULL : (lhs = (g_object_unref (lhs), NULL)));
				}
			}
		}
	}
	ValaSymbol* init_sym = vala_symbol_lookup (vala_code_node_get_symbol (VALA_CODE_NODE (cl)), "init");
	if (init_sym != NULL) {
		ValaMethod* init_fun = VALA_METHOD (vala_symbol_get_node (init_sym));
		vala_ccode_block_add_statement (init_block, vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_method_get_body (init_fun))));
	}
	vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (instance_init));
	(instance_init == NULL ? NULL : (instance_init = (g_object_unref (instance_init), NULL)));
	(init_block == NULL ? NULL : (init_block = (g_object_unref (init_block), NULL)));
	(fields == NULL ? NULL : (fields = (g_list_free (fields), NULL)));
}


static void vala_code_generator_add_dispose_function (ValaCodeGenerator* self, ValaClass* cl)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (VALA_IS_CLASS (cl));
	ValaCCodeFunction* __temp168 = NULL;
	char* __temp167 = NULL;
	self->priv->function = (__temp168 = vala_ccode_function_new (g_strdup_printf ("%s_dispose", (__temp167 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL))), "void"), (self->priv->function == NULL ? NULL : (self->priv->function = (g_object_unref (self->priv->function), NULL))), __temp168);
	(__temp167 == NULL ? NULL : (__temp167 = (g_free (__temp167), NULL)));
	vala_ccode_function_set_modifiers (self->priv->function, VALA_CCODE_MODIFIERS_STATIC);
	ValaCCodeFormalParameter* __temp169 = NULL;
	vala_ccode_function_add_parameter (self->priv->function, (__temp169 = vala_ccode_formal_parameter_new ("obj", "GObject *")));
	(__temp169 == NULL ? NULL : (__temp169 = (g_object_unref (__temp169), NULL)));
	ValaCCodeFunction* __temp170 = NULL;
	vala_ccode_fragment_append (self->priv->source_type_member_declaration, VALA_CCODE_NODE ((__temp170 = vala_ccode_function_copy (self->priv->function))));
	(__temp170 == NULL ? NULL : (__temp170 = (g_object_unref (__temp170), NULL)));
	ValaCCodeBlock* cblock = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
	ValaCCodeFunctionCall* ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL))));
	ValaCCodeIdentifier* __temp171 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp171 = vala_ccode_identifier_new ("obj"))));
	(__temp171 == NULL ? NULL : (__temp171 = (g_object_unref (__temp171), NULL)));
	ValaCCodeDeclaration* cdecl = vala_ccode_declaration_new (g_strdup_printf ("%s *", vala_data_type_get_cname (VALA_DATA_TYPE (cl))));
	ValaCCodeVariableDeclarator* __temp172 = NULL;
	vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp172 = vala_ccode_variable_declarator_new_with_initializer ("self", VALA_CCODE_EXPRESSION (ccall)))));
	(__temp172 == NULL ? NULL : (__temp172 = (g_object_unref (__temp172), NULL)));
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (cdecl));
	GList* fields = vala_class_get_fields (cl);
	{
		GList* f_it;
		for (f_it = fields; f_it != NULL; f_it = f_it->next) {
			ValaField* f = f_it->data;
			{
				if (vala_field_get_instance (f) && vala_type_reference_get_takes_ownership (vala_field_get_type_reference (f))) {
					ValaCCodeIdentifier* cself = vala_ccode_identifier_new ("self");
					ValaCCodeIdentifier* __temp173 = NULL;
					ValaCCodeExpression* cstruct = VALA_CCODE_EXPRESSION ((__temp173 = cself, (__temp173 == NULL ? __temp173 : g_object_ref (__temp173))));
					if (f->access == VALA_MEMBER_ACCESSIBILITY_PRIVATE) {
						ValaCCodeExpression* __temp174 = NULL;
						cstruct = (__temp174 = VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (cself), "priv")), (cstruct == NULL ? NULL : (cstruct = (g_object_unref (cstruct), NULL))), __temp174);
					}
					ValaCCodeMemberAccess* cfield = vala_ccode_member_access_new_pointer (cstruct, vala_field_get_cname (f));
					ValaCCodeExpressionStatement* __temp175 = NULL;
					vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE ((__temp175 = vala_ccode_expression_statement_new (vala_code_generator_get_unref_expression (self, VALA_CCODE_EXPRESSION (cfield), vala_field_get_type_reference (f))))));
					(__temp175 == NULL ? NULL : (__temp175 = (g_object_unref (__temp175), NULL)));
					(cself == NULL ? NULL : (cself = (g_object_unref (cself), NULL)));
					(cstruct == NULL ? NULL : (cstruct = (g_object_unref (cstruct), NULL)));
					(cfield == NULL ? NULL : (cfield = (g_object_unref (cfield), NULL)));
				}
			}
		}
	}
	ValaCCodeDeclaration* __temp176 = NULL;
	cdecl = (__temp176 = vala_ccode_declaration_new (g_strdup_printf ("%sClass *", vala_data_type_get_cname (VALA_DATA_TYPE (cl)))), (cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL))), __temp176);
	ValaCCodeVariableDeclarator* __temp177 = NULL;
	vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp177 = vala_ccode_variable_declarator_new ("klass"))));
	(__temp177 == NULL ? NULL : (__temp177 = (g_object_unref (__temp177), NULL)));
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (cdecl));
	ValaCCodeDeclaration* __temp178 = NULL;
	cdecl = (__temp178 = vala_ccode_declaration_new ("GObjectClass *"), (cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL))), __temp178);
	ValaCCodeVariableDeclarator* __temp179 = NULL;
	vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp179 = vala_ccode_variable_declarator_new ("parent_class"))));
	(__temp179 == NULL ? NULL : (__temp179 = (g_object_unref (__temp179), NULL)));
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (cdecl));
	ValaCCodeFunctionCall* __temp180 = NULL;
	ccall = (__temp180 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_type_class_peek"))), (ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL))), __temp180);
	ValaCCodeIdentifier* __temp181 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp181 = vala_ccode_identifier_new (vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), "TYPE_")))));
	(__temp181 == NULL ? NULL : (__temp181 = (g_object_unref (__temp181), NULL)));
	char* __temp182 = NULL;
	ValaCCodeFunctionCall* __temp183 = NULL;
	ValaCCodeFunctionCall* ccast = (__temp183 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (g_strdup_printf ("%s_CLASS", (__temp182 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL)))))), (__temp182 == NULL ? NULL : (__temp182 = (g_free (__temp182), NULL))), __temp183);
	vala_ccode_function_call_add_argument (ccast, VALA_CCODE_EXPRESSION (ccall));
	ValaCCodeExpressionStatement* __temp184 = NULL;
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE ((__temp184 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("klass")), VALA_CCODE_EXPRESSION (ccast), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))))));
	(__temp184 == NULL ? NULL : (__temp184 = (g_object_unref (__temp184), NULL)));
	ValaCCodeFunctionCall* __temp185 = NULL;
	ccall = (__temp185 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_type_class_peek_parent"))), (ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL))), __temp185);
	ValaCCodeIdentifier* __temp186 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp186 = vala_ccode_identifier_new ("klass"))));
	(__temp186 == NULL ? NULL : (__temp186 = (g_object_unref (__temp186), NULL)));
	ValaCCodeFunctionCall* __temp187 = NULL;
	ccast = (__temp187 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("G_OBJECT_CLASS"))), (ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL))), __temp187);
	vala_ccode_function_call_add_argument (ccast, VALA_CCODE_EXPRESSION (ccall));
	ValaCCodeExpressionStatement* __temp188 = NULL;
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE ((__temp188 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("parent_class")), VALA_CCODE_EXPRESSION (ccast), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))))));
	(__temp188 == NULL ? NULL : (__temp188 = (g_object_unref (__temp188), NULL)));
	ValaCCodeFunctionCall* __temp189 = NULL;
	ccall = (__temp189 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("parent_class")), "dispose"))), (ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL))), __temp189);
	ValaCCodeIdentifier* __temp190 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp190 = vala_ccode_identifier_new ("obj"))));
	(__temp190 == NULL ? NULL : (__temp190 = (g_object_unref (__temp190), NULL)));
	ValaCCodeExpressionStatement* __temp191 = NULL;
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE ((__temp191 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (ccall)))));
	(__temp191 == NULL ? NULL : (__temp191 = (g_object_unref (__temp191), NULL)));
	vala_ccode_function_set_block (self->priv->function, cblock);
	vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (self->priv->function));
	(cblock == NULL ? NULL : (cblock = (g_object_unref (cblock), NULL)));
	(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
	(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
	(fields == NULL ? NULL : (fields = (g_list_free (fields), NULL)));
	(ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL)));
}


static ValaCCodeIdentifier* vala_code_generator_get_value_setter_function (ValaCodeGenerator* self, ValaTypeReference* type_reference)
{
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (self), NULL);
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (type_reference), NULL);
	if (VALA_IS_CLASS (vala_type_reference_get_data_type (type_reference))) {
		return vala_ccode_identifier_new ("g_value_set_object");
	} else {
		if (vala_type_reference_get_data_type (type_reference) == vala_type_reference_get_data_type (self->priv->string_type)) {
			return vala_ccode_identifier_new ("g_value_set_string");
		} else {
			if (g_utf8_collate (vala_type_reference_get_type_name (type_reference), "int") == 0 || VALA_IS_ENUM (vala_type_reference_get_data_type (type_reference))) {
				return vala_ccode_identifier_new ("g_value_set_int");
			} else {
				if (vala_type_reference_get_data_type (type_reference) == vala_type_reference_get_data_type (self->priv->bool_type)) {
					return vala_ccode_identifier_new ("g_value_set_boolean");
				} else {
					return vala_ccode_identifier_new ("g_value_set_pointer");
				}
			}
		}
	}
}


static void vala_code_generator_add_get_property_function (ValaCodeGenerator* self, ValaClass* cl)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (VALA_IS_CLASS (cl));
	char* __temp197 = NULL;
	ValaCCodeFunction* __temp198 = NULL;
	ValaCCodeFunction* get_prop = (__temp198 = vala_ccode_function_new (g_strdup_printf ("%s_get_property", (__temp197 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL))), "void"), (__temp197 == NULL ? NULL : (__temp197 = (g_free (__temp197), NULL))), __temp198);
	ValaCCodeFormalParameter* __temp199 = NULL;
	vala_ccode_function_add_parameter (get_prop, (__temp199 = vala_ccode_formal_parameter_new ("object", "GObject *")));
	(__temp199 == NULL ? NULL : (__temp199 = (g_object_unref (__temp199), NULL)));
	ValaCCodeFormalParameter* __temp200 = NULL;
	vala_ccode_function_add_parameter (get_prop, (__temp200 = vala_ccode_formal_parameter_new ("property_id", "guint")));
	(__temp200 == NULL ? NULL : (__temp200 = (g_object_unref (__temp200), NULL)));
	ValaCCodeFormalParameter* __temp201 = NULL;
	vala_ccode_function_add_parameter (get_prop, (__temp201 = vala_ccode_formal_parameter_new ("value", "GValue *")));
	(__temp201 == NULL ? NULL : (__temp201 = (g_object_unref (__temp201), NULL)));
	ValaCCodeFormalParameter* __temp202 = NULL;
	vala_ccode_function_add_parameter (get_prop, (__temp202 = vala_ccode_formal_parameter_new ("pspec", "GParamSpec *")));
	(__temp202 == NULL ? NULL : (__temp202 = (g_object_unref (__temp202), NULL)));
	ValaCCodeBlock* block = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
	ValaCCodeFunctionCall* ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL))));
	ValaCCodeIdentifier* __temp203 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp203 = vala_ccode_identifier_new ("object"))));
	(__temp203 == NULL ? NULL : (__temp203 = (g_object_unref (__temp203), NULL)));
	ValaCCodeDeclaration* cdecl = vala_ccode_declaration_new (g_strdup_printf ("%s *", vala_data_type_get_cname (VALA_DATA_TYPE (cl))));
	ValaCCodeVariableDeclarator* __temp204 = NULL;
	vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp204 = vala_ccode_variable_declarator_new_with_initializer ("self", VALA_CCODE_EXPRESSION (ccall)))));
	(__temp204 == NULL ? NULL : (__temp204 = (g_object_unref (__temp204), NULL)));
	vala_ccode_block_add_statement (block, VALA_CCODE_NODE (cdecl));
	ValaCCodeSwitchStatement* cswitch = vala_ccode_switch_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("property_id")));
	GList* props = vala_class_get_properties (cl);
	{
		GList* prop_it;
		for (prop_it = props; prop_it != NULL; prop_it = prop_it->next) {
			ValaProperty* prop = prop_it->data;
			{
				if (vala_property_get_get_accessor (prop) == NULL) {
					continue;
				}
				ValaCCodeCaseStatement* ccase = vala_ccode_case_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_property_get_upper_case_cname (prop))));
				char* __temp205 = NULL;
				ValaCCodeFunctionCall* __temp206 = NULL;
				ValaCCodeFunctionCall* ccall = (__temp206 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (g_strdup_printf ("%s_get_%s", (__temp205 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL)), vala_property_get_name (prop))))), (__temp205 == NULL ? NULL : (__temp205 = (g_free (__temp205), NULL))), __temp206);
				ValaCCodeIdentifier* __temp207 = NULL;
				vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp207 = vala_ccode_identifier_new ("self"))));
				(__temp207 == NULL ? NULL : (__temp207 = (g_object_unref (__temp207), NULL)));
				ValaCCodeFunctionCall* csetcall = vala_ccode_function_call_new (NULL);
				ValaCCodeIdentifier* __temp208 = NULL;
				vala_ccode_function_call_set_call (csetcall, VALA_CCODE_EXPRESSION ((__temp208 = vala_code_generator_get_value_setter_function (self, vala_property_get_type_reference (prop)))));
				(__temp208 == NULL ? NULL : (__temp208 = (g_object_unref (__temp208), NULL)));
				ValaCCodeIdentifier* __temp209 = NULL;
				vala_ccode_function_call_add_argument (csetcall, VALA_CCODE_EXPRESSION ((__temp209 = vala_ccode_identifier_new ("value"))));
				(__temp209 == NULL ? NULL : (__temp209 = (g_object_unref (__temp209), NULL)));
				vala_ccode_function_call_add_argument (csetcall, VALA_CCODE_EXPRESSION (ccall));
				ValaCCodeExpressionStatement* __temp210 = NULL;
				vala_ccode_case_statement_add_statement (ccase, VALA_CCODE_STATEMENT ((__temp210 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (csetcall)))));
				(__temp210 == NULL ? NULL : (__temp210 = (g_object_unref (__temp210), NULL)));
				ValaCCodeBreakStatement* __temp211 = NULL;
				vala_ccode_case_statement_add_statement (ccase, VALA_CCODE_STATEMENT ((__temp211 = g_object_new (VALA_TYPE_CCODE_BREAK_STATEMENT, NULL))));
				(__temp211 == NULL ? NULL : (__temp211 = (g_object_unref (__temp211), NULL)));
				vala_ccode_switch_statement_add_case (cswitch, ccase);
				(ccase == NULL ? NULL : (ccase = (g_object_unref (ccase), NULL)));
				(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
				(csetcall == NULL ? NULL : (csetcall = (g_object_unref (csetcall), NULL)));
			}
		}
	}
	vala_ccode_block_add_statement (block, VALA_CCODE_NODE (cswitch));
	vala_ccode_function_set_block (get_prop, block);
	vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (get_prop));
	(get_prop == NULL ? NULL : (get_prop = (g_object_unref (get_prop), NULL)));
	(block == NULL ? NULL : (block = (g_object_unref (block), NULL)));
	(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
	(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
	(cswitch == NULL ? NULL : (cswitch = (g_object_unref (cswitch), NULL)));
	(props == NULL ? NULL : (props = (g_list_free (props), NULL)));
}


static void vala_code_generator_add_set_property_function (ValaCodeGenerator* self, ValaClass* cl)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (VALA_IS_CLASS (cl));
	char* __temp212 = NULL;
	ValaCCodeFunction* __temp213 = NULL;
	ValaCCodeFunction* set_prop = (__temp213 = vala_ccode_function_new (g_strdup_printf ("%s_set_property", (__temp212 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL))), "void"), (__temp212 == NULL ? NULL : (__temp212 = (g_free (__temp212), NULL))), __temp213);
	ValaCCodeFormalParameter* __temp214 = NULL;
	vala_ccode_function_add_parameter (set_prop, (__temp214 = vala_ccode_formal_parameter_new ("object", "GObject *")));
	(__temp214 == NULL ? NULL : (__temp214 = (g_object_unref (__temp214), NULL)));
	ValaCCodeFormalParameter* __temp215 = NULL;
	vala_ccode_function_add_parameter (set_prop, (__temp215 = vala_ccode_formal_parameter_new ("property_id", "guint")));
	(__temp215 == NULL ? NULL : (__temp215 = (g_object_unref (__temp215), NULL)));
	ValaCCodeFormalParameter* __temp216 = NULL;
	vala_ccode_function_add_parameter (set_prop, (__temp216 = vala_ccode_formal_parameter_new ("value", "const GValue *")));
	(__temp216 == NULL ? NULL : (__temp216 = (g_object_unref (__temp216), NULL)));
	ValaCCodeFormalParameter* __temp217 = NULL;
	vala_ccode_function_add_parameter (set_prop, (__temp217 = vala_ccode_formal_parameter_new ("pspec", "GParamSpec *")));
	(__temp217 == NULL ? NULL : (__temp217 = (g_object_unref (__temp217), NULL)));
	ValaCCodeBlock* block = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
	ValaCCodeFunctionCall* ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL))));
	ValaCCodeIdentifier* __temp218 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp218 = vala_ccode_identifier_new ("object"))));
	(__temp218 == NULL ? NULL : (__temp218 = (g_object_unref (__temp218), NULL)));
	ValaCCodeDeclaration* cdecl = vala_ccode_declaration_new (g_strdup_printf ("%s *", vala_data_type_get_cname (VALA_DATA_TYPE (cl))));
	ValaCCodeVariableDeclarator* __temp219 = NULL;
	vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp219 = vala_ccode_variable_declarator_new_with_initializer ("self", VALA_CCODE_EXPRESSION (ccall)))));
	(__temp219 == NULL ? NULL : (__temp219 = (g_object_unref (__temp219), NULL)));
	vala_ccode_block_add_statement (block, VALA_CCODE_NODE (cdecl));
	ValaCCodeSwitchStatement* cswitch = vala_ccode_switch_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("property_id")));
	GList* props = vala_class_get_properties (cl);
	{
		GList* prop_it;
		for (prop_it = props; prop_it != NULL; prop_it = prop_it->next) {
			ValaProperty* prop = prop_it->data;
			{
				if (vala_property_get_set_accessor (prop) == NULL) {
					continue;
				}
				ValaCCodeCaseStatement* ccase = vala_ccode_case_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_property_get_upper_case_cname (prop))));
				char* __temp220 = NULL;
				ValaCCodeFunctionCall* __temp221 = NULL;
				ValaCCodeFunctionCall* ccall = (__temp221 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (g_strdup_printf ("%s_set_%s", (__temp220 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL)), vala_property_get_name (prop))))), (__temp220 == NULL ? NULL : (__temp220 = (g_free (__temp220), NULL))), __temp221);
				ValaCCodeIdentifier* __temp222 = NULL;
				vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp222 = vala_ccode_identifier_new ("self"))));
				(__temp222 == NULL ? NULL : (__temp222 = (g_object_unref (__temp222), NULL)));
				ValaCCodeFunctionCall* cgetcall = vala_ccode_function_call_new (NULL);
				if (VALA_IS_CLASS (vala_type_reference_get_data_type (vala_property_get_type_reference (prop)))) {
					ValaCCodeIdentifier* __temp223 = NULL;
					vala_ccode_function_call_set_call (cgetcall, VALA_CCODE_EXPRESSION ((__temp223 = vala_ccode_identifier_new ("g_value_get_object"))));
					(__temp223 == NULL ? NULL : (__temp223 = (g_object_unref (__temp223), NULL)));
				} else {
					if (g_utf8_collate (vala_type_reference_get_type_name (vala_property_get_type_reference (prop)), "string") == 0) {
						ValaCCodeIdentifier* __temp224 = NULL;
						vala_ccode_function_call_set_call (cgetcall, VALA_CCODE_EXPRESSION ((__temp224 = vala_ccode_identifier_new ("g_value_get_string"))));
						(__temp224 == NULL ? NULL : (__temp224 = (g_object_unref (__temp224), NULL)));
					} else {
						if (g_utf8_collate (vala_type_reference_get_type_name (vala_property_get_type_reference (prop)), "int") == 0 || VALA_IS_ENUM (vala_type_reference_get_data_type (vala_property_get_type_reference (prop)))) {
							ValaCCodeIdentifier* __temp225 = NULL;
							vala_ccode_function_call_set_call (cgetcall, VALA_CCODE_EXPRESSION ((__temp225 = vala_ccode_identifier_new ("g_value_get_int"))));
							(__temp225 == NULL ? NULL : (__temp225 = (g_object_unref (__temp225), NULL)));
						} else {
							if (g_utf8_collate (vala_type_reference_get_type_name (vala_property_get_type_reference (prop)), "bool") == 0) {
								ValaCCodeIdentifier* __temp226 = NULL;
								vala_ccode_function_call_set_call (cgetcall, VALA_CCODE_EXPRESSION ((__temp226 = vala_ccode_identifier_new ("g_value_get_boolean"))));
								(__temp226 == NULL ? NULL : (__temp226 = (g_object_unref (__temp226), NULL)));
							} else {
								ValaCCodeIdentifier* __temp227 = NULL;
								vala_ccode_function_call_set_call (cgetcall, VALA_CCODE_EXPRESSION ((__temp227 = vala_ccode_identifier_new ("g_value_get_pointer"))));
								(__temp227 == NULL ? NULL : (__temp227 = (g_object_unref (__temp227), NULL)));
							}
						}
					}
				}
				ValaCCodeIdentifier* __temp228 = NULL;
				vala_ccode_function_call_add_argument (cgetcall, VALA_CCODE_EXPRESSION ((__temp228 = vala_ccode_identifier_new ("value"))));
				(__temp228 == NULL ? NULL : (__temp228 = (g_object_unref (__temp228), NULL)));
				vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION (cgetcall));
				ValaCCodeExpressionStatement* __temp229 = NULL;
				vala_ccode_case_statement_add_statement (ccase, VALA_CCODE_STATEMENT ((__temp229 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (ccall)))));
				(__temp229 == NULL ? NULL : (__temp229 = (g_object_unref (__temp229), NULL)));
				ValaCCodeBreakStatement* __temp230 = NULL;
				vala_ccode_case_statement_add_statement (ccase, VALA_CCODE_STATEMENT ((__temp230 = g_object_new (VALA_TYPE_CCODE_BREAK_STATEMENT, NULL))));
				(__temp230 == NULL ? NULL : (__temp230 = (g_object_unref (__temp230), NULL)));
				vala_ccode_switch_statement_add_case (cswitch, ccase);
				(ccase == NULL ? NULL : (ccase = (g_object_unref (ccase), NULL)));
				(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
				(cgetcall == NULL ? NULL : (cgetcall = (g_object_unref (cgetcall), NULL)));
			}
		}
	}
	vala_ccode_block_add_statement (block, VALA_CCODE_NODE (cswitch));
	vala_ccode_function_set_block (set_prop, block);
	vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (set_prop));
	(set_prop == NULL ? NULL : (set_prop = (g_object_unref (set_prop), NULL)));
	(block == NULL ? NULL : (block = (g_object_unref (block), NULL)));
	(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
	(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
	(cswitch == NULL ? NULL : (cswitch = (g_object_unref (cswitch), NULL)));
	(props == NULL ? NULL : (props = (g_list_free (props), NULL)));
}


static void vala_code_generator_real_visit_begin_struct (ValaCodeVisitor* base, ValaStruct* st)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_STRUCT (st));
	ValaCCodeStruct* __temp231 = NULL;
	self->priv->instance_struct = (__temp231 = vala_ccode_struct_new (g_strdup_printf ("_%s", vala_data_type_get_name (VALA_DATA_TYPE (st)))), (self->priv->instance_struct == NULL ? NULL : (self->priv->instance_struct = (g_object_unref (self->priv->instance_struct), NULL))), __temp231);
	if (vala_source_reference_get_comment (vala_code_node_get_source_reference (VALA_CODE_NODE (st))) != NULL) {
		ValaCCodeComment* __temp232 = NULL;
		vala_ccode_fragment_append (self->priv->header_type_definition, VALA_CCODE_NODE ((__temp232 = vala_ccode_comment_new (vala_source_reference_get_comment (vala_code_node_get_source_reference (VALA_CODE_NODE (st)))))));
		(__temp232 == NULL ? NULL : (__temp232 = (g_object_unref (__temp232), NULL)));
	}
	vala_ccode_fragment_append (self->priv->header_type_definition, VALA_CCODE_NODE (self->priv->instance_struct));
}


static void vala_code_generator_real_visit_begin_interface (ValaCodeVisitor* base, ValaInterface* iface)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_INTERFACE (iface));
	ValaSymbol* __temp234 = NULL;
	ValaSymbol* __temp233 = NULL;
	self->priv->current_symbol = (__temp234 = (__temp233 = vala_code_node_get_symbol (VALA_CODE_NODE (iface)), (__temp233 == NULL ? __temp233 : g_object_ref (__temp233))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp234);
	ValaSymbol* __temp236 = NULL;
	ValaSymbol* __temp235 = NULL;
	self->priv->current_type_symbol = (__temp236 = (__temp235 = vala_code_node_get_symbol (VALA_CODE_NODE (iface)), (__temp235 == NULL ? __temp235 : g_object_ref (__temp235))), (self->priv->current_type_symbol == NULL ? NULL : (self->priv->current_type_symbol = (g_object_unref (self->priv->current_type_symbol), NULL))), __temp236);
	ValaCCodeStruct* __temp237 = NULL;
	self->priv->type_struct = (__temp237 = vala_ccode_struct_new (g_strdup_printf ("_%sInterface", vala_data_type_get_cname (VALA_DATA_TYPE (iface)))), (self->priv->type_struct == NULL ? NULL : (self->priv->type_struct = (g_object_unref (self->priv->type_struct), NULL))), __temp237);
	ValaCCodeNewline* __temp238 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp238 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp238 == NULL ? NULL : (__temp238 = (g_object_unref (__temp238), NULL)));
	char* __temp239 = NULL;
	char* __temp240 = NULL;
	char* macro = (__temp240 = g_strdup_printf ("(%s_get_type ())", (__temp239 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (iface), NULL))), (__temp239 == NULL ? NULL : (__temp239 = (g_free (__temp239), NULL))), __temp240);
	ValaCCodeMacroReplacement* __temp241 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp241 = vala_ccode_macro_replacement_new (vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (iface), "TYPE_"), macro))));
	(__temp241 == NULL ? NULL : (__temp241 = (g_object_unref (__temp241), NULL)));
	char* __temp243 = NULL;
	char* __temp242 = NULL;
	macro = (__temp243 = g_strdup_printf ("(G_TYPE_CHECK_INSTANCE_CAST ((obj), %s, %s))", (__temp242 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (iface), "TYPE_")), vala_data_type_get_cname (VALA_DATA_TYPE (iface))), (macro == NULL ? NULL : (macro = (g_free (macro), NULL))), __temp243);
	(__temp242 == NULL ? NULL : (__temp242 = (g_free (__temp242), NULL)));
	ValaCCodeMacroReplacement* __temp245 = NULL;
	char* __temp244 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp245 = vala_ccode_macro_replacement_new (g_strdup_printf ("%s(obj)", (__temp244 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (iface), NULL))), macro))));
	(__temp245 == NULL ? NULL : (__temp245 = (g_object_unref (__temp245), NULL)));
	(__temp244 == NULL ? NULL : (__temp244 = (g_free (__temp244), NULL)));
	char* __temp247 = NULL;
	char* __temp246 = NULL;
	macro = (__temp247 = g_strdup_printf ("(G_TYPE_CHECK_INSTANCE_TYPE ((obj), %s))", (__temp246 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (iface), "TYPE_"))), (macro == NULL ? NULL : (macro = (g_free (macro), NULL))), __temp247);
	(__temp246 == NULL ? NULL : (__temp246 = (g_free (__temp246), NULL)));
	ValaCCodeMacroReplacement* __temp249 = NULL;
	char* __temp248 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp249 = vala_ccode_macro_replacement_new (g_strdup_printf ("%s(obj)", (__temp248 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (iface), "IS_"))), macro))));
	(__temp249 == NULL ? NULL : (__temp249 = (g_object_unref (__temp249), NULL)));
	(__temp248 == NULL ? NULL : (__temp248 = (g_free (__temp248), NULL)));
	char* __temp251 = NULL;
	char* __temp250 = NULL;
	macro = (__temp251 = g_strdup_printf ("(G_TYPE_INSTANCE_GET_INTERFACE ((obj), %s, %sInterface))", (__temp250 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (iface), "TYPE_")), vala_data_type_get_cname (VALA_DATA_TYPE (iface))), (macro == NULL ? NULL : (macro = (g_free (macro), NULL))), __temp251);
	(__temp250 == NULL ? NULL : (__temp250 = (g_free (__temp250), NULL)));
	ValaCCodeMacroReplacement* __temp253 = NULL;
	char* __temp252 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp253 = vala_ccode_macro_replacement_new (g_strdup_printf ("%s_GET_INTERFACE(obj)", (__temp252 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (iface), NULL))), macro))));
	(__temp253 == NULL ? NULL : (__temp253 = (g_object_unref (__temp253), NULL)));
	(__temp252 == NULL ? NULL : (__temp252 = (g_free (__temp252), NULL)));
	ValaCCodeNewline* __temp254 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp254 = g_object_new (VALA_TYPE_CCODE_NEWLINE, NULL))));
	(__temp254 == NULL ? NULL : (__temp254 = (g_object_unref (__temp254), NULL)));
	if (vala_source_file_get_cycle (vala_source_reference_get_file (vala_code_node_get_source_reference (VALA_CODE_NODE (iface)))) == NULL) {
		ValaCCodeTypeDefinition* __temp255 = NULL;
		vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp255 = vala_ccode_type_definition_new (g_strdup_printf ("struct _%s", vala_data_type_get_cname (VALA_DATA_TYPE (iface))), VALA_CCODE_DECLARATOR (vala_ccode_variable_declarator_new (vala_data_type_get_cname (VALA_DATA_TYPE (iface))))))));
		(__temp255 == NULL ? NULL : (__temp255 = (g_object_unref (__temp255), NULL)));
		ValaCCodeTypeDefinition* __temp256 = NULL;
		vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE ((__temp256 = vala_ccode_type_definition_new (g_strdup_printf ("struct %s", vala_ccode_struct_get_name (self->priv->type_struct)), VALA_CCODE_DECLARATOR (vala_ccode_variable_declarator_new (g_strdup_printf ("%sInterface", vala_data_type_get_cname (VALA_DATA_TYPE (iface)))))))));
		(__temp256 == NULL ? NULL : (__temp256 = (g_object_unref (__temp256), NULL)));
	}
	vala_ccode_struct_add_field (self->priv->type_struct, "GTypeInterface", "parent");
	if (vala_source_reference_get_comment (vala_code_node_get_source_reference (VALA_CODE_NODE (iface))) != NULL) {
		ValaCCodeComment* __temp257 = NULL;
		vala_ccode_fragment_append (self->priv->header_type_definition, VALA_CCODE_NODE ((__temp257 = vala_ccode_comment_new (vala_source_reference_get_comment (vala_code_node_get_source_reference (VALA_CODE_NODE (iface)))))));
		(__temp257 == NULL ? NULL : (__temp257 = (g_object_unref (__temp257), NULL)));
	}
	vala_ccode_fragment_append (self->priv->header_type_definition, VALA_CCODE_NODE (self->priv->type_struct));
	(macro == NULL ? NULL : (macro = (g_free (macro), NULL)));
}


static void vala_code_generator_real_visit_end_interface (ValaCodeVisitor* base, ValaInterface* iface)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_INTERFACE (iface));
	ValaInterfaceRegisterFunction* type_fun = vala_interface_register_function_new (iface);
	vala_type_register_function_init_from_type (VALA_TYPE_REGISTER_FUNCTION (type_fun));
	ValaCCodeFunction* __temp258 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_member_declaration, VALA_CCODE_NODE ((__temp258 = vala_type_register_function_get_declaration (VALA_TYPE_REGISTER_FUNCTION (type_fun)))));
	(__temp258 == NULL ? NULL : (__temp258 = (g_object_unref (__temp258), NULL)));
	vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (type_fun));
	ValaSymbol* __temp259 = NULL;
	self->priv->current_type_symbol = (__temp259 = NULL, (self->priv->current_type_symbol == NULL ? NULL : (self->priv->current_type_symbol = (g_object_unref (self->priv->current_type_symbol), NULL))), __temp259);
	(type_fun == NULL ? NULL : (type_fun = (g_object_unref (type_fun), NULL)));
}


static void vala_code_generator_real_visit_begin_enum (ValaCodeVisitor* base, ValaEnum* en)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_ENUM (en));
	ValaCCodeEnum* __temp260 = NULL;
	self->priv->cenum = (__temp260 = vala_ccode_enum_new (vala_data_type_get_cname (VALA_DATA_TYPE (en))), (self->priv->cenum == NULL ? NULL : (self->priv->cenum = (g_object_unref (self->priv->cenum), NULL))), __temp260);
	if (vala_source_reference_get_comment (vala_code_node_get_source_reference (VALA_CODE_NODE (en))) != NULL) {
		ValaCCodeComment* __temp261 = NULL;
		vala_ccode_fragment_append (self->priv->header_type_definition, VALA_CCODE_NODE ((__temp261 = vala_ccode_comment_new (vala_source_reference_get_comment (vala_code_node_get_source_reference (VALA_CODE_NODE (en)))))));
		(__temp261 == NULL ? NULL : (__temp261 = (g_object_unref (__temp261), NULL)));
	}
	vala_ccode_fragment_append (self->priv->header_type_definition, VALA_CCODE_NODE (self->priv->cenum));
}


static void vala_code_generator_real_visit_enum_value (ValaCodeVisitor* base, ValaEnumValue* ev)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_ENUM_VALUE (ev));
	vala_ccode_enum_add_value (self->priv->cenum, vala_enum_value_get_cname (ev), NULL);
}


static void vala_code_generator_real_visit_end_callback (ValaCodeVisitor* base, ValaCallback* cb)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_CALLBACK (cb));
	ValaCCodeFunctionDeclarator* cfundecl = vala_ccode_function_declarator_new (vala_data_type_get_cname (VALA_DATA_TYPE (cb)));
	{
		GList* param_it;
		for (param_it = vala_callback_get_parameters (cb); param_it != NULL; param_it = param_it->next) {
			ValaFormalParameter* param = param_it->data;
			{
				vala_ccode_function_declarator_add_parameter (cfundecl, VALA_CCODE_FORMAL_PARAMETER (vala_code_node_get_ccodenode (VALA_CODE_NODE (param))));
			}
		}
	}
	ValaCCodeTypeDefinition* ctypedef = vala_ccode_type_definition_new (vala_type_reference_get_cname (vala_callback_get_return_type (cb), FALSE), VALA_CCODE_DECLARATOR (cfundecl));
	if (VALA_DATA_TYPE (cb)->access == VALA_MEMBER_ACCESSIBILITY_PUBLIC) {
		vala_ccode_fragment_append (self->priv->header_type_declaration, VALA_CCODE_NODE (ctypedef));
	} else {
		vala_ccode_fragment_append (self->priv->source_type_member_declaration, VALA_CCODE_NODE (ctypedef));
	}
	(cfundecl == NULL ? NULL : (cfundecl = (g_object_unref (cfundecl), NULL)));
	(ctypedef == NULL ? NULL : (ctypedef = (g_object_unref (ctypedef), NULL)));
}


static void vala_code_generator_real_visit_constant (ValaCodeVisitor* base, ValaConstant* c)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_CONSTANT (c));
	if (VALA_IS_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (c)))))) {
		ValaDataType* t = VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (c)))));
		ValaCCodeDeclaration* cdecl = vala_ccode_declaration_new (vala_type_reference_get_const_cname (vala_constant_get_type_reference (c)));
		char* arr = "";
		if (VALA_IS_ARRAY (vala_type_reference_get_data_type (vala_constant_get_type_reference (c)))) {
			arr = "[]";
		}
		ValaCCodeVariableDeclarator* __temp262 = NULL;
		vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp262 = vala_ccode_variable_declarator_new_with_initializer (g_strdup_printf ("%s%s", vala_constant_get_cname (c), arr), VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_constant_get_initializer (c))))))));
		(__temp262 == NULL ? NULL : (__temp262 = (g_object_unref (__temp262), NULL)));
		vala_ccode_declaration_set_modifiers (cdecl, VALA_CCODE_MODIFIERS_STATIC);
		vala_ccode_fragment_append (self->priv->source_type_member_declaration, VALA_CCODE_NODE (cdecl));
		(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
	}
}


static void vala_code_generator_real_visit_field (ValaCodeVisitor* base, ValaField* f)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_FIELD (f));
	if (f->access == VALA_MEMBER_ACCESSIBILITY_PUBLIC) {
		char* __temp263 = NULL;
		vala_ccode_struct_add_field (self->priv->instance_struct, (__temp263 = vala_type_reference_get_cname (vala_field_get_type_reference (f), FALSE)), vala_field_get_cname (f));
		(__temp263 == NULL ? NULL : (__temp263 = (g_free (__temp263), NULL)));
	} else {
		if (f->access == VALA_MEMBER_ACCESSIBILITY_PRIVATE) {
			if (vala_field_get_instance (f)) {
				char* __temp264 = NULL;
				vala_ccode_struct_add_field (self->priv->instance_priv_struct, (__temp264 = vala_type_reference_get_cname (vala_field_get_type_reference (f), FALSE)), vala_field_get_cname (f));
				(__temp264 == NULL ? NULL : (__temp264 = (g_free (__temp264), NULL)));
			} else {
				if (VALA_IS_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (f)))))) {
					ValaDataType* t = VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (f)))));
					ValaCCodeDeclaration* cdecl = vala_ccode_declaration_new (vala_type_reference_get_cname (vala_field_get_type_reference (f), FALSE));
					char* __temp265 = NULL;
					ValaCCodeVariableDeclarator* __temp266 = NULL;
					ValaCCodeVariableDeclarator* var_decl = (__temp266 = vala_ccode_variable_declarator_new (g_strdup_printf ("%s_%s", (__temp265 = vala_data_type_get_lower_case_cname (t, NULL)), vala_field_get_cname (f))), (__temp265 == NULL ? NULL : (__temp265 = (g_free (__temp265), NULL))), __temp266);
					if (vala_field_get_initializer (f) != NULL) {
						vala_ccode_variable_declarator_set_initializer (var_decl, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_field_get_initializer (f)))));
					}
					vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR (var_decl));
					vala_ccode_declaration_set_modifiers (cdecl, VALA_CCODE_MODIFIERS_STATIC);
					vala_ccode_fragment_append (self->priv->source_type_member_declaration, VALA_CCODE_NODE (cdecl));
					(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
					(var_decl == NULL ? NULL : (var_decl = (g_object_unref (var_decl), NULL)));
				}
			}
		}
	}
}


static void vala_code_generator_real_visit_begin_method (ValaCodeVisitor* base, ValaMethod* m)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_METHOD (m));
	ValaSymbol* __temp268 = NULL;
	ValaSymbol* __temp267 = NULL;
	self->priv->current_symbol = (__temp268 = (__temp267 = vala_code_node_get_symbol (VALA_CODE_NODE (m)), (__temp267 == NULL ? __temp267 : g_object_ref (__temp267))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp268);
}


static ValaCCodeStatement* vala_code_generator_create_method_type_check_statement (ValaCodeGenerator* self, ValaMethod* m, ValaDataType* t, gboolean non_null, char* var_name)
{
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (self), NULL);
	g_return_val_if_fail (VALA_IS_METHOD (m), NULL);
	g_return_val_if_fail (VALA_IS_DATA_TYPE (t), NULL);
	g_return_val_if_fail (var_name != NULL, NULL);
	return vala_code_generator_create_type_check_statement (self, VALA_CODE_NODE (m), vala_type_reference_get_data_type (vala_method_get_return_type (m)), t, non_null, var_name);
}


static ValaCCodeStatement* vala_code_generator_create_property_type_check_statement (ValaCodeGenerator* self, ValaProperty* prop, gboolean getter, ValaDataType* t, gboolean non_null, char* var_name)
{
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (self), NULL);
	g_return_val_if_fail (VALA_IS_PROPERTY (prop), NULL);
	g_return_val_if_fail (VALA_IS_DATA_TYPE (t), NULL);
	g_return_val_if_fail (var_name != NULL, NULL);
	if (getter) {
		return vala_code_generator_create_type_check_statement (self, VALA_CODE_NODE (prop), vala_type_reference_get_data_type (vala_property_get_type_reference (prop)), t, non_null, var_name);
	} else {
		return vala_code_generator_create_type_check_statement (self, VALA_CODE_NODE (prop), NULL, t, non_null, var_name);
	}
}


static ValaCCodeStatement* vala_code_generator_create_type_check_statement (ValaCodeGenerator* self, ValaCodeNode* method_node, ValaDataType* ret_type, ValaDataType* t, gboolean non_null, char* var_name)
{
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (self), NULL);
	g_return_val_if_fail (VALA_IS_CODE_NODE (method_node), NULL);
	g_return_val_if_fail (ret_type == NULL || VALA_IS_DATA_TYPE (ret_type), NULL);
	g_return_val_if_fail (VALA_IS_DATA_TYPE (t), NULL);
	g_return_val_if_fail (var_name != NULL, NULL);
	ValaCCodeFunctionCall* ccheck = vala_ccode_function_call_new (NULL);
	if (VALA_IS_CLASS (t) || VALA_IS_INTERFACE (t)) {
		ValaCCodeFunctionCall* ctype_check = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_data_type_get_upper_case_cname (t, "IS_"))));
		ValaCCodeIdentifier* __temp272 = NULL;
		vala_ccode_function_call_add_argument (ctype_check, VALA_CCODE_EXPRESSION ((__temp272 = vala_ccode_identifier_new (var_name))));
		(__temp272 == NULL ? NULL : (__temp272 = (g_object_unref (__temp272), NULL)));
		ValaCCodeFunctionCall* __temp273 = NULL;
		ValaCCodeExpression* cexpr = VALA_CCODE_EXPRESSION ((__temp273 = ctype_check, (__temp273 == NULL ? __temp273 : g_object_ref (__temp273))));
		if (!non_null) {
			ValaCCodeBinaryExpression* cnull = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_EQUALITY, VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (var_name)), VALA_CCODE_EXPRESSION (vala_ccode_constant_new ("NULL")));
			ValaCCodeExpression* __temp274 = NULL;
			cexpr = (__temp274 = VALA_CCODE_EXPRESSION (vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_OR, VALA_CCODE_EXPRESSION (cnull), VALA_CCODE_EXPRESSION (ctype_check))), (cexpr == NULL ? NULL : (cexpr = (g_object_unref (cexpr), NULL))), __temp274);
			(cnull == NULL ? NULL : (cnull = (g_object_unref (cnull), NULL)));
		}
		vala_ccode_function_call_add_argument (ccheck, cexpr);
		(ctype_check == NULL ? NULL : (ctype_check = (g_object_unref (ctype_check), NULL)));
		(cexpr == NULL ? NULL : (cexpr = (g_object_unref (cexpr), NULL)));
	} else {
		if (!non_null) {
			gpointer __temp275;
			return (__temp275 = NULL, (ccheck == NULL ? NULL : (ccheck = (g_object_unref (ccheck), NULL))), __temp275);
		} else {
			ValaCCodeBinaryExpression* cnonnull = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_INEQUALITY, VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (var_name)), VALA_CCODE_EXPRESSION (vala_ccode_constant_new ("NULL")));
			vala_ccode_function_call_add_argument (ccheck, VALA_CCODE_EXPRESSION (cnonnull));
			(cnonnull == NULL ? NULL : (cnonnull = (g_object_unref (cnonnull), NULL)));
		}
	}
	if (ret_type == NULL) {
		/* void function */
		ValaCCodeIdentifier* __temp276 = NULL;
		vala_ccode_function_call_set_call (ccheck, VALA_CCODE_EXPRESSION ((__temp276 = vala_ccode_identifier_new ("g_return_if_fail"))));
		(__temp276 == NULL ? NULL : (__temp276 = (g_object_unref (__temp276), NULL)));
	} else {
		ValaCCodeIdentifier* __temp277 = NULL;
		vala_ccode_function_call_set_call (ccheck, VALA_CCODE_EXPRESSION ((__temp277 = vala_ccode_identifier_new ("g_return_val_if_fail"))));
		(__temp277 == NULL ? NULL : (__temp277 = (g_object_unref (__temp277), NULL)));
		if (vala_data_type_is_reference_type (ret_type)) {
			ValaCCodeConstant* __temp278 = NULL;
			vala_ccode_function_call_add_argument (ccheck, VALA_CCODE_EXPRESSION ((__temp278 = vala_ccode_constant_new ("NULL"))));
			(__temp278 == NULL ? NULL : (__temp278 = (g_object_unref (__temp278), NULL)));
		} else {
			if (g_utf8_collate (vala_data_type_get_name (ret_type), "bool") == 0) {
				ValaCCodeConstant* __temp279 = NULL;
				vala_ccode_function_call_add_argument (ccheck, VALA_CCODE_EXPRESSION ((__temp279 = vala_ccode_constant_new ("FALSE"))));
				(__temp279 == NULL ? NULL : (__temp279 = (g_object_unref (__temp279), NULL)));
			} else {
				if (g_utf8_collate (vala_data_type_get_name (ret_type), "int") == 0 || g_utf8_collate (vala_data_type_get_name (ret_type), "long") == 0 || g_utf8_collate (vala_data_type_get_name (ret_type), "double") == 0 || g_utf8_collate (vala_data_type_get_name (ret_type), "float") == 0 || VALA_IS_ENUM (ret_type) || VALA_IS_FLAGS (ret_type)) {
					ValaCCodeConstant* __temp280 = NULL;
					vala_ccode_function_call_add_argument (ccheck, VALA_CCODE_EXPRESSION ((__temp280 = vala_ccode_constant_new ("0"))));
					(__temp280 == NULL ? NULL : (__temp280 = (g_object_unref (__temp280), NULL)));
				} else {
					vala_report_error (vala_code_node_get_source_reference (method_node), "not supported return type for runtime type checks");
					gpointer __temp281;
					return (__temp281 = NULL, (ccheck == NULL ? NULL : (ccheck = (g_object_unref (ccheck), NULL))), __temp281);
				}
			}
		}
	}
	ValaCCodeExpressionStatement* __temp282 = NULL;
	return (__temp282 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (ccheck)), (ccheck == NULL ? NULL : (ccheck = (g_object_unref (ccheck), NULL))), __temp282);
	(ccheck == NULL ? NULL : (ccheck = (g_object_unref (ccheck), NULL)));
}


static ValaDataType* vala_code_generator_find_parent_type (ValaCodeGenerator* self, ValaCodeNode* node)
{
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (self), NULL);
	g_return_val_if_fail (node == NULL || VALA_IS_CODE_NODE (node), NULL);
	ValaSymbol* sym = vala_code_node_get_symbol (node);
	while (sym != NULL) {
		if (VALA_IS_DATA_TYPE (vala_symbol_get_node (sym))) {
			return VALA_DATA_TYPE (vala_symbol_get_node (sym));
		}
		sym = vala_symbol_get_parent_symbol (sym);
	}
	return NULL;
}


static void vala_code_generator_real_visit_end_method (ValaCodeVisitor* base, ValaMethod* m)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_METHOD (m));
	ValaSymbol* __temp286 = NULL;
	ValaSymbol* __temp285 = NULL;
	self->priv->current_symbol = (__temp286 = (__temp285 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp285 == NULL ? __temp285 : g_object_ref (__temp285))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp286);
	ValaCCodeFunction* __temp287 = NULL;
	self->priv->function = (__temp287 = vala_ccode_function_new (vala_method_get_real_cname (m), vala_type_reference_get_cname (vala_method_get_return_type (m), FALSE)), (self->priv->function == NULL ? NULL : (self->priv->function = (g_object_unref (self->priv->function), NULL))), __temp287);
	ValaCCodeFunctionDeclarator* vdeclarator = NULL;
	ValaCCodeFormalParameter* instance_param = NULL;
	if (vala_method_get_instance (m)) {
		ValaTypeReference* this_type = vala_type_reference_new ();
		vala_type_reference_set_data_type (this_type, vala_code_generator_find_parent_type (self, VALA_CODE_NODE (m)));
		if (!vala_method_get_overrides (m)) {
			ValaCCodeFormalParameter* __temp288 = NULL;
			instance_param = (__temp288 = vala_ccode_formal_parameter_new ("self", vala_type_reference_get_cname (this_type, FALSE)), (instance_param == NULL ? NULL : (instance_param = (g_object_unref (instance_param), NULL))), __temp288);
		} else {
			ValaTypeReference* base_type = vala_type_reference_new ();
			vala_type_reference_set_data_type (base_type, VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (vala_method_get_base_method (m)))))));
			ValaCCodeFormalParameter* __temp289 = NULL;
			instance_param = (__temp289 = vala_ccode_formal_parameter_new ("base", vala_type_reference_get_cname (base_type, FALSE)), (instance_param == NULL ? NULL : (instance_param = (g_object_unref (instance_param), NULL))), __temp289);
			(base_type == NULL ? NULL : (base_type = (g_object_unref (base_type), NULL)));
		}
		if (!vala_method_get_instance_last (m)) {
			vala_ccode_function_add_parameter (self->priv->function, instance_param);
		}
		if (vala_method_get_is_abstract (m) || vala_method_get_is_virtual (m)) {
			ValaCCodeDeclaration* vdecl = vala_ccode_declaration_new (vala_type_reference_get_cname (vala_method_get_return_type (m), FALSE));
			ValaCCodeFunctionDeclarator* __temp290 = NULL;
			vdeclarator = (__temp290 = vala_ccode_function_declarator_new (vala_method_get_name (m)), (vdeclarator == NULL ? NULL : (vdeclarator = (g_object_unref (vdeclarator), NULL))), __temp290);
			vala_ccode_declaration_add_declarator (vdecl, VALA_CCODE_DECLARATOR (vdeclarator));
			vala_ccode_struct_add_declaration (self->priv->type_struct, vdecl);
			vala_ccode_function_declarator_add_parameter (vdeclarator, instance_param);
			(vdecl == NULL ? NULL : (vdecl = (g_object_unref (vdecl), NULL)));
		}
		(this_type == NULL ? NULL : (this_type = (g_object_unref (this_type), NULL)));
	}
	GList* params = vala_method_get_parameters (m);
	{
		GList* param_it;
		for (param_it = params; param_it != NULL; param_it = param_it->next) {
			ValaFormalParameter* param = param_it->data;
			{
				vala_ccode_function_add_parameter (self->priv->function, VALA_CCODE_FORMAL_PARAMETER (vala_code_node_get_ccodenode (VALA_CODE_NODE (param))));
				if (vdeclarator != NULL) {
					vala_ccode_function_declarator_add_parameter (vdeclarator, VALA_CCODE_FORMAL_PARAMETER (vala_code_node_get_ccodenode (VALA_CODE_NODE (param))));
				}
			}
		}
	}
	if (vala_method_get_instance (m) && vala_method_get_instance_last (m)) {
		vala_ccode_function_add_parameter (self->priv->function, instance_param);
	}
	/* real function declaration and definition not needed
	 * for abstract methods */
	if (!vala_method_get_is_abstract (m)) {
		if (m->access == VALA_MEMBER_ACCESSIBILITY_PUBLIC && !(vala_method_get_is_virtual (m) || vala_method_get_overrides (m))) {
			/* public methods need function declaration in
			 * header file except virtual/overridden methods */
			ValaCCodeFunction* __temp291 = NULL;
			vala_ccode_fragment_append (self->priv->header_type_member_declaration, VALA_CCODE_NODE ((__temp291 = vala_ccode_function_copy (self->priv->function))));
			(__temp291 == NULL ? NULL : (__temp291 = (g_object_unref (__temp291), NULL)));
		} else {
			/* declare all other functions in source file to
			 * avoid dependency on order within source file */
			vala_ccode_function_set_modifiers (self->priv->function, VALA_CCODE_MODIFIERS_STATIC);
			ValaCCodeFunction* __temp292 = NULL;
			vala_ccode_fragment_append (self->priv->source_type_member_declaration, VALA_CCODE_NODE ((__temp292 = vala_ccode_function_copy (self->priv->function))));
			(__temp292 == NULL ? NULL : (__temp292 = (g_object_unref (__temp292), NULL)));
		}
		/* Methods imported from a plain C file don't
		 * have a body, e.g. Vala.Parser.parse_file () */
		if (vala_method_get_body (m) != NULL) {
			vala_ccode_function_set_block (self->priv->function, VALA_CCODE_BLOCK (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_method_get_body (m)))));
			ValaCCodeFragment* cinit = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL);
			vala_ccode_block_prepend_statement (vala_ccode_function_get_block (self->priv->function), VALA_CCODE_NODE (cinit));
			if (VALA_IS_CLASS (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (m)))))) {
				ValaClass* cl = VALA_CLASS (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (m)))));
				if (vala_method_get_overrides (m)) {
					ValaCCodeFunctionCall* ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL))));
					ValaCCodeIdentifier* __temp293 = NULL;
					vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp293 = vala_ccode_identifier_new ("base"))));
					(__temp293 == NULL ? NULL : (__temp293 = (g_object_unref (__temp293), NULL)));
					ValaCCodeDeclaration* cdecl = vala_ccode_declaration_new (g_strdup_printf ("%s *", vala_data_type_get_cname (VALA_DATA_TYPE (cl))));
					ValaCCodeVariableDeclarator* __temp294 = NULL;
					vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp294 = vala_ccode_variable_declarator_new_with_initializer ("self", VALA_CCODE_EXPRESSION (ccall)))));
					(__temp294 == NULL ? NULL : (__temp294 = (g_object_unref (__temp294), NULL)));
					vala_ccode_fragment_append (cinit, VALA_CCODE_NODE (cdecl));
					(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
					(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
				} else {
					if (vala_method_get_instance (m)) {
						ValaCCodeStatement* __temp295 = NULL;
						vala_ccode_fragment_append (cinit, VALA_CCODE_NODE ((__temp295 = vala_code_generator_create_method_type_check_statement (self, m, VALA_DATA_TYPE (cl), TRUE, "self"))));
						(__temp295 == NULL ? NULL : (__temp295 = (g_object_unref (__temp295), NULL)));
					}
				}
			}
			{
				GList* param_it;
				for (param_it = vala_method_get_parameters (m); param_it != NULL; param_it = param_it->next) {
					ValaFormalParameter* param = param_it->data;
					{
						ValaDataType* t = vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param));
						if (t != NULL && vala_data_type_is_reference_type (t) && !vala_type_reference_get_is_out (vala_formal_parameter_get_type_reference (param))) {
							ValaCCodeStatement* type_check = vala_code_generator_create_method_type_check_statement (self, m, t, vala_type_reference_get_non_null (vala_formal_parameter_get_type_reference (param)), vala_formal_parameter_get_name (param));
							if (type_check != NULL) {
								vala_ccode_fragment_append (cinit, VALA_CCODE_NODE (type_check));
							}
							(type_check == NULL ? NULL : (type_check = (g_object_unref (type_check), NULL)));
						}
					}
				}
			}
			if (vala_code_node_get_source_reference (VALA_CODE_NODE (m)) != NULL && vala_source_reference_get_comment (vala_code_node_get_source_reference (VALA_CODE_NODE (m))) != NULL) {
				ValaCCodeComment* __temp296 = NULL;
				vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE ((__temp296 = vala_ccode_comment_new (vala_source_reference_get_comment (vala_code_node_get_source_reference (VALA_CODE_NODE (m)))))));
				(__temp296 == NULL ? NULL : (__temp296 = (g_object_unref (__temp296), NULL)));
			}
			vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (self->priv->function));
			if (vala_method_get_construction (m)) {
				/* declare construction parameter array*/
				ValaCCodeFunctionCall* cparamsinit = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_new0")));
				ValaCCodeIdentifier* __temp297 = NULL;
				vala_ccode_function_call_add_argument (cparamsinit, VALA_CCODE_EXPRESSION ((__temp297 = vala_ccode_identifier_new ("GParameter"))));
				(__temp297 == NULL ? NULL : (__temp297 = (g_object_unref (__temp297), NULL)));
				ValaCCodeConstant* __temp298 = NULL;
				vala_ccode_function_call_add_argument (cparamsinit, VALA_CCODE_EXPRESSION ((__temp298 = vala_ccode_constant_new (g_strdup_printf ("%i", vala_method_get_n_construction_params (m))))));
				(__temp298 == NULL ? NULL : (__temp298 = (g_object_unref (__temp298), NULL)));
				ValaCCodeDeclaration* cdecl = vala_ccode_declaration_new ("GParameter *");
				ValaCCodeVariableDeclarator* __temp299 = NULL;
				vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp299 = vala_ccode_variable_declarator_new_with_initializer ("__params", VALA_CCODE_EXPRESSION (cparamsinit)))));
				(__temp299 == NULL ? NULL : (__temp299 = (g_object_unref (__temp299), NULL)));
				vala_ccode_fragment_append (cinit, VALA_CCODE_NODE (cdecl));
				ValaCCodeDeclaration* __temp300 = NULL;
				cdecl = (__temp300 = vala_ccode_declaration_new ("GParameter *"), (cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL))), __temp300);
				ValaCCodeVariableDeclarator* __temp301 = NULL;
				vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp301 = vala_ccode_variable_declarator_new_with_initializer ("__params_it", VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("__params"))))));
				(__temp301 == NULL ? NULL : (__temp301 = (g_object_unref (__temp301), NULL)));
				vala_ccode_fragment_append (cinit, VALA_CCODE_NODE (cdecl));
				(cparamsinit == NULL ? NULL : (cparamsinit = (g_object_unref (cparamsinit), NULL)));
				(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
			}
			(cinit == NULL ? NULL : (cinit = (g_object_unref (cinit), NULL)));
		}
	}
	if (vala_method_get_is_abstract (m) || vala_method_get_is_virtual (m)) {
		ValaCCodeFunction* vfunc = vala_ccode_function_new (vala_method_get_cname (m), vala_type_reference_get_cname (vala_method_get_return_type (m), FALSE));
		ValaTypeReference* this_type = vala_type_reference_new ();
		vala_type_reference_set_data_type (this_type, VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (m))))));
		ValaCCodeFormalParameter* cparam = vala_ccode_formal_parameter_new ("self", vala_type_reference_get_cname (this_type, FALSE));
		vala_ccode_function_add_parameter (vfunc, cparam);
		ValaCCodeBlock* vblock = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
		ValaCCodeFunctionCall* vcast = NULL;
		if (VALA_IS_INTERFACE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (m)))))) {
			ValaInterface* iface = VALA_INTERFACE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (m)))));
			ValaCCodeFunctionCall* __temp303 = NULL;
			char* __temp302 = NULL;
			vcast = (__temp303 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (g_strdup_printf ("%s_GET_INTERFACE", (__temp302 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (iface), NULL)))))), (vcast == NULL ? NULL : (vcast = (g_object_unref (vcast), NULL))), __temp303);
			(__temp302 == NULL ? NULL : (__temp302 = (g_free (__temp302), NULL)));
		} else {
			ValaClass* cl = VALA_CLASS (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (m)))));
			ValaCCodeFunctionCall* __temp305 = NULL;
			char* __temp304 = NULL;
			vcast = (__temp305 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (g_strdup_printf ("%s_GET_CLASS", (__temp304 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL)))))), (vcast == NULL ? NULL : (vcast = (g_object_unref (vcast), NULL))), __temp305);
			(__temp304 == NULL ? NULL : (__temp304 = (g_free (__temp304), NULL)));
		}
		ValaCCodeIdentifier* __temp306 = NULL;
		vala_ccode_function_call_add_argument (vcast, VALA_CCODE_EXPRESSION ((__temp306 = vala_ccode_identifier_new ("self"))));
		(__temp306 == NULL ? NULL : (__temp306 = (g_object_unref (__temp306), NULL)));
		ValaCCodeFunctionCall* vcall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (vcast), vala_method_get_name (m))));
		ValaCCodeIdentifier* __temp307 = NULL;
		vala_ccode_function_call_add_argument (vcall, VALA_CCODE_EXPRESSION ((__temp307 = vala_ccode_identifier_new ("self"))));
		(__temp307 == NULL ? NULL : (__temp307 = (g_object_unref (__temp307), NULL)));
		GList* params = vala_method_get_parameters (m);
		{
			GList* param_it;
			for (param_it = params; param_it != NULL; param_it = param_it->next) {
				ValaFormalParameter* param = param_it->data;
				{
					vala_ccode_function_add_parameter (vfunc, VALA_CCODE_FORMAL_PARAMETER (vala_code_node_get_ccodenode (VALA_CODE_NODE (param))));
					ValaCCodeIdentifier* __temp308 = NULL;
					vala_ccode_function_call_add_argument (vcall, VALA_CCODE_EXPRESSION ((__temp308 = vala_ccode_identifier_new (vala_formal_parameter_get_name (param)))));
					(__temp308 == NULL ? NULL : (__temp308 = (g_object_unref (__temp308), NULL)));
				}
			}
		}
		ValaCCodeExpressionStatement* __temp309 = NULL;
		vala_ccode_block_add_statement (vblock, VALA_CCODE_NODE ((__temp309 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vcall)))));
		(__temp309 == NULL ? NULL : (__temp309 = (g_object_unref (__temp309), NULL)));
		ValaCCodeFunction* __temp310 = NULL;
		vala_ccode_fragment_append (self->priv->header_type_member_declaration, VALA_CCODE_NODE ((__temp310 = vala_ccode_function_copy (vfunc))));
		(__temp310 == NULL ? NULL : (__temp310 = (g_object_unref (__temp310), NULL)));
		vala_ccode_function_set_block (vfunc, vblock);
		vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (vfunc));
		(vfunc == NULL ? NULL : (vfunc = (g_object_unref (vfunc), NULL)));
		(this_type == NULL ? NULL : (this_type = (g_object_unref (this_type), NULL)));
		(cparam == NULL ? NULL : (cparam = (g_object_unref (cparam), NULL)));
		(vblock == NULL ? NULL : (vblock = (g_object_unref (vblock), NULL)));
		(vcast == NULL ? NULL : (vcast = (g_object_unref (vcast), NULL)));
		(vcall == NULL ? NULL : (vcall = (g_object_unref (vcall), NULL)));
		(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
	}
	if (vala_method_get_construction (m)) {
		ValaCCodeReturnStatement* creturn = vala_ccode_return_statement_new (NULL);
		ValaCCodeIdentifier* __temp311 = NULL;
		vala_ccode_return_statement_set_return_expression (creturn, VALA_CCODE_EXPRESSION ((__temp311 = vala_ccode_identifier_new ("self"))));
		(__temp311 == NULL ? NULL : (__temp311 = (g_object_unref (__temp311), NULL)));
		vala_ccode_block_add_statement (vala_ccode_function_get_block (self->priv->function), VALA_CCODE_NODE (creturn));
		(creturn == NULL ? NULL : (creturn = (g_object_unref (creturn), NULL)));
	}
	if (vala_method_get_name (m) != NULL && g_utf8_collate (vala_method_get_name (m), "main") == 0) {
		ValaCCodeFunction* cmain = vala_ccode_function_new ("main", "int");
		ValaCCodeFormalParameter* __temp312 = NULL;
		vala_ccode_function_add_parameter (cmain, (__temp312 = vala_ccode_formal_parameter_new ("argc", "int")));
		(__temp312 == NULL ? NULL : (__temp312 = (g_object_unref (__temp312), NULL)));
		ValaCCodeFormalParameter* __temp313 = NULL;
		vala_ccode_function_add_parameter (cmain, (__temp313 = vala_ccode_formal_parameter_new ("argv", "char **")));
		(__temp313 == NULL ? NULL : (__temp313 = (g_object_unref (__temp313), NULL)));
		ValaCCodeBlock* main_block = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
		ValaCCodeExpressionStatement* __temp314 = NULL;
		vala_ccode_block_add_statement (main_block, VALA_CCODE_NODE ((__temp314 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_type_init"))))))));
		(__temp314 == NULL ? NULL : (__temp314 = (g_object_unref (__temp314), NULL)));
		ValaCCodeFunctionCall* main_call = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_ccode_function_get_name (self->priv->function))));
		ValaCCodeIdentifier* __temp315 = NULL;
		vala_ccode_function_call_add_argument (main_call, VALA_CCODE_EXPRESSION ((__temp315 = vala_ccode_identifier_new ("argc"))));
		(__temp315 == NULL ? NULL : (__temp315 = (g_object_unref (__temp315), NULL)));
		ValaCCodeIdentifier* __temp316 = NULL;
		vala_ccode_function_call_add_argument (main_call, VALA_CCODE_EXPRESSION ((__temp316 = vala_ccode_identifier_new ("argv"))));
		(__temp316 == NULL ? NULL : (__temp316 = (g_object_unref (__temp316), NULL)));
		ValaCCodeReturnStatement* __temp317 = NULL;
		vala_ccode_block_add_statement (main_block, VALA_CCODE_NODE ((__temp317 = vala_ccode_return_statement_new (VALA_CCODE_EXPRESSION (main_call)))));
		(__temp317 == NULL ? NULL : (__temp317 = (g_object_unref (__temp317), NULL)));
		vala_ccode_function_set_block (cmain, main_block);
		vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (cmain));
		(cmain == NULL ? NULL : (cmain = (g_object_unref (cmain), NULL)));
		(main_block == NULL ? NULL : (main_block = (g_object_unref (main_block), NULL)));
		(main_call == NULL ? NULL : (main_call = (g_object_unref (main_call), NULL)));
	}
	(vdeclarator == NULL ? NULL : (vdeclarator = (g_object_unref (vdeclarator), NULL)));
	(instance_param == NULL ? NULL : (instance_param = (g_object_unref (instance_param), NULL)));
	(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
}


static void vala_code_generator_real_visit_formal_parameter (ValaCodeVisitor* base, ValaFormalParameter* p)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_FORMAL_PARAMETER (p));
	if (!vala_formal_parameter_get_ellipsis (p)) {
		ValaCCodeFormalParameter* __temp318 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (p), VALA_CCODE_NODE ((__temp318 = vala_ccode_formal_parameter_new (vala_formal_parameter_get_name (p), vala_type_reference_get_cname (vala_formal_parameter_get_type_reference (p), FALSE)))));
		(__temp318 == NULL ? NULL : (__temp318 = (g_object_unref (__temp318), NULL)));
	}
}


static void vala_code_generator_real_visit_end_property (ValaCodeVisitor* base, ValaProperty* prop)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_PROPERTY (prop));
	char* __temp319 = NULL;
	vala_ccode_enum_add_value (self->priv->prop_enum, (__temp319 = vala_property_get_upper_case_cname (prop)), NULL);
	(__temp319 == NULL ? NULL : (__temp319 = (g_free (__temp319), NULL)));
}


static void vala_code_generator_real_visit_end_property_accessor (ValaCodeVisitor* base, ValaPropertyAccessor* acc)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_PROPERTY_ACCESSOR (acc));
	ValaProperty* prop = VALA_PROPERTY (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (acc)))));
	ValaClass* cl = VALA_CLASS (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (prop)))));
	if (vala_property_accessor_get_readable (acc)) {
		ValaCCodeFunction* __temp321 = NULL;
		char* __temp320 = NULL;
		self->priv->function = (__temp321 = vala_ccode_function_new (g_strdup_printf ("%s_get_%s", (__temp320 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL)), vala_property_get_name (prop)), vala_type_reference_get_cname (vala_property_get_type_reference (prop), FALSE)), (self->priv->function == NULL ? NULL : (self->priv->function = (g_object_unref (self->priv->function), NULL))), __temp321);
		(__temp320 == NULL ? NULL : (__temp320 = (g_free (__temp320), NULL)));
	} else {
		ValaCCodeFunction* __temp323 = NULL;
		char* __temp322 = NULL;
		self->priv->function = (__temp323 = vala_ccode_function_new (g_strdup_printf ("%s_set_%s", (__temp322 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL)), vala_property_get_name (prop)), "void"), (self->priv->function == NULL ? NULL : (self->priv->function = (g_object_unref (self->priv->function), NULL))), __temp323);
		(__temp322 == NULL ? NULL : (__temp322 = (g_free (__temp322), NULL)));
	}
	ValaTypeReference* this_type = vala_type_reference_new ();
	vala_type_reference_set_data_type (this_type, VALA_DATA_TYPE (cl));
	ValaCCodeFormalParameter* cparam = vala_ccode_formal_parameter_new ("self", vala_type_reference_get_cname (this_type, FALSE));
	vala_ccode_function_add_parameter (self->priv->function, cparam);
	if (vala_property_accessor_get_writable (acc) || vala_property_accessor_get_construction (acc)) {
		ValaCCodeFormalParameter* __temp324 = NULL;
		vala_ccode_function_add_parameter (self->priv->function, (__temp324 = vala_ccode_formal_parameter_new ("value", vala_type_reference_get_cname (vala_property_get_type_reference (prop), FALSE))));
		(__temp324 == NULL ? NULL : (__temp324 = (g_object_unref (__temp324), NULL)));
	}
	ValaCCodeFunction* __temp325 = NULL;
	vala_ccode_fragment_append (self->priv->header_type_member_declaration, VALA_CCODE_NODE ((__temp325 = vala_ccode_function_copy (self->priv->function))));
	(__temp325 == NULL ? NULL : (__temp325 = (g_object_unref (__temp325), NULL)));
	if (vala_property_accessor_get_body (acc) != NULL) {
		vala_ccode_function_set_block (self->priv->function, VALA_CCODE_BLOCK (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_property_accessor_get_body (acc)))));
		ValaCCodeStatement* __temp326 = NULL;
		vala_ccode_block_prepend_statement (vala_ccode_function_get_block (self->priv->function), VALA_CCODE_NODE ((__temp326 = vala_code_generator_create_property_type_check_statement (self, prop, vala_property_accessor_get_readable (acc), VALA_DATA_TYPE (cl), TRUE, "self"))));
		(__temp326 == NULL ? NULL : (__temp326 = (g_object_unref (__temp326), NULL)));
	}
	vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (self->priv->function));
	(this_type == NULL ? NULL : (this_type = (g_object_unref (this_type), NULL)));
	(cparam == NULL ? NULL : (cparam = (g_object_unref (cparam), NULL)));
}


static void vala_code_generator_real_visit_end_constructor (ValaCodeVisitor* base, ValaConstructor* c)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_CONSTRUCTOR (c));
	ValaClass* cl = VALA_CLASS (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (c)))));
	ValaCCodeFunction* __temp328 = NULL;
	char* __temp327 = NULL;
	self->priv->function = (__temp328 = vala_ccode_function_new (g_strdup_printf ("%s_constructor", (__temp327 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL))), "GObject *"), (self->priv->function == NULL ? NULL : (self->priv->function = (g_object_unref (self->priv->function), NULL))), __temp328);
	(__temp327 == NULL ? NULL : (__temp327 = (g_free (__temp327), NULL)));
	vala_ccode_function_set_modifiers (self->priv->function, VALA_CCODE_MODIFIERS_STATIC);
	ValaCCodeFormalParameter* __temp329 = NULL;
	vala_ccode_function_add_parameter (self->priv->function, (__temp329 = vala_ccode_formal_parameter_new ("type", "GType")));
	(__temp329 == NULL ? NULL : (__temp329 = (g_object_unref (__temp329), NULL)));
	ValaCCodeFormalParameter* __temp330 = NULL;
	vala_ccode_function_add_parameter (self->priv->function, (__temp330 = vala_ccode_formal_parameter_new ("n_construct_properties", "guint")));
	(__temp330 == NULL ? NULL : (__temp330 = (g_object_unref (__temp330), NULL)));
	ValaCCodeFormalParameter* __temp331 = NULL;
	vala_ccode_function_add_parameter (self->priv->function, (__temp331 = vala_ccode_formal_parameter_new ("construct_properties", "GObjectConstructParam *")));
	(__temp331 == NULL ? NULL : (__temp331 = (g_object_unref (__temp331), NULL)));
	ValaCCodeFunction* __temp332 = NULL;
	vala_ccode_fragment_append (self->priv->source_type_member_declaration, VALA_CCODE_NODE ((__temp332 = vala_ccode_function_copy (self->priv->function))));
	(__temp332 == NULL ? NULL : (__temp332 = (g_object_unref (__temp332), NULL)));
	ValaCCodeBlock* cblock = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
	ValaCCodeDeclaration* cdecl = vala_ccode_declaration_new ("GObject *");
	ValaCCodeVariableDeclarator* __temp333 = NULL;
	vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp333 = vala_ccode_variable_declarator_new ("obj"))));
	(__temp333 == NULL ? NULL : (__temp333 = (g_object_unref (__temp333), NULL)));
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (cdecl));
	ValaCCodeDeclaration* __temp334 = NULL;
	cdecl = (__temp334 = vala_ccode_declaration_new (g_strdup_printf ("%sClass *", vala_data_type_get_cname (VALA_DATA_TYPE (cl)))), (cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL))), __temp334);
	ValaCCodeVariableDeclarator* __temp335 = NULL;
	vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp335 = vala_ccode_variable_declarator_new ("klass"))));
	(__temp335 == NULL ? NULL : (__temp335 = (g_object_unref (__temp335), NULL)));
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (cdecl));
	ValaCCodeDeclaration* __temp336 = NULL;
	cdecl = (__temp336 = vala_ccode_declaration_new ("GObjectClass *"), (cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL))), __temp336);
	ValaCCodeVariableDeclarator* __temp337 = NULL;
	vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp337 = vala_ccode_variable_declarator_new ("parent_class"))));
	(__temp337 == NULL ? NULL : (__temp337 = (g_object_unref (__temp337), NULL)));
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (cdecl));
	ValaCCodeFunctionCall* ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_type_class_peek")));
	ValaCCodeIdentifier* __temp338 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp338 = vala_ccode_identifier_new (vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), "TYPE_")))));
	(__temp338 == NULL ? NULL : (__temp338 = (g_object_unref (__temp338), NULL)));
	char* __temp339 = NULL;
	ValaCCodeFunctionCall* __temp340 = NULL;
	ValaCCodeFunctionCall* ccast = (__temp340 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (g_strdup_printf ("%s_CLASS", (__temp339 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL)))))), (__temp339 == NULL ? NULL : (__temp339 = (g_free (__temp339), NULL))), __temp340);
	vala_ccode_function_call_add_argument (ccast, VALA_CCODE_EXPRESSION (ccall));
	ValaCCodeExpressionStatement* __temp341 = NULL;
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE ((__temp341 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("klass")), VALA_CCODE_EXPRESSION (ccast), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))))));
	(__temp341 == NULL ? NULL : (__temp341 = (g_object_unref (__temp341), NULL)));
	ValaCCodeFunctionCall* __temp342 = NULL;
	ccall = (__temp342 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_type_class_peek_parent"))), (ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL))), __temp342);
	ValaCCodeIdentifier* __temp343 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp343 = vala_ccode_identifier_new ("klass"))));
	(__temp343 == NULL ? NULL : (__temp343 = (g_object_unref (__temp343), NULL)));
	ValaCCodeFunctionCall* __temp344 = NULL;
	ccast = (__temp344 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("G_OBJECT_CLASS"))), (ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL))), __temp344);
	vala_ccode_function_call_add_argument (ccast, VALA_CCODE_EXPRESSION (ccall));
	ValaCCodeExpressionStatement* __temp345 = NULL;
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE ((__temp345 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("parent_class")), VALA_CCODE_EXPRESSION (ccast), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))))));
	(__temp345 == NULL ? NULL : (__temp345 = (g_object_unref (__temp345), NULL)));
	ValaCCodeFunctionCall* __temp346 = NULL;
	ccall = (__temp346 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("parent_class")), "constructor"))), (ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL))), __temp346);
	ValaCCodeIdentifier* __temp347 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp347 = vala_ccode_identifier_new ("type"))));
	(__temp347 == NULL ? NULL : (__temp347 = (g_object_unref (__temp347), NULL)));
	ValaCCodeIdentifier* __temp348 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp348 = vala_ccode_identifier_new ("n_construct_properties"))));
	(__temp348 == NULL ? NULL : (__temp348 = (g_object_unref (__temp348), NULL)));
	ValaCCodeIdentifier* __temp349 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp349 = vala_ccode_identifier_new ("construct_properties"))));
	(__temp349 == NULL ? NULL : (__temp349 = (g_object_unref (__temp349), NULL)));
	ValaCCodeExpressionStatement* __temp350 = NULL;
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE ((__temp350 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("obj")), VALA_CCODE_EXPRESSION (ccall), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))))));
	(__temp350 == NULL ? NULL : (__temp350 = (g_object_unref (__temp350), NULL)));
	ValaCCodeFunctionCall* __temp351 = NULL;
	ccall = (__temp351 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (cl), NULL)))), (ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL))), __temp351);
	ValaCCodeIdentifier* __temp352 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp352 = vala_ccode_identifier_new ("obj"))));
	(__temp352 == NULL ? NULL : (__temp352 = (g_object_unref (__temp352), NULL)));
	ValaCCodeDeclaration* __temp353 = NULL;
	cdecl = (__temp353 = vala_ccode_declaration_new (g_strdup_printf ("%s *", vala_data_type_get_cname (VALA_DATA_TYPE (cl)))), (cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL))), __temp353);
	ValaCCodeVariableDeclarator* __temp354 = NULL;
	vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp354 = vala_ccode_variable_declarator_new_with_initializer ("self", VALA_CCODE_EXPRESSION (ccall)))));
	(__temp354 == NULL ? NULL : (__temp354 = (g_object_unref (__temp354), NULL)));
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (cdecl));
	vala_ccode_block_add_statement (cblock, vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_constructor_get_body (c))));
	ValaCCodeReturnStatement* __temp355 = NULL;
	vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE ((__temp355 = vala_ccode_return_statement_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("obj"))))));
	(__temp355 == NULL ? NULL : (__temp355 = (g_object_unref (__temp355), NULL)));
	vala_ccode_function_set_block (self->priv->function, cblock);
	if (vala_source_reference_get_comment (vala_code_node_get_source_reference (VALA_CODE_NODE (c))) != NULL) {
		ValaCCodeComment* __temp356 = NULL;
		vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE ((__temp356 = vala_ccode_comment_new (vala_source_reference_get_comment (vala_code_node_get_source_reference (VALA_CODE_NODE (c)))))));
		(__temp356 == NULL ? NULL : (__temp356 = (g_object_unref (__temp356), NULL)));
	}
	vala_ccode_fragment_append (self->priv->source_type_member_definition, VALA_CCODE_NODE (self->priv->function));
	(cblock == NULL ? NULL : (cblock = (g_object_unref (cblock), NULL)));
	(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
	(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
	(ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL)));
}


static void vala_code_generator_real_visit_begin_block (ValaCodeVisitor* base, ValaBlock* b)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_BLOCK (b));
	ValaSymbol* __temp358 = NULL;
	ValaSymbol* __temp357 = NULL;
	self->priv->current_symbol = (__temp358 = (__temp357 = vala_code_node_get_symbol (VALA_CODE_NODE (b)), (__temp357 == NULL ? __temp357 : g_object_ref (__temp357))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp358);
}


static void vala_code_generator_add_object_creation (ValaCodeGenerator* self, ValaCCodeBlock* b)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (VALA_IS_CCODE_BLOCK (b));
	ValaClass* cl = VALA_CLASS (vala_symbol_get_node (self->priv->current_type_symbol));
	ValaCCodeFunctionCall* ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_object_newv")));
	ValaCCodeConstant* __temp359 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp359 = vala_ccode_constant_new (vala_data_type_get_type_id (VALA_DATA_TYPE (cl))))));
	(__temp359 == NULL ? NULL : (__temp359 = (g_object_unref (__temp359), NULL)));
	ValaCCodeConstant* __temp360 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp360 = vala_ccode_constant_new ("__params_it - __params"))));
	(__temp360 == NULL ? NULL : (__temp360 = (g_object_unref (__temp360), NULL)));
	ValaCCodeConstant* __temp361 = NULL;
	vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp361 = vala_ccode_constant_new ("__params"))));
	(__temp361 == NULL ? NULL : (__temp361 = (g_object_unref (__temp361), NULL)));
	ValaCCodeVariableDeclarator* cdecl = vala_ccode_variable_declarator_new ("self");
	vala_ccode_variable_declarator_set_initializer (cdecl, VALA_CCODE_EXPRESSION (ccall));
	ValaCCodeDeclaration* cdeclaration = vala_ccode_declaration_new (g_strdup_printf ("%s *", vala_data_type_get_cname (VALA_DATA_TYPE (cl))));
	vala_ccode_declaration_add_declarator (cdeclaration, VALA_CCODE_DECLARATOR (cdecl));
	vala_ccode_block_add_statement (b, VALA_CCODE_NODE (cdeclaration));
	(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
	(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
	(cdeclaration == NULL ? NULL : (cdeclaration = (g_object_unref (cdeclaration), NULL)));
}


static void vala_code_generator_real_visit_end_block (ValaCodeVisitor* base, ValaBlock* b)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_BLOCK (b));
	GList* local_vars = vala_block_get_local_variables (b);
	{
		GList* decl_it;
		for (decl_it = local_vars; decl_it != NULL; decl_it = decl_it->next) {
			ValaVariableDeclarator* decl = decl_it->data;
			{
				vala_symbol_set_active (vala_code_node_get_symbol (VALA_CODE_NODE (decl)), FALSE);
			}
		}
	}
	gboolean in_construction = vala_statement_get_construction (VALA_STATEMENT (b));
	ValaCCodeBlock* cblock = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
	{
		GList* stmt_it;
		for (stmt_it = vala_block_get_statements (b); stmt_it != NULL; stmt_it = stmt_it->next) {
			ValaStatement* stmt = stmt_it->data;
			{
				if (in_construction && !vala_statement_get_construction (stmt)) {
					/* construction part of construction method ends here*/
					vala_code_generator_add_object_creation (self, cblock);
					in_construction = FALSE;
				}
				ValaSourceReference* src = vala_code_node_get_source_reference (VALA_CODE_NODE (stmt));
				if (src != NULL && vala_source_reference_get_comment (src) != NULL) {
					ValaCCodeComment* __temp362 = NULL;
					vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE ((__temp362 = vala_ccode_comment_new (vala_source_reference_get_comment (src)))));
					(__temp362 == NULL ? NULL : (__temp362 = (g_object_unref (__temp362), NULL)));
				}
				if (VALA_IS_CCODE_FRAGMENT (vala_code_node_get_ccodenode (VALA_CODE_NODE (stmt)))) {
					{
						GList* cstmt_it;
						for (cstmt_it = vala_ccode_fragment_get_children ((VALA_CCODE_FRAGMENT (vala_code_node_get_ccodenode (VALA_CODE_NODE (stmt))))); cstmt_it != NULL; cstmt_it = cstmt_it->next) {
							ValaCCodeStatement* cstmt = cstmt_it->data;
							{
								vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (cstmt));
							}
						}
					}
				} else {
					vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (VALA_CCODE_STATEMENT (vala_code_node_get_ccodenode (VALA_CODE_NODE (stmt)))));
				}
			}
		}
	}
	if (in_construction) {
		/* construction method doesn't contain non-construction parts*/
		vala_code_generator_add_object_creation (self, cblock);
	}
	if (vala_code_generator_get_memory_management (self)) {
		{
			GList* decl_it;
			for (decl_it = local_vars; decl_it != NULL; decl_it = decl_it->next) {
				ValaVariableDeclarator* decl = decl_it->data;
				{
					if (vala_data_type_is_reference_type (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl))) && vala_type_reference_get_takes_ownership (vala_variable_declarator_get_type_reference (decl))) {
						ValaCCodeExpressionStatement* __temp364 = NULL;
						ValaCCodeIdentifier* __temp363 = NULL;
						vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE ((__temp364 = vala_ccode_expression_statement_new (vala_code_generator_get_unref_expression (self, VALA_CCODE_EXPRESSION ((__temp363 = vala_ccode_identifier_new (vala_variable_declarator_get_name (decl)))), vala_variable_declarator_get_type_reference (decl))))));
						(__temp364 == NULL ? NULL : (__temp364 = (g_object_unref (__temp364), NULL)));
						(__temp363 == NULL ? NULL : (__temp363 = (g_object_unref (__temp363), NULL)));
					}
				}
			}
		}
	}
	vala_code_node_set_ccodenode (VALA_CODE_NODE (b), VALA_CCODE_NODE (cblock));
	ValaSymbol* __temp366 = NULL;
	ValaSymbol* __temp365 = NULL;
	self->priv->current_symbol = (__temp366 = (__temp365 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp365 == NULL ? __temp365 : g_object_ref (__temp365))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp366);
	(local_vars == NULL ? NULL : (local_vars = (g_list_free (local_vars), NULL)));
	(cblock == NULL ? NULL : (cblock = (g_object_unref (cblock), NULL)));
}


static void vala_code_generator_real_visit_empty_statement (ValaCodeVisitor* base, ValaEmptyStatement* stmt)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_EMPTY_STATEMENT (stmt));
	ValaCCodeEmptyStatement* __temp367 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE ((__temp367 = g_object_new (VALA_TYPE_CCODE_EMPTY_STATEMENT, NULL))));
	(__temp367 == NULL ? NULL : (__temp367 = (g_object_unref (__temp367), NULL)));
}


static void vala_code_generator_real_visit_declaration_statement (ValaCodeVisitor* base, ValaDeclarationStatement* stmt)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_DECLARATION_STATEMENT (stmt));
	/* split declaration statement as var declarators
	 * might have different types */
	ValaCCodeFragment* cfrag = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL);
	{
		GList* decl_it;
		for (decl_it = vala_local_variable_declaration_get_variable_declarators (vala_declaration_statement_get_declaration (stmt)); decl_it != NULL; decl_it = decl_it->next) {
			ValaVariableDeclarator* decl = decl_it->data;
			{
				ValaCCodeDeclaration* cdecl = vala_ccode_declaration_new (vala_type_reference_get_cname (vala_variable_declarator_get_type_reference (decl), FALSE));
				vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR (VALA_CCODE_VARIABLE_DECLARATOR (vala_code_node_get_ccodenode (VALA_CODE_NODE (decl)))));
				vala_ccode_fragment_append (cfrag, VALA_CCODE_NODE (cdecl));
				(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
			}
		}
	}
	vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE (cfrag));
	{
		GList* decl_it;
		for (decl_it = vala_local_variable_declaration_get_variable_declarators (vala_declaration_statement_get_declaration (stmt)); decl_it != NULL; decl_it = decl_it->next) {
			ValaVariableDeclarator* decl = decl_it->data;
			{
				if (vala_variable_declarator_get_initializer (decl) != NULL) {
					vala_code_generator_create_temp_decl (self, VALA_STATEMENT (stmt), vala_variable_declarator_get_initializer (decl)->temp_vars);
				}
			}
		}
	}
	(cfrag == NULL ? NULL : (cfrag = (g_object_unref (cfrag), NULL)));
}


static void vala_code_generator_real_visit_variable_declarator (ValaCodeVisitor* base, ValaVariableDeclarator* decl)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_VARIABLE_DECLARATOR (decl));
	ValaCCodeExpression* rhs = NULL;
	if (vala_variable_declarator_get_initializer (decl) != NULL) {
		ValaCCodeExpression* __temp369 = NULL;
		ValaCCodeExpression* __temp368 = NULL;
		rhs = (__temp369 = (__temp368 = VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_variable_declarator_get_initializer (decl)))), (__temp368 == NULL ? __temp368 : g_object_ref (__temp368))), (rhs == NULL ? NULL : (rhs = (g_object_unref (rhs), NULL))), __temp369);
		if (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl)) != NULL && vala_type_reference_get_data_type (vala_expression_get_static_type (vala_variable_declarator_get_initializer (decl))) != NULL && vala_data_type_is_reference_type (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl))) && vala_type_reference_get_data_type (vala_expression_get_static_type (vala_variable_declarator_get_initializer (decl))) != vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl))) {
			ValaCCodeExpression* __temp370 = NULL;
			rhs = (__temp370 = VALA_CCODE_EXPRESSION (vala_instance_cast_new (rhs, vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl)))), (rhs == NULL ? NULL : (rhs = (g_object_unref (rhs), NULL))), __temp370);
		}
	} else {
		if (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl)) != NULL && vala_data_type_is_reference_type (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl)))) {
			ValaCCodeExpression* __temp371 = NULL;
			rhs = (__temp371 = VALA_CCODE_EXPRESSION (vala_ccode_constant_new ("NULL")), (rhs == NULL ? NULL : (rhs = (g_object_unref (rhs), NULL))), __temp371);
		}
	}
	ValaCCodeVariableDeclarator* __temp372 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (decl), VALA_CCODE_NODE ((__temp372 = vala_ccode_variable_declarator_new_with_initializer (vala_variable_declarator_get_name (decl), rhs))));
	(__temp372 == NULL ? NULL : (__temp372 = (g_object_unref (__temp372), NULL)));
	vala_symbol_set_active (vala_code_node_get_symbol (VALA_CODE_NODE (decl)), TRUE);
	(rhs == NULL ? NULL : (rhs = (g_object_unref (rhs), NULL)));
}


static void vala_code_generator_real_visit_initializer_list (ValaCodeVisitor* base, ValaInitializerList* list)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_INITIALIZER_LIST (list));
	ValaCCodeInitializerList* clist = g_object_new (VALA_TYPE_CCODE_INITIALIZER_LIST, NULL);
	{
		GList* expr_it;
		for (expr_it = vala_initializer_list_get_initializers (list); expr_it != NULL; expr_it = expr_it->next) {
			ValaExpression* expr = expr_it->data;
			{
				vala_ccode_initializer_list_append (clist, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (expr))));
			}
		}
	}
	vala_code_node_set_ccodenode (VALA_CODE_NODE (list), VALA_CCODE_NODE (clist));
	(clist == NULL ? NULL : (clist = (g_object_unref (clist), NULL)));
}


static ValaVariableDeclarator* vala_code_generator_get_temp_variable_declarator (ValaCodeGenerator* self, ValaTypeReference* type)
{
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (self), NULL);
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (type), NULL);
	ValaVariableDeclarator* decl = vala_variable_declarator_new (g_strdup_printf ("__temp%d", self->priv->next_temp_var_id), NULL, NULL);
	ValaTypeReference* __temp373 = NULL;
	vala_variable_declarator_set_type_reference (decl, (__temp373 = vala_type_reference_copy (type)));
	(__temp373 == NULL ? NULL : (__temp373 = (g_object_unref (__temp373), NULL)));
	vala_type_reference_set_reference_to_value_type (vala_variable_declarator_get_type_reference (decl), FALSE);
	vala_type_reference_set_is_out (vala_variable_declarator_get_type_reference (decl), FALSE);
	self->priv->next_temp_var_id++;
	ValaVariableDeclarator* __temp374 = NULL;
	ValaVariableDeclarator* __temp375 = NULL;
	return (__temp375 = (__temp374 = decl, (__temp374 == NULL ? __temp374 : g_object_ref (__temp374))), (decl == NULL ? NULL : (decl = (g_object_unref (decl), NULL))), __temp375);
	(decl == NULL ? NULL : (decl = (g_object_unref (decl), NULL)));
}


static ValaCCodeExpression* vala_code_generator_get_unref_expression (ValaCodeGenerator* self, ValaCCodeExpression* cvar, ValaTypeReference* type)
{
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (self), NULL);
	g_return_val_if_fail (VALA_IS_CCODE_EXPRESSION (cvar), NULL);
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (type), NULL);
	/* (foo == NULL ? NULL : foo = (unref (foo), NULL)) 
	 can be simplified to
	 * foo = (unref (foo), NULL)
	 * if foo is of static type non-null
	 */
	ValaCCodeBinaryExpression* cisnull = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_EQUALITY, cvar, VALA_CCODE_EXPRESSION (vala_ccode_constant_new ("NULL")));
	char* unref_function = NULL;
	if (vala_data_type_is_reference_counting (vala_type_reference_get_data_type (type))) {
		unref_function = vala_data_type_get_unref_function (vala_type_reference_get_data_type (type));
	} else {
		unref_function = vala_data_type_get_free_function (vala_type_reference_get_data_type (type));
	}
	if (VALA_IS_ARRAY (vala_type_reference_get_data_type (type)) && g_utf8_collate (vala_data_type_get_name (vala_array_get_element_type ((VALA_ARRAY (vala_type_reference_get_data_type (type))))), "string") == 0) {
		unref_function = "g_strfreev";
	}
	ValaCCodeFunctionCall* ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (unref_function)));
	vala_ccode_function_call_add_argument (ccall, cvar);
	/* set freed references to NULL to prevent further use */
	ValaCCodeCommaExpression* ccomma = g_object_new (VALA_TYPE_CCODE_COMMA_EXPRESSION, NULL);
	if (g_utf8_collate (unref_function, "g_list_free") == 0) {
		gboolean is_ref = FALSE;
		gboolean is_class = FALSE;
		GList* type_args = vala_type_reference_get_type_arguments (type);
		{
			GList* type_arg_it;
			for (type_arg_it = type_args; type_arg_it != NULL; type_arg_it = type_arg_it->next) {
				ValaTypeReference* type_arg = type_arg_it->data;
				{
					is_ref = vala_type_reference_get_takes_ownership (type_arg);
					is_class = VALA_IS_CLASS (vala_type_reference_get_data_type (type_arg));
				}
			}
		}
		if (is_ref) {
			ValaCCodeFunctionCall* cunrefcall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_list_foreach")));
			vala_ccode_function_call_add_argument (cunrefcall, cvar);
			if (is_class) {
				ValaCCodeIdentifier* __temp376 = NULL;
				vala_ccode_function_call_add_argument (cunrefcall, VALA_CCODE_EXPRESSION ((__temp376 = vala_ccode_identifier_new ("(GFunc) g_object_unref"))));
				(__temp376 == NULL ? NULL : (__temp376 = (g_object_unref (__temp376), NULL)));
			} else {
				ValaCCodeIdentifier* __temp377 = NULL;
				vala_ccode_function_call_add_argument (cunrefcall, VALA_CCODE_EXPRESSION ((__temp377 = vala_ccode_identifier_new ("(GFunc) g_free"))));
				(__temp377 == NULL ? NULL : (__temp377 = (g_object_unref (__temp377), NULL)));
			}
			ValaCCodeConstant* __temp378 = NULL;
			vala_ccode_function_call_add_argument (cunrefcall, VALA_CCODE_EXPRESSION ((__temp378 = vala_ccode_constant_new ("NULL"))));
			(__temp378 == NULL ? NULL : (__temp378 = (g_object_unref (__temp378), NULL)));
			vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION (cunrefcall));
			(cunrefcall == NULL ? NULL : (cunrefcall = (g_object_unref (cunrefcall), NULL)));
		}
		(type_args == NULL ? NULL : (type_args = (g_list_free (type_args), NULL)));
	} else {
		if (g_utf8_collate (unref_function, "g_string_free") == 0) {
			ValaCCodeConstant* __temp379 = NULL;
			vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp379 = vala_ccode_constant_new ("TRUE"))));
			(__temp379 == NULL ? NULL : (__temp379 = (g_object_unref (__temp379), NULL)));
		}
	}
	vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION (ccall));
	ValaCCodeConstant* __temp380 = NULL;
	vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION ((__temp380 = vala_ccode_constant_new ("NULL"))));
	(__temp380 == NULL ? NULL : (__temp380 = (g_object_unref (__temp380), NULL)));
	ValaCCodeAssignment* cassign = vala_ccode_assignment_new (cvar, VALA_CCODE_EXPRESSION (ccomma), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE);
	ValaCCodeConditionalExpression* __temp381 = NULL;
	return (__temp381 = vala_ccode_conditional_expression_new (VALA_CCODE_EXPRESSION (cisnull), VALA_CCODE_EXPRESSION (vala_ccode_constant_new ("NULL")), VALA_CCODE_EXPRESSION (vala_ccode_parenthesized_expression_new (VALA_CCODE_EXPRESSION (cassign)))), (cisnull == NULL ? NULL : (cisnull = (g_object_unref (cisnull), NULL))), (ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL))), (ccomma == NULL ? NULL : (ccomma = (g_object_unref (ccomma), NULL))), (cassign == NULL ? NULL : (cassign = (g_object_unref (cassign), NULL))), __temp381);
	(cisnull == NULL ? NULL : (cisnull = (g_object_unref (cisnull), NULL)));
	(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
	(ccomma == NULL ? NULL : (ccomma = (g_object_unref (ccomma), NULL)));
	(cassign == NULL ? NULL : (cassign = (g_object_unref (cassign), NULL)));
}


static void vala_code_generator_real_visit_end_full_expression (ValaCodeVisitor* base, ValaExpression* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_EXPRESSION (expr));
	if (!vala_code_generator_get_memory_management (self)) {
		GList* __temp382 = NULL;
		self->priv->temp_vars = (__temp382 = NULL, (self->priv->temp_vars == NULL ? NULL : (self->priv->temp_vars = (g_list_foreach (self->priv->temp_vars, (GFunc) g_object_unref, NULL), g_list_free (self->priv->temp_vars), NULL))), __temp382);
		GList* __temp383 = NULL;
		self->priv->temp_ref_vars = (__temp383 = NULL, (self->priv->temp_ref_vars == NULL ? NULL : (self->priv->temp_ref_vars = (g_list_foreach (self->priv->temp_ref_vars, (GFunc) g_object_unref, NULL), g_list_free (self->priv->temp_ref_vars), NULL))), __temp383);
		return;
	}
	/* expr is a full expression, i.e. an initializer, the
	 * expression in an expression statement, the controlling
	 * expression in if, while, for, or foreach statements
	 *
	 * we unref temporary variables at the end of a full
	 * expression
	 
	 can't automatically deep copy lists yet, so do it
	 * manually for now
	 * replace with
	 * expr.temp_vars = temp_vars;
	 * when deep list copying works
	 */
	GList* __temp384 = NULL;
	expr->temp_vars = (__temp384 = NULL, (expr->temp_vars == NULL ? NULL : (expr->temp_vars = (g_list_foreach (expr->temp_vars, (GFunc) g_object_unref, NULL), g_list_free (expr->temp_vars), NULL))), __temp384);
	{
		GList* decl1_it;
		for (decl1_it = self->priv->temp_vars; decl1_it != NULL; decl1_it = decl1_it->next) {
			ValaVariableDeclarator* decl1 = decl1_it->data;
			{
				ValaVariableDeclarator* __temp385 = NULL;
				expr->temp_vars = g_list_append (expr->temp_vars, (__temp385 = decl1, (__temp385 == NULL ? __temp385 : g_object_ref (__temp385))));
			}
		}
	}
	GList* __temp386 = NULL;
	self->priv->temp_vars = (__temp386 = NULL, (self->priv->temp_vars == NULL ? NULL : (self->priv->temp_vars = (g_list_foreach (self->priv->temp_vars, (GFunc) g_object_unref, NULL), g_list_free (self->priv->temp_vars), NULL))), __temp386);
	if (self->priv->temp_ref_vars == NULL) {
		return;
	}
	/* nothing to do without temporary variables */
	ValaVariableDeclarator* full_expr_decl = vala_code_generator_get_temp_variable_declarator (self, vala_expression_get_static_type (expr));
	ValaVariableDeclarator* __temp387 = NULL;
	expr->temp_vars = g_list_append (expr->temp_vars, (__temp387 = full_expr_decl, (__temp387 == NULL ? __temp387 : g_object_ref (__temp387))));
	ValaCCodeCommaExpression* expr_list = g_object_new (VALA_TYPE_CCODE_COMMA_EXPRESSION, NULL);
	ValaCCodeAssignment* __temp388 = NULL;
	vala_ccode_comma_expression_append_expression (expr_list, VALA_CCODE_EXPRESSION ((__temp388 = vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_variable_declarator_get_name (full_expr_decl))), VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (expr))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))));
	(__temp388 == NULL ? NULL : (__temp388 = (g_object_unref (__temp388), NULL)));
	{
		GList* decl_it;
		for (decl_it = self->priv->temp_ref_vars; decl_it != NULL; decl_it = decl_it->next) {
			ValaVariableDeclarator* decl = decl_it->data;
			{
				ValaCCodeExpression* __temp390 = NULL;
				ValaCCodeIdentifier* __temp389 = NULL;
				vala_ccode_comma_expression_append_expression (expr_list, (__temp390 = vala_code_generator_get_unref_expression (self, VALA_CCODE_EXPRESSION ((__temp389 = vala_ccode_identifier_new (vala_variable_declarator_get_name (decl)))), vala_variable_declarator_get_type_reference (decl))));
				(__temp390 == NULL ? NULL : (__temp390 = (g_object_unref (__temp390), NULL)));
				(__temp389 == NULL ? NULL : (__temp389 = (g_object_unref (__temp389), NULL)));
			}
		}
	}
	ValaCCodeIdentifier* __temp391 = NULL;
	vala_ccode_comma_expression_append_expression (expr_list, VALA_CCODE_EXPRESSION ((__temp391 = vala_ccode_identifier_new (vala_variable_declarator_get_name (full_expr_decl)))));
	(__temp391 == NULL ? NULL : (__temp391 = (g_object_unref (__temp391), NULL)));
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE (expr_list));
	GList* __temp392 = NULL;
	self->priv->temp_ref_vars = (__temp392 = NULL, (self->priv->temp_ref_vars == NULL ? NULL : (self->priv->temp_ref_vars = (g_list_foreach (self->priv->temp_ref_vars, (GFunc) g_object_unref, NULL), g_list_free (self->priv->temp_ref_vars), NULL))), __temp392);
	(full_expr_decl == NULL ? NULL : (full_expr_decl = (g_object_unref (full_expr_decl), NULL)));
	(expr_list == NULL ? NULL : (expr_list = (g_object_unref (expr_list), NULL)));
}


static void vala_code_generator_append_temp_decl (ValaCodeGenerator* self, ValaCCodeFragment* cfrag, GList* temp_vars)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (VALA_IS_CCODE_FRAGMENT (cfrag));
	{
		GList* decl_it;
		for (decl_it = temp_vars; decl_it != NULL; decl_it = decl_it->next) {
			ValaVariableDeclarator* decl = decl_it->data;
			{
				ValaCCodeDeclaration* cdecl = vala_ccode_declaration_new (vala_type_reference_get_cname (vala_variable_declarator_get_type_reference (decl), TRUE));
				ValaCCodeVariableDeclarator* vardecl = vala_ccode_variable_declarator_new (vala_variable_declarator_get_name (decl));
				vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR (vardecl));
				if (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl)) != NULL && vala_data_type_is_reference_type (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl)))) {
					ValaCCodeConstant* __temp393 = NULL;
					vala_ccode_variable_declarator_set_initializer (vardecl, VALA_CCODE_EXPRESSION ((__temp393 = vala_ccode_constant_new ("NULL"))));
					(__temp393 == NULL ? NULL : (__temp393 = (g_object_unref (__temp393), NULL)));
				}
				vala_ccode_fragment_append (cfrag, VALA_CCODE_NODE (cdecl));
				(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
				(vardecl == NULL ? NULL : (vardecl = (g_object_unref (vardecl), NULL)));
			}
		}
	}
}


static void vala_code_generator_real_visit_expression_statement (ValaCodeVisitor* base, ValaExpressionStatement* stmt)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_EXPRESSION_STATEMENT (stmt));
	ValaCCodeExpressionStatement* __temp394 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE ((__temp394 = vala_ccode_expression_statement_new (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_expression_statement_get_expression (stmt))))))));
	(__temp394 == NULL ? NULL : (__temp394 = (g_object_unref (__temp394), NULL)));
	/* free temporary objects */
	if (!vala_code_generator_get_memory_management (self)) {
		GList* __temp395 = NULL;
		self->priv->temp_vars = (__temp395 = NULL, (self->priv->temp_vars == NULL ? NULL : (self->priv->temp_vars = (g_list_foreach (self->priv->temp_vars, (GFunc) g_object_unref, NULL), g_list_free (self->priv->temp_vars), NULL))), __temp395);
		GList* __temp396 = NULL;
		self->priv->temp_ref_vars = (__temp396 = NULL, (self->priv->temp_ref_vars == NULL ? NULL : (self->priv->temp_ref_vars = (g_list_foreach (self->priv->temp_ref_vars, (GFunc) g_object_unref, NULL), g_list_free (self->priv->temp_ref_vars), NULL))), __temp396);
		return;
	}
	if (self->priv->temp_vars == NULL) {
		return;
	}
	/* nothing to do without temporary variables */
	ValaCCodeFragment* cfrag = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL);
	vala_code_generator_append_temp_decl (self, cfrag, self->priv->temp_vars);
	vala_ccode_fragment_append (cfrag, vala_code_node_get_ccodenode (VALA_CODE_NODE (stmt)));
	{
		GList* decl_it;
		for (decl_it = self->priv->temp_ref_vars; decl_it != NULL; decl_it = decl_it->next) {
			ValaVariableDeclarator* decl = decl_it->data;
			{
				ValaCCodeExpressionStatement* __temp398 = NULL;
				ValaCCodeIdentifier* __temp397 = NULL;
				vala_ccode_fragment_append (cfrag, VALA_CCODE_NODE ((__temp398 = vala_ccode_expression_statement_new (vala_code_generator_get_unref_expression (self, VALA_CCODE_EXPRESSION ((__temp397 = vala_ccode_identifier_new (vala_variable_declarator_get_name (decl)))), vala_variable_declarator_get_type_reference (decl))))));
				(__temp398 == NULL ? NULL : (__temp398 = (g_object_unref (__temp398), NULL)));
				(__temp397 == NULL ? NULL : (__temp397 = (g_object_unref (__temp397), NULL)));
			}
		}
	}
	vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE (cfrag));
	GList* __temp399 = NULL;
	self->priv->temp_vars = (__temp399 = NULL, (self->priv->temp_vars == NULL ? NULL : (self->priv->temp_vars = (g_list_foreach (self->priv->temp_vars, (GFunc) g_object_unref, NULL), g_list_free (self->priv->temp_vars), NULL))), __temp399);
	GList* __temp400 = NULL;
	self->priv->temp_ref_vars = (__temp400 = NULL, (self->priv->temp_ref_vars == NULL ? NULL : (self->priv->temp_ref_vars = (g_list_foreach (self->priv->temp_ref_vars, (GFunc) g_object_unref, NULL), g_list_free (self->priv->temp_ref_vars), NULL))), __temp400);
	(cfrag == NULL ? NULL : (cfrag = (g_object_unref (cfrag), NULL)));
}


static void vala_code_generator_create_temp_decl (ValaCodeGenerator* self, ValaStatement* stmt, GList* temp_vars)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (VALA_IS_STATEMENT (stmt));
	/* declare temporary variables */
	if (temp_vars == NULL) {
		return;
	}
	/* nothing to do without temporary variables */
	ValaCCodeFragment* cfrag = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL);
	vala_code_generator_append_temp_decl (self, cfrag, temp_vars);
	vala_ccode_fragment_append (cfrag, vala_code_node_get_ccodenode (VALA_CODE_NODE (stmt)));
	vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE (cfrag));
	(cfrag == NULL ? NULL : (cfrag = (g_object_unref (cfrag), NULL)));
}


static void vala_code_generator_real_visit_if_statement (ValaCodeVisitor* base, ValaIfStatement* stmt)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_IF_STATEMENT (stmt));
	if (vala_if_statement_get_false_statement (stmt) != NULL) {
		ValaCCodeIfStatement* __temp401 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE ((__temp401 = vala_ccode_if_statement_new (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_if_statement_get_condition (stmt)))), VALA_CCODE_STATEMENT (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_if_statement_get_true_statement (stmt)))), VALA_CCODE_STATEMENT (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_if_statement_get_false_statement (stmt))))))));
		(__temp401 == NULL ? NULL : (__temp401 = (g_object_unref (__temp401), NULL)));
	} else {
		ValaCCodeIfStatement* __temp402 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE ((__temp402 = vala_ccode_if_statement_new (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_if_statement_get_condition (stmt)))), VALA_CCODE_STATEMENT (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_if_statement_get_true_statement (stmt)))), NULL))));
		(__temp402 == NULL ? NULL : (__temp402 = (g_object_unref (__temp402), NULL)));
	}
	vala_code_generator_create_temp_decl (self, VALA_STATEMENT (stmt), vala_if_statement_get_condition (stmt)->temp_vars);
}


static void vala_code_generator_real_visit_while_statement (ValaCodeVisitor* base, ValaWhileStatement* stmt)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_WHILE_STATEMENT (stmt));
	ValaCCodeWhileStatement* __temp403 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE ((__temp403 = vala_ccode_while_statement_new (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_while_statement_get_condition (stmt)))), VALA_CCODE_STATEMENT (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_while_statement_get_body (stmt))))))));
	(__temp403 == NULL ? NULL : (__temp403 = (g_object_unref (__temp403), NULL)));
	vala_code_generator_create_temp_decl (self, VALA_STATEMENT (stmt), vala_while_statement_get_condition (stmt)->temp_vars);
}


static void vala_code_generator_real_visit_for_statement (ValaCodeVisitor* base, ValaForStatement* stmt)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_FOR_STATEMENT (stmt));
	ValaCCodeForStatement* cfor = vala_ccode_for_statement_new (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_for_statement_get_condition (stmt)))), VALA_CCODE_STATEMENT (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_for_statement_get_body (stmt)))));
	{
		GList* init_expr_it;
		for (init_expr_it = vala_for_statement_get_initializer (stmt); init_expr_it != NULL; init_expr_it = init_expr_it->next) {
			ValaExpression* init_expr = init_expr_it->data;
			{
				vala_ccode_for_statement_add_initializer (cfor, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (init_expr))));
			}
		}
	}
	{
		GList* it_expr_it;
		for (it_expr_it = vala_for_statement_get_iterator (stmt); it_expr_it != NULL; it_expr_it = it_expr_it->next) {
			ValaExpression* it_expr = it_expr_it->data;
			{
				vala_ccode_for_statement_add_iterator (cfor, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (it_expr))));
			}
		}
	}
	vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE (cfor));
	vala_code_generator_create_temp_decl (self, VALA_STATEMENT (stmt), vala_for_statement_get_condition (stmt)->temp_vars);
	(cfor == NULL ? NULL : (cfor = (g_object_unref (cfor), NULL)));
}


static void vala_code_generator_real_visit_end_foreach_statement (ValaCodeVisitor* base, ValaForeachStatement* stmt)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_FOREACH_STATEMENT (stmt));
	ValaCCodeBlock* cblock = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
	if (VALA_IS_ARRAY (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_foreach_statement_get_collection (stmt))))) {
		char* it_name = g_strdup_printf ("%s_it", vala_foreach_statement_get_variable_name (stmt));
		ValaCCodeDeclaration* citdecl = vala_ccode_declaration_new (vala_type_reference_get_cname (vala_expression_get_static_type (vala_foreach_statement_get_collection (stmt)), FALSE));
		ValaCCodeVariableDeclarator* __temp404 = NULL;
		vala_ccode_declaration_add_declarator (citdecl, VALA_CCODE_DECLARATOR ((__temp404 = vala_ccode_variable_declarator_new (it_name))));
		(__temp404 == NULL ? NULL : (__temp404 = (g_object_unref (__temp404), NULL)));
		vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (citdecl));
		ValaCCodeBlock* cbody = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
		ValaCCodeDeclaration* cdecl = vala_ccode_declaration_new (vala_type_reference_get_cname (vala_foreach_statement_get_type_reference (stmt), FALSE));
		ValaCCodeVariableDeclarator* __temp405 = NULL;
		vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp405 = vala_ccode_variable_declarator_new_with_initializer (vala_foreach_statement_get_variable_name (stmt), VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (g_strdup_printf ("*%s", it_name)))))));
		(__temp405 == NULL ? NULL : (__temp405 = (g_object_unref (__temp405), NULL)));
		vala_ccode_block_add_statement (cbody, VALA_CCODE_NODE (cdecl));
		vala_ccode_block_add_statement (cbody, vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_foreach_statement_get_body (stmt))));
		ValaCCodeBinaryExpression* ccond = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_INEQUALITY, VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (g_strdup_printf ("*%s", it_name))), VALA_CCODE_EXPRESSION (vala_ccode_constant_new ("NULL")));
		ValaCCodeForStatement* cfor = vala_ccode_for_statement_new (VALA_CCODE_EXPRESSION (ccond), VALA_CCODE_STATEMENT (cbody));
		ValaCCodeAssignment* __temp406 = NULL;
		vala_ccode_for_statement_add_initializer (cfor, VALA_CCODE_EXPRESSION ((__temp406 = vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (it_name)), VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_foreach_statement_get_collection (stmt)))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))));
		(__temp406 == NULL ? NULL : (__temp406 = (g_object_unref (__temp406), NULL)));
		ValaCCodeAssignment* __temp407 = NULL;
		vala_ccode_for_statement_add_iterator (cfor, VALA_CCODE_EXPRESSION ((__temp407 = vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (it_name)), VALA_CCODE_EXPRESSION (vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_PLUS, VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (it_name)), VALA_CCODE_EXPRESSION (vala_ccode_constant_new ("1")))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))));
		(__temp407 == NULL ? NULL : (__temp407 = (g_object_unref (__temp407), NULL)));
		vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (cfor));
		(it_name == NULL ? NULL : (it_name = (g_free (it_name), NULL)));
		(citdecl == NULL ? NULL : (citdecl = (g_object_unref (citdecl), NULL)));
		(cbody == NULL ? NULL : (cbody = (g_object_unref (cbody), NULL)));
		(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
		(ccond == NULL ? NULL : (ccond = (g_object_unref (ccond), NULL)));
		(cfor == NULL ? NULL : (cfor = (g_object_unref (cfor), NULL)));
	} else {
		if (g_utf8_collate (vala_data_type_get_name (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_foreach_statement_get_collection (stmt)))), "List") == 0 || g_utf8_collate (vala_data_type_get_name (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_foreach_statement_get_collection (stmt)))), "SList") == 0) {
			char* it_name = g_strdup_printf ("%s_it", vala_foreach_statement_get_variable_name (stmt));
			ValaCCodeDeclaration* citdecl = vala_ccode_declaration_new (vala_type_reference_get_cname (vala_expression_get_static_type (vala_foreach_statement_get_collection (stmt)), FALSE));
			ValaCCodeVariableDeclarator* __temp408 = NULL;
			vala_ccode_declaration_add_declarator (citdecl, VALA_CCODE_DECLARATOR ((__temp408 = vala_ccode_variable_declarator_new (it_name))));
			(__temp408 == NULL ? NULL : (__temp408 = (g_object_unref (__temp408), NULL)));
			vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (citdecl));
			ValaCCodeBlock* cbody = g_object_new (VALA_TYPE_CCODE_BLOCK, NULL);
			ValaCCodeDeclaration* cdecl = vala_ccode_declaration_new (vala_type_reference_get_cname (vala_foreach_statement_get_type_reference (stmt), FALSE));
			ValaCCodeVariableDeclarator* __temp409 = NULL;
			vala_ccode_declaration_add_declarator (cdecl, VALA_CCODE_DECLARATOR ((__temp409 = vala_ccode_variable_declarator_new_with_initializer (vala_foreach_statement_get_variable_name (stmt), VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (it_name)), "data"))))));
			(__temp409 == NULL ? NULL : (__temp409 = (g_object_unref (__temp409), NULL)));
			vala_ccode_block_add_statement (cbody, VALA_CCODE_NODE (cdecl));
			vala_ccode_block_add_statement (cbody, vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_foreach_statement_get_body (stmt))));
			ValaCCodeBinaryExpression* ccond = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_INEQUALITY, VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (it_name)), VALA_CCODE_EXPRESSION (vala_ccode_constant_new ("NULL")));
			ValaCCodeForStatement* cfor = vala_ccode_for_statement_new (VALA_CCODE_EXPRESSION (ccond), VALA_CCODE_STATEMENT (cbody));
			ValaCCodeAssignment* __temp410 = NULL;
			vala_ccode_for_statement_add_initializer (cfor, VALA_CCODE_EXPRESSION ((__temp410 = vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (it_name)), VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_foreach_statement_get_collection (stmt)))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))));
			(__temp410 == NULL ? NULL : (__temp410 = (g_object_unref (__temp410), NULL)));
			ValaCCodeAssignment* __temp411 = NULL;
			vala_ccode_for_statement_add_iterator (cfor, VALA_CCODE_EXPRESSION ((__temp411 = vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (it_name)), VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (it_name)), "next")), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))));
			(__temp411 == NULL ? NULL : (__temp411 = (g_object_unref (__temp411), NULL)));
			vala_ccode_block_add_statement (cblock, VALA_CCODE_NODE (cfor));
			(it_name == NULL ? NULL : (it_name = (g_free (it_name), NULL)));
			(citdecl == NULL ? NULL : (citdecl = (g_object_unref (citdecl), NULL)));
			(cbody == NULL ? NULL : (cbody = (g_object_unref (cbody), NULL)));
			(cdecl == NULL ? NULL : (cdecl = (g_object_unref (cdecl), NULL)));
			(ccond == NULL ? NULL : (ccond = (g_object_unref (ccond), NULL)));
			(cfor == NULL ? NULL : (cfor = (g_object_unref (cfor), NULL)));
		}
	}
	vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE (cblock));
	(cblock == NULL ? NULL : (cblock = (g_object_unref (cblock), NULL)));
}


static void vala_code_generator_real_visit_break_statement (ValaCodeVisitor* base, ValaBreakStatement* stmt)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_BREAK_STATEMENT (stmt));
	ValaCCodeBreakStatement* __temp412 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE ((__temp412 = g_object_new (VALA_TYPE_CCODE_BREAK_STATEMENT, NULL))));
	(__temp412 == NULL ? NULL : (__temp412 = (g_object_unref (__temp412), NULL)));
}


static void vala_code_generator_real_visit_continue_statement (ValaCodeVisitor* base, ValaContinueStatement* stmt)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_CONTINUE_STATEMENT (stmt));
	ValaCCodeContinueStatement* __temp413 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE ((__temp413 = g_object_new (VALA_TYPE_CCODE_CONTINUE_STATEMENT, NULL))));
	(__temp413 == NULL ? NULL : (__temp413 = (g_object_unref (__temp413), NULL)));
}


static void vala_code_generator_append_local_free (ValaCodeGenerator* self, ValaSymbol* sym, ValaCCodeFragment* cfrag, gboolean stop_at_loop)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (sym == NULL || VALA_IS_SYMBOL (sym));
	g_return_if_fail (cfrag == NULL || VALA_IS_CCODE_FRAGMENT (cfrag));
	ValaBlock* b = VALA_BLOCK (vala_symbol_get_node (sym));
	GList* local_vars = vala_block_get_local_variables (b);
	{
		GList* decl_it;
		for (decl_it = local_vars; decl_it != NULL; decl_it = decl_it->next) {
			ValaVariableDeclarator* decl = decl_it->data;
			{
				if (vala_symbol_get_active (vala_code_node_get_symbol (VALA_CODE_NODE (decl))) && vala_data_type_is_reference_type (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl))) && vala_type_reference_get_takes_ownership (vala_variable_declarator_get_type_reference (decl))) {
					ValaCCodeExpressionStatement* __temp415 = NULL;
					ValaCCodeIdentifier* __temp414 = NULL;
					vala_ccode_fragment_append (cfrag, VALA_CCODE_NODE ((__temp415 = vala_ccode_expression_statement_new (vala_code_generator_get_unref_expression (self, VALA_CCODE_EXPRESSION ((__temp414 = vala_ccode_identifier_new (vala_variable_declarator_get_name (decl)))), vala_variable_declarator_get_type_reference (decl))))));
					(__temp415 == NULL ? NULL : (__temp415 = (g_object_unref (__temp415), NULL)));
					(__temp414 == NULL ? NULL : (__temp414 = (g_object_unref (__temp414), NULL)));
				}
			}
		}
	}
	if (VALA_IS_BLOCK (vala_symbol_get_node (vala_symbol_get_parent_symbol (sym)))) {
		vala_code_generator_append_local_free (self, vala_symbol_get_parent_symbol (sym), cfrag, stop_at_loop);
	}
	(local_vars == NULL ? NULL : (local_vars = (g_list_free (local_vars), NULL)));
}


static void vala_code_generator_create_local_free (ValaCodeGenerator* self, ValaStatement* stmt)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (stmt == NULL || VALA_IS_STATEMENT (stmt));
	if (!vala_code_generator_get_memory_management (self)) {
		return;
	}
	ValaCCodeFragment* cfrag = g_object_new (VALA_TYPE_CCODE_FRAGMENT, NULL);
	vala_code_generator_append_local_free (self, self->priv->current_symbol, cfrag, FALSE);
	vala_ccode_fragment_append (cfrag, vala_code_node_get_ccodenode (VALA_CODE_NODE (stmt)));
	vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE (cfrag));
	(cfrag == NULL ? NULL : (cfrag = (g_object_unref (cfrag), NULL)));
}


static gboolean vala_code_generator_append_local_free_expr (ValaCodeGenerator* self, ValaSymbol* sym, ValaCCodeCommaExpression* ccomma, gboolean stop_at_loop)
{
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (self), FALSE);
	g_return_val_if_fail (sym == NULL || VALA_IS_SYMBOL (sym), FALSE);
	g_return_val_if_fail (ccomma == NULL || VALA_IS_CCODE_COMMA_EXPRESSION (ccomma), FALSE);
	gboolean found = FALSE;
	ValaBlock* b = VALA_BLOCK (vala_symbol_get_node (sym));
	GList* local_vars = vala_block_get_local_variables (b);
	{
		GList* decl_it;
		for (decl_it = local_vars; decl_it != NULL; decl_it = decl_it->next) {
			ValaVariableDeclarator* decl = decl_it->data;
			{
				if (vala_symbol_get_active (vala_code_node_get_symbol (VALA_CODE_NODE (decl))) && vala_data_type_is_reference_type (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl))) && vala_type_reference_get_takes_ownership (vala_variable_declarator_get_type_reference (decl))) {
					found = TRUE;
					ValaCCodeExpression* __temp417 = NULL;
					ValaCCodeIdentifier* __temp416 = NULL;
					vala_ccode_comma_expression_append_expression (ccomma, (__temp417 = vala_code_generator_get_unref_expression (self, VALA_CCODE_EXPRESSION ((__temp416 = vala_ccode_identifier_new (vala_variable_declarator_get_name (decl)))), vala_variable_declarator_get_type_reference (decl))));
					(__temp417 == NULL ? NULL : (__temp417 = (g_object_unref (__temp417), NULL)));
					(__temp416 == NULL ? NULL : (__temp416 = (g_object_unref (__temp416), NULL)));
				}
			}
		}
	}
	if (VALA_IS_BLOCK (vala_symbol_get_node (vala_symbol_get_parent_symbol (sym)))) {
		found = found || vala_code_generator_append_local_free_expr (self, vala_symbol_get_parent_symbol (sym), ccomma, stop_at_loop);
	}
	gboolean __temp418;
	return (__temp418 = found, (local_vars == NULL ? NULL : (local_vars = (g_list_free (local_vars), NULL))), __temp418);
	(local_vars == NULL ? NULL : (local_vars = (g_list_free (local_vars), NULL)));
}


static void vala_code_generator_create_local_free_expr (ValaCodeGenerator* self, ValaExpression* expr)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (expr == NULL || VALA_IS_EXPRESSION (expr));
	if (!vala_code_generator_get_memory_management (self)) {
		return;
	}
	ValaVariableDeclarator* return_expr_decl = vala_code_generator_get_temp_variable_declarator (self, vala_expression_get_static_type (expr));
	ValaCCodeCommaExpression* ccomma = g_object_new (VALA_TYPE_CCODE_COMMA_EXPRESSION, NULL);
	ValaCCodeAssignment* __temp419 = NULL;
	vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION ((__temp419 = vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_variable_declarator_get_name (return_expr_decl))), VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (expr))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))));
	(__temp419 == NULL ? NULL : (__temp419 = (g_object_unref (__temp419), NULL)));
	if (!vala_code_generator_append_local_free_expr (self, self->priv->current_symbol, ccomma, FALSE)) {
		(return_expr_decl == NULL ? NULL : (return_expr_decl = (g_object_unref (return_expr_decl), NULL)));
		(ccomma == NULL ? NULL : (ccomma = (g_object_unref (ccomma), NULL)));
		return;
	}
	/* no local variables need to be freed */
	ValaCCodeIdentifier* __temp420 = NULL;
	vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION ((__temp420 = vala_ccode_identifier_new (vala_variable_declarator_get_name (return_expr_decl)))));
	(__temp420 == NULL ? NULL : (__temp420 = (g_object_unref (__temp420), NULL)));
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE (ccomma));
	ValaVariableDeclarator* __temp421 = NULL;
	expr->temp_vars = g_list_append (expr->temp_vars, (__temp421 = return_expr_decl, (__temp421 == NULL ? __temp421 : g_object_ref (__temp421))));
	(return_expr_decl == NULL ? NULL : (return_expr_decl = (g_object_unref (return_expr_decl), NULL)));
	(ccomma == NULL ? NULL : (ccomma = (g_object_unref (ccomma), NULL)));
}


static void vala_code_generator_real_visit_return_statement (ValaCodeVisitor* base, ValaReturnStatement* stmt)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_RETURN_STATEMENT (stmt));
	if (vala_return_statement_get_return_expression (stmt) == NULL) {
		ValaCCodeReturnStatement* __temp422 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE ((__temp422 = vala_ccode_return_statement_new (NULL))));
		(__temp422 == NULL ? NULL : (__temp422 = (g_object_unref (__temp422), NULL)));
		vala_code_generator_create_local_free (self, VALA_STATEMENT (stmt));
	} else {
		vala_code_generator_create_local_free_expr (self, vala_return_statement_get_return_expression (stmt));
		ValaCCodeReturnStatement* __temp423 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (stmt), VALA_CCODE_NODE ((__temp423 = vala_ccode_return_statement_new (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_return_statement_get_return_expression (stmt))))))));
		(__temp423 == NULL ? NULL : (__temp423 = (g_object_unref (__temp423), NULL)));
		vala_code_generator_create_temp_decl (self, VALA_STATEMENT (stmt), vala_return_statement_get_return_expression (stmt)->temp_vars);
	}
}


static void vala_code_generator_real_visit_boolean_literal (ValaCodeVisitor* base, ValaBooleanLiteral* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_BOOLEAN_LITERAL (expr));
	if (vala_boolean_literal_get_value (expr)) {
		ValaCCodeConstant* __temp424 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp424 = vala_ccode_constant_new ("TRUE"))));
		(__temp424 == NULL ? NULL : (__temp424 = (g_object_unref (__temp424), NULL)));
	} else {
		ValaCCodeConstant* __temp425 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp425 = vala_ccode_constant_new ("FALSE"))));
		(__temp425 == NULL ? NULL : (__temp425 = (g_object_unref (__temp425), NULL)));
	}
}


static void vala_code_generator_real_visit_character_literal (ValaCodeVisitor* base, ValaCharacterLiteral* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_CHARACTER_LITERAL (expr));
	ValaCCodeConstant* __temp426 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp426 = vala_ccode_constant_new (vala_character_literal_get_value (expr)))));
	(__temp426 == NULL ? NULL : (__temp426 = (g_object_unref (__temp426), NULL)));
}


static void vala_code_generator_real_visit_integer_literal (ValaCodeVisitor* base, ValaIntegerLiteral* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_INTEGER_LITERAL (expr));
	ValaCCodeConstant* __temp427 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp427 = vala_ccode_constant_new (vala_integer_literal_get_value (expr)))));
	(__temp427 == NULL ? NULL : (__temp427 = (g_object_unref (__temp427), NULL)));
}


static void vala_code_generator_real_visit_real_literal (ValaCodeVisitor* base, ValaRealLiteral* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_REAL_LITERAL (expr));
	ValaCCodeConstant* __temp428 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp428 = vala_ccode_constant_new (vala_real_literal_get_value (expr)))));
	(__temp428 == NULL ? NULL : (__temp428 = (g_object_unref (__temp428), NULL)));
}


static void vala_code_generator_real_visit_string_literal (ValaCodeVisitor* base, ValaStringLiteral* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_STRING_LITERAL (expr));
	ValaCCodeConstant* __temp429 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp429 = vala_ccode_constant_new (vala_string_literal_get_value (expr)))));
	(__temp429 == NULL ? NULL : (__temp429 = (g_object_unref (__temp429), NULL)));
}


static void vala_code_generator_real_visit_null_literal (ValaCodeVisitor* base, ValaNullLiteral* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_NULL_LITERAL (expr));
	ValaCCodeConstant* __temp430 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp430 = vala_ccode_constant_new ("NULL"))));
	(__temp430 == NULL ? NULL : (__temp430 = (g_object_unref (__temp430), NULL)));
}


static void vala_code_generator_real_visit_literal_expression (ValaCodeVisitor* base, ValaLiteralExpression* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_LITERAL_EXPRESSION (expr));
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_literal_expression_get_literal (expr))));
	vala_code_generator_visit_expression (self, VALA_EXPRESSION (expr));
}


static void vala_code_generator_process_cmember (ValaCodeGenerator* self, ValaMemberAccess* expr, ValaCCodeExpression* pub_inst, ValaDataType* base_type)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (VALA_IS_MEMBER_ACCESS (expr));
	g_return_if_fail (pub_inst == NULL || VALA_IS_CCODE_EXPRESSION (pub_inst));
	g_return_if_fail (base_type == NULL || VALA_IS_DATA_TYPE (base_type));
	if (VALA_IS_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))))) {
		ValaMethod* m = VALA_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))));
		if (!vala_method_get_overrides (m)) {
			ValaCCodeIdentifier* __temp431 = NULL;
			vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp431 = vala_ccode_identifier_new (vala_method_get_cname (m)))));
			(__temp431 == NULL ? NULL : (__temp431 = (g_object_unref (__temp431), NULL)));
		} else {
			ValaCCodeIdentifier* __temp432 = NULL;
			vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp432 = vala_ccode_identifier_new (vala_method_get_cname (vala_method_get_base_method (m))))));
			(__temp432 == NULL ? NULL : (__temp432 = (g_object_unref (__temp432), NULL)));
		}
	} else {
		if (VALA_IS_FIELD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))))) {
			ValaField* f = VALA_FIELD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))));
			if (vala_field_get_instance (f)) {
				ValaCCodeExpression* typed_inst = NULL;
				if (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (f)))) != base_type) {
					ValaCCodeExpression* __temp433 = NULL;
					typed_inst = (__temp433 = VALA_CCODE_EXPRESSION (vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_data_type_get_upper_case_cname ((VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (f)))))), NULL))))), (typed_inst == NULL ? NULL : (typed_inst = (g_object_unref (typed_inst), NULL))), __temp433);
					vala_ccode_function_call_add_argument ((VALA_CCODE_FUNCTION_CALL (typed_inst)), pub_inst);
				} else {
					ValaCCodeExpression* __temp435 = NULL;
					ValaCCodeExpression* __temp434 = NULL;
					typed_inst = (__temp435 = (__temp434 = pub_inst, (__temp434 == NULL ? __temp434 : g_object_ref (__temp434))), (typed_inst == NULL ? NULL : (typed_inst = (g_object_unref (typed_inst), NULL))), __temp435);
				}
				ValaCCodeExpression* inst = NULL;
				if (f->access == VALA_MEMBER_ACCESSIBILITY_PRIVATE) {
					ValaCCodeExpression* __temp436 = NULL;
					inst = (__temp436 = VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (typed_inst, "priv")), (inst == NULL ? NULL : (inst = (g_object_unref (inst), NULL))), __temp436);
				} else {
					ValaCCodeExpression* __temp438 = NULL;
					ValaCCodeExpression* __temp437 = NULL;
					inst = (__temp438 = (__temp437 = typed_inst, (__temp437 == NULL ? __temp437 : g_object_ref (__temp437))), (inst == NULL ? NULL : (inst = (g_object_unref (inst), NULL))), __temp438);
				}
				if (vala_data_type_is_reference_type ((VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (f)))))))) {
					ValaCCodeMemberAccess* __temp439 = NULL;
					vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp439 = vala_ccode_member_access_new_pointer (inst, vala_field_get_cname (f)))));
					(__temp439 == NULL ? NULL : (__temp439 = (g_object_unref (__temp439), NULL)));
				} else {
					ValaCCodeMemberAccess* __temp440 = NULL;
					vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp440 = vala_ccode_member_access_new (inst, vala_field_get_cname (f)))));
					(__temp440 == NULL ? NULL : (__temp440 = (g_object_unref (__temp440), NULL)));
				}
				(typed_inst == NULL ? NULL : (typed_inst = (g_object_unref (typed_inst), NULL)));
				(inst == NULL ? NULL : (inst = (g_object_unref (inst), NULL)));
			} else {
				if (VALA_IS_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (f)))))) {
					ValaDataType* t = VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (f)))));
					ValaCCodeIdentifier* __temp442 = NULL;
					char* __temp441 = NULL;
					vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp442 = vala_ccode_identifier_new (g_strdup_printf ("%s_%s", (__temp441 = vala_data_type_get_lower_case_cname (t, NULL)), vala_field_get_cname (f))))));
					(__temp442 == NULL ? NULL : (__temp442 = (g_object_unref (__temp442), NULL)));
					(__temp441 == NULL ? NULL : (__temp441 = (g_free (__temp441), NULL)));
				} else {
					ValaCCodeIdentifier* __temp443 = NULL;
					vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp443 = vala_ccode_identifier_new (vala_field_get_cname (f)))));
					(__temp443 == NULL ? NULL : (__temp443 = (g_object_unref (__temp443), NULL)));
				}
			}
		} else {
			if (VALA_IS_CONSTANT (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))))) {
				ValaConstant* c = VALA_CONSTANT (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))));
				ValaCCodeIdentifier* __temp444 = NULL;
				vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp444 = vala_ccode_identifier_new (vala_constant_get_cname (c)))));
				(__temp444 == NULL ? NULL : (__temp444 = (g_object_unref (__temp444), NULL)));
			} else {
				if (VALA_IS_PROPERTY (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))))) {
					ValaProperty* prop = VALA_PROPERTY (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))));
					ValaClass* cl = VALA_CLASS (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (prop)))));
					if (!vala_property_get_no_accessor_method (prop)) {
						char* __temp445 = NULL;
						ValaCCodeFunctionCall* __temp446 = NULL;
						ValaCCodeFunctionCall* ccall = (__temp446 = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (g_strdup_printf ("%s_get_%s", (__temp445 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL)), vala_property_get_name (prop))))), (__temp445 == NULL ? NULL : (__temp445 = (g_free (__temp445), NULL))), __temp446);
						/* explicitly use strong reference as ccast
						 * gets unrefed at the end of the inner block
						 */
						ValaCCodeExpression* __temp447 = NULL;
						ValaCCodeExpression* typed_pub_inst = (__temp447 = pub_inst, (__temp447 == NULL ? __temp447 : g_object_ref (__temp447)));
						/* cast if necessary */
						if (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (prop)))) != base_type) {
							ValaCCodeFunctionCall* ccast = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_data_type_get_upper_case_cname ((VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (prop)))))), NULL))));
							vala_ccode_function_call_add_argument (ccast, pub_inst);
							ValaCCodeExpression* __temp449 = NULL;
							ValaCCodeFunctionCall* __temp448 = NULL;
							typed_pub_inst = (__temp449 = VALA_CCODE_EXPRESSION ((__temp448 = ccast, (__temp448 == NULL ? __temp448 : g_object_ref (__temp448)))), (typed_pub_inst == NULL ? NULL : (typed_pub_inst = (g_object_unref (typed_pub_inst), NULL))), __temp449);
							(ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL)));
						}
						vala_ccode_function_call_add_argument (ccall, typed_pub_inst);
						vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE (ccall));
						(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
						(typed_pub_inst == NULL ? NULL : (typed_pub_inst = (g_object_unref (typed_pub_inst), NULL)));
					} else {
						ValaCCodeFunctionCall* ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_object_get")));
						ValaCCodeFunctionCall* ccast = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("G_OBJECT")));
						vala_ccode_function_call_add_argument (ccast, pub_inst);
						vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION (ccast));
						/* property name is second argument of g_object_get*/
						ValaCCodeConstant* __temp450 = NULL;
						vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp450 = vala_property_get_canonical_cconstant (prop))));
						(__temp450 == NULL ? NULL : (__temp450 = (g_object_unref (__temp450), NULL)));
						/* we need a temporary variable to save the property value*/
						ValaVariableDeclarator* temp_decl = vala_code_generator_get_temp_variable_declarator (self, vala_expression_get_static_type (VALA_EXPRESSION (expr)));
						ValaVariableDeclarator* __temp451 = NULL;
						self->priv->temp_vars = g_list_prepend (self->priv->temp_vars, (__temp451 = temp_decl, (__temp451 == NULL ? __temp451 : g_object_ref (__temp451))));
						ValaCCodeIdentifier* ctemp = vala_ccode_identifier_new (vala_variable_declarator_get_name (temp_decl));
						ValaCCodeUnaryExpression* __temp452 = NULL;
						vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp452 = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, VALA_CCODE_EXPRESSION (ctemp)))));
						(__temp452 == NULL ? NULL : (__temp452 = (g_object_unref (__temp452), NULL)));
						ValaCCodeConstant* __temp453 = NULL;
						vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp453 = vala_ccode_constant_new ("NULL"))));
						(__temp453 == NULL ? NULL : (__temp453 = (g_object_unref (__temp453), NULL)));
						ValaCCodeCommaExpression* ccomma = g_object_new (VALA_TYPE_CCODE_COMMA_EXPRESSION, NULL);
						vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION (ccall));
						vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION (ctemp));
						vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE (ccomma));
						(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
						(ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL)));
						(temp_decl == NULL ? NULL : (temp_decl = (g_object_unref (temp_decl), NULL)));
						(ctemp == NULL ? NULL : (ctemp = (g_object_unref (ctemp), NULL)));
						(ccomma == NULL ? NULL : (ccomma = (g_object_unref (ccomma), NULL)));
					}
				} else {
					if (VALA_IS_ENUM_VALUE (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))))) {
						ValaEnumValue* ev = VALA_ENUM_VALUE (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))));
						ValaCCodeConstant* __temp454 = NULL;
						vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp454 = vala_ccode_constant_new (vala_enum_value_get_cname (ev)))));
						(__temp454 == NULL ? NULL : (__temp454 = (g_object_unref (__temp454), NULL)));
					} else {
						if (VALA_IS_VARIABLE_DECLARATOR (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))))) {
							ValaVariableDeclarator* decl = VALA_VARIABLE_DECLARATOR (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))));
							ValaCCodeIdentifier* __temp455 = NULL;
							vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp455 = vala_ccode_identifier_new (vala_variable_declarator_get_name (decl)))));
							(__temp455 == NULL ? NULL : (__temp455 = (g_object_unref (__temp455), NULL)));
						} else {
							if (VALA_IS_FORMAL_PARAMETER (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))))) {
								ValaFormalParameter* p = VALA_FORMAL_PARAMETER (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))));
								if (g_utf8_collate (vala_formal_parameter_get_name (p), "this") == 0) {
									vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE (pub_inst));
								} else {
									if (vala_type_reference_get_is_out (vala_formal_parameter_get_type_reference (p)) || vala_type_reference_get_reference_to_value_type (vala_formal_parameter_get_type_reference (p))) {
										ValaCCodeIdentifier* __temp456 = NULL;
										vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp456 = vala_ccode_identifier_new (g_strdup_printf ("*%s", vala_formal_parameter_get_name (p))))));
										(__temp456 == NULL ? NULL : (__temp456 = (g_object_unref (__temp456), NULL)));
									} else {
										ValaCCodeIdentifier* __temp457 = NULL;
										vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp457 = vala_ccode_identifier_new (vala_formal_parameter_get_name (p)))));
										(__temp457 == NULL ? NULL : (__temp457 = (g_object_unref (__temp457), NULL)));
									}
								}
							} else {
								if (VALA_IS_SIGNAL (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))))) {
									ValaSignal* sig = VALA_SIGNAL (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))));
									ValaCCodeFunctionCall* ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_signal_emit_by_name")));
									ValaCCodeFunctionCall* ccast = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("G_OBJECT")));
									vala_ccode_function_call_add_argument (ccast, pub_inst);
									vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION (ccast));
									ValaCCodeConstant* __temp458 = NULL;
									vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp458 = vala_ccode_constant_new (g_strdup_printf ("\"%s\"", vala_signal_get_name (sig))))));
									(__temp458 == NULL ? NULL : (__temp458 = (g_object_unref (__temp458), NULL)));
									vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE (ccall));
									(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
									(ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL)));
								}
							}
						}
					}
				}
			}
		}
	}
}


static void vala_code_generator_real_visit_parenthesized_expression (ValaCodeVisitor* base, ValaParenthesizedExpression* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_PARENTHESIZED_EXPRESSION (expr));
	ValaCCodeParenthesizedExpression* __temp459 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp459 = vala_ccode_parenthesized_expression_new (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_parenthesized_expression_get_inner (expr))))))));
	(__temp459 == NULL ? NULL : (__temp459 = (g_object_unref (__temp459), NULL)));
	vala_code_generator_visit_expression (self, VALA_EXPRESSION (expr));
}


static void vala_code_generator_real_visit_member_access (ValaCodeVisitor* base, ValaMemberAccess* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_MEMBER_ACCESS (expr));
	ValaCCodeExpression* pub_inst = NULL;
	ValaDataType* base_type = NULL;
	if (vala_member_access_get_inner (expr) == NULL) {
		ValaCCodeExpression* __temp460 = NULL;
		pub_inst = (__temp460 = VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("self")), (pub_inst == NULL ? NULL : (pub_inst = (g_object_unref (pub_inst), NULL))), __temp460);
		if (self->priv->current_type_symbol != NULL) {
			/* base type is available if this is a type method */
			base_type = VALA_DATA_TYPE (vala_symbol_get_node (self->priv->current_type_symbol));
		}
	} else {
		ValaCCodeExpression* __temp462 = NULL;
		ValaCCodeExpression* __temp461 = NULL;
		pub_inst = (__temp462 = (__temp461 = VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_member_access_get_inner (expr)))), (__temp461 == NULL ? __temp461 : g_object_ref (__temp461))), (pub_inst == NULL ? NULL : (pub_inst = (g_object_unref (pub_inst), NULL))), __temp462);
		if (vala_expression_get_static_type (vala_member_access_get_inner (expr)) != NULL) {
			base_type = vala_type_reference_get_data_type (vala_expression_get_static_type (vala_member_access_get_inner (expr)));
		}
	}
	vala_code_generator_process_cmember (self, expr, pub_inst, base_type);
	vala_code_generator_visit_expression (self, VALA_EXPRESSION (expr));
	(pub_inst == NULL ? NULL : (pub_inst = (g_object_unref (pub_inst), NULL)));
}


static void vala_code_generator_real_visit_end_invocation_expression (ValaCodeVisitor* base, ValaInvocationExpression* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_INVOCATION_EXPRESSION (expr));
	ValaCCodeFunctionCall* ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_invocation_expression_get_call (expr)))));
	ValaMethod* m = NULL;
	GList* params = NULL;
	if (!(VALA_IS_MEMBER_ACCESS (vala_invocation_expression_get_call (expr)))) {
		vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "unsupported method invocation");
		(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
		(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
		return;
	}
	ValaMemberAccess* ma = VALA_MEMBER_ACCESS (vala_invocation_expression_get_call (expr));
	if (VALA_IS_VARIABLE_DECLARATOR (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_invocation_expression_get_call (expr))))) {
		ValaVariableDeclarator* decl = VALA_VARIABLE_DECLARATOR (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_invocation_expression_get_call (expr))));
		ValaCallback* cb = VALA_CALLBACK (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl)));
		GList* __temp463 = NULL;
		params = (__temp463 = vala_callback_get_parameters (cb), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp463);
	} else {
		if (VALA_IS_FORMAL_PARAMETER (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_invocation_expression_get_call (expr))))) {
			ValaFormalParameter* param = VALA_FORMAL_PARAMETER (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_invocation_expression_get_call (expr))));
			ValaCallback* cb = VALA_CALLBACK (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param)));
			GList* __temp464 = NULL;
			params = (__temp464 = vala_callback_get_parameters (cb), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp464);
		} else {
			if (VALA_IS_FIELD (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_invocation_expression_get_call (expr))))) {
				ValaField* f = VALA_FIELD (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_invocation_expression_get_call (expr))));
				ValaCallback* cb = VALA_CALLBACK (vala_type_reference_get_data_type (vala_field_get_type_reference (f)));
				GList* __temp465 = NULL;
				params = (__temp465 = vala_callback_get_parameters (cb), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp465);
			} else {
				if (VALA_IS_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_invocation_expression_get_call (expr))))) {
					m = VALA_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_invocation_expression_get_call (expr))));
					GList* __temp466 = NULL;
					params = (__temp466 = vala_method_get_parameters (m), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp466);
				} else {
					if (VALA_IS_SIGNAL (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_invocation_expression_get_call (expr))))) {
						ValaSignal* sig = VALA_SIGNAL (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_invocation_expression_get_call (expr))));
						GList* __temp467 = NULL;
						params = (__temp467 = vala_signal_get_parameters (sig), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp467);
						ValaCCodeFunctionCall* __temp469 = NULL;
						ValaCCodeFunctionCall* __temp468 = NULL;
						ccall = (__temp469 = (__temp468 = VALA_CCODE_FUNCTION_CALL (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_invocation_expression_get_call (expr)))), (__temp468 == NULL ? __temp468 : g_object_ref (__temp468))), (ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL))), __temp469);
					}
				}
			}
		}
	}
	/* explicitly use strong reference as ccall gets unrefed
	 * at end of inner block
	 */
	ValaCCodeExpression* instance = NULL;
	if (m != NULL && vala_method_get_instance (m)) {
		ValaMethod* base_method = m;
		if (vala_method_get_overrides (m)) {
			base_method = vala_method_get_base_method (m);
		}
		gboolean req_cast = FALSE;
		if (vala_member_access_get_inner (ma) == NULL) {
			ValaCCodeExpression* __temp470 = NULL;
			instance = (__temp470 = VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("self")), (instance == NULL ? NULL : (instance = (g_object_unref (instance), NULL))), __temp470);
			/* require casts for overriden and inherited methods */
			req_cast = vala_method_get_overrides (m) || (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (m))) != self->priv->current_type_symbol);
		} else {
			ValaCCodeExpression* __temp472 = NULL;
			ValaCCodeExpression* __temp471 = NULL;
			instance = (__temp472 = (__temp471 = VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_member_access_get_inner (ma)))), (__temp471 == NULL ? __temp471 : g_object_ref (__temp471))), (instance == NULL ? NULL : (instance = (g_object_unref (instance), NULL))), __temp472);
			/* reqiure casts if the type of the used instance is
			 * different than the type which declared the method */
			req_cast = vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (base_method)))) != vala_type_reference_get_data_type (vala_expression_get_static_type (vala_member_access_get_inner (ma)));
		}
		if (req_cast && vala_data_type_is_reference_type ((VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (m)))))))) {
			ValaCCodeFunctionCall* ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_data_type_get_upper_case_cname ((VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (base_method)))))), NULL))));
			vala_ccode_function_call_add_argument (ccall, instance);
			ValaCCodeExpression* __temp474 = NULL;
			ValaCCodeFunctionCall* __temp473 = NULL;
			instance = (__temp474 = VALA_CCODE_EXPRESSION ((__temp473 = ccall, (__temp473 == NULL ? __temp473 : g_object_ref (__temp473)))), (instance == NULL ? NULL : (instance = (g_object_unref (instance), NULL))), __temp474);
			(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
		}
		if (!vala_method_get_instance_last (m)) {
			vala_ccode_function_call_add_argument (ccall, instance);
		}
	}
	int i = 1;
	{
		GList* arg_it;
		for (arg_it = vala_invocation_expression_get_argument_list (expr); arg_it != NULL; arg_it = arg_it->next) {
			ValaExpression* arg = arg_it->data;
			{
				/* explicitly use strong reference as ccall gets
				 * unrefed at end of inner block
				 */
				ValaCCodeExpression* __temp475 = NULL;
				ValaCCodeExpression* cexpr = (__temp475 = VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (arg))), (__temp475 == NULL ? __temp475 : g_object_ref (__temp475)));
				if (params != NULL) {
					ValaFormalParameter* param = VALA_FORMAL_PARAMETER (params->data);
					if (!vala_formal_parameter_get_ellipsis (param) && vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param)) != NULL && vala_data_type_is_reference_type (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param))) && vala_type_reference_get_data_type (vala_expression_get_static_type (arg)) != NULL && vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param)) != vala_type_reference_get_data_type (vala_expression_get_static_type (arg))) {
						ValaCCodeFunctionCall* ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_data_type_get_upper_case_cname (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param)), NULL))));
						vala_ccode_function_call_add_argument (ccall, cexpr);
						ValaCCodeExpression* __temp477 = NULL;
						ValaCCodeFunctionCall* __temp476 = NULL;
						cexpr = (__temp477 = VALA_CCODE_EXPRESSION ((__temp476 = ccall, (__temp476 == NULL ? __temp476 : g_object_ref (__temp476)))), (cexpr == NULL ? NULL : (cexpr = (g_object_unref (cexpr), NULL))), __temp477);
						(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
					}
				}
				vala_ccode_function_call_add_argument (ccall, cexpr);
				i++;
				if (params != NULL) {
					GList* __temp479 = NULL;
					GList* __temp478 = NULL;
					params = (__temp479 = (__temp478 = params->next, (__temp478 == NULL ? __temp478 : g_list_copy (__temp478))), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp479);
				}
				(cexpr == NULL ? NULL : (cexpr = (g_object_unref (cexpr), NULL)));
			}
		}
	}
	while (params != NULL) {
		ValaFormalParameter* param = VALA_FORMAL_PARAMETER (params->data);
		if (vala_formal_parameter_get_ellipsis (param)) {
			break;
		}
		if (vala_formal_parameter_get_default_expression (param) == NULL) {
			char* __temp480 = NULL;
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp480 = g_strdup_printf ("no default expression for argument %d", i)));
			(__temp480 == NULL ? NULL : (__temp480 = (g_free (__temp480), NULL)));
			(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
			(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
			(instance == NULL ? NULL : (instance = (g_object_unref (instance), NULL)));
			return;
		}
		/* evaluate default expression here as the code
		 * generator might not have visited the formal
		 * parameter yet */
		vala_code_node_accept (VALA_CODE_NODE (vala_formal_parameter_get_default_expression (param)), VALA_CODE_VISITOR (self));
		vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_formal_parameter_get_default_expression (param)))));
		i++;
		GList* __temp482 = NULL;
		GList* __temp481 = NULL;
		params = (__temp482 = (__temp481 = params->next, (__temp481 == NULL ? __temp481 : g_list_copy (__temp481))), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp482);
	}
	if (m != NULL && vala_method_get_instance (m) && vala_method_get_instance_last (m)) {
		vala_ccode_function_call_add_argument (ccall, instance);
	}
	if (m != NULL && vala_method_get_instance (m) && vala_method_get_returns_modified_pointer (m)) {
		ValaCCodeAssignment* __temp483 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp483 = vala_ccode_assignment_new (instance, VALA_CCODE_EXPRESSION (ccall), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))));
		(__temp483 == NULL ? NULL : (__temp483 = (g_object_unref (__temp483), NULL)));
	} else {
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE (ccall));
		vala_code_generator_visit_expression (self, VALA_EXPRESSION (expr));
	}
	(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
	(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
	(instance == NULL ? NULL : (instance = (g_object_unref (instance), NULL)));
}


static void vala_code_generator_real_visit_element_access (ValaCodeVisitor* base, ValaElementAccess* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_ELEMENT_ACCESS (expr));
	ValaCCodeElementAccess* __temp484 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp484 = vala_ccode_element_access_new (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_element_access_get_container (expr)))), VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_element_access_get_index (expr))))))));
	(__temp484 == NULL ? NULL : (__temp484 = (g_object_unref (__temp484), NULL)));
}


static void vala_code_generator_real_visit_postfix_expression (ValaCodeVisitor* base, ValaPostfixExpression* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_POSTFIX_EXPRESSION (expr));
	if (vala_postfix_expression_get_increment (expr)) {
		ValaCCodeUnaryExpression* __temp485 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp485 = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_POSTFIX_INCREMENT, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_postfix_expression_get_inner (expr))))))));
		(__temp485 == NULL ? NULL : (__temp485 = (g_object_unref (__temp485), NULL)));
	} else {
		ValaCCodeUnaryExpression* __temp486 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp486 = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_POSTFIX_DECREMENT, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_postfix_expression_get_inner (expr))))))));
		(__temp486 == NULL ? NULL : (__temp486 = (g_object_unref (__temp486), NULL)));
	}
	vala_code_generator_visit_expression (self, VALA_EXPRESSION (expr));
}


static ValaCCodeExpression* vala_code_generator_get_ref_expression (ValaCodeGenerator* self, ValaExpression* expr)
{
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (self), NULL);
	g_return_val_if_fail (VALA_IS_EXPRESSION (expr), NULL);
	/* (temp = expr, temp == NULL ? temp : ref (temp))
	 *
	 * can be simplified to
	 * ref (expr)
	 * if static type of expr is non-null
	 */
	if (vala_type_reference_get_data_type (vala_expression_get_static_type (expr)) == NULL && vala_type_reference_get_type_parameter (vala_expression_get_static_type (expr)) != NULL) {
		vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "Missing generics support for memory management");
		return NULL;
	}
	char* ref_function = NULL;
	if (vala_data_type_is_reference_counting (vala_type_reference_get_data_type (vala_expression_get_static_type (expr)))) {
		ref_function = vala_data_type_get_ref_function (vala_type_reference_get_data_type (vala_expression_get_static_type (expr)));
	} else {
		ref_function = vala_data_type_get_dup_function (vala_type_reference_get_data_type (vala_expression_get_static_type (expr)));
	}
	ValaCCodeFunctionCall* ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (ref_function)));
	if (vala_type_reference_get_non_null (vala_expression_get_static_type (expr))) {
		vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (expr))));
		ValaCCodeFunctionCall* __temp488 = NULL;
		ValaCCodeFunctionCall* __temp489 = NULL;
		return (__temp489 = (__temp488 = ccall, (__temp488 == NULL ? __temp488 : g_object_ref (__temp488))), (ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL))), __temp489);
	} else {
		ValaVariableDeclarator* decl = vala_code_generator_get_temp_variable_declarator (self, vala_expression_get_static_type (expr));
		ValaVariableDeclarator* __temp490 = NULL;
		self->priv->temp_vars = g_list_prepend (self->priv->temp_vars, (__temp490 = decl, (__temp490 == NULL ? __temp490 : g_object_ref (__temp490))));
		ValaCCodeIdentifier* ctemp = vala_ccode_identifier_new (vala_variable_declarator_get_name (decl));
		ValaCCodeBinaryExpression* cisnull = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_EQUALITY, VALA_CCODE_EXPRESSION (ctemp), VALA_CCODE_EXPRESSION (vala_ccode_constant_new ("NULL")));
		vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION (ctemp));
		ValaCCodeCommaExpression* ccomma = g_object_new (VALA_TYPE_CCODE_COMMA_EXPRESSION, NULL);
		ValaCCodeAssignment* __temp491 = NULL;
		vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION ((__temp491 = vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (ctemp), VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (expr))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))));
		(__temp491 == NULL ? NULL : (__temp491 = (g_object_unref (__temp491), NULL)));
		ValaCCodeConditionalExpression* __temp492 = NULL;
		vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION ((__temp492 = vala_ccode_conditional_expression_new (VALA_CCODE_EXPRESSION (cisnull), VALA_CCODE_EXPRESSION (ctemp), VALA_CCODE_EXPRESSION (ccall)))));
		(__temp492 == NULL ? NULL : (__temp492 = (g_object_unref (__temp492), NULL)));
		ValaCCodeCommaExpression* __temp493 = NULL;
		ValaCCodeCommaExpression* __temp494 = NULL;
		return (__temp494 = (__temp493 = ccomma, (__temp493 == NULL ? __temp493 : g_object_ref (__temp493))), (decl == NULL ? NULL : (decl = (g_object_unref (decl), NULL))), (ctemp == NULL ? NULL : (ctemp = (g_object_unref (ctemp), NULL))), (cisnull == NULL ? NULL : (cisnull = (g_object_unref (cisnull), NULL))), (ccomma == NULL ? NULL : (ccomma = (g_object_unref (ccomma), NULL))), __temp494);
		(decl == NULL ? NULL : (decl = (g_object_unref (decl), NULL)));
		(ctemp == NULL ? NULL : (ctemp = (g_object_unref (ctemp), NULL)));
		(cisnull == NULL ? NULL : (cisnull = (g_object_unref (cisnull), NULL)));
		(ccomma == NULL ? NULL : (ccomma = (g_object_unref (ccomma), NULL)));
	}
	(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
}


static void vala_code_generator_visit_expression (ValaCodeGenerator* self, ValaExpression* expr)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	g_return_if_fail (VALA_IS_EXPRESSION (expr));
	if (vala_expression_get_static_type (expr) != NULL && vala_type_reference_get_transfers_ownership (vala_expression_get_static_type (expr)) && vala_type_reference_get_floating_reference (vala_expression_get_static_type (expr))) {
		/* constructor of GInitiallyUnowned subtype
		 * returns floating reference, sink it
		 */
		ValaCCodeFunctionCall* csink = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_object_ref_sink")));
		vala_ccode_function_call_add_argument (csink, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (expr))));
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE (csink));
		(csink == NULL ? NULL : (csink = (g_object_unref (csink), NULL)));
	}
	if (vala_expression_get_ref_leaked (expr)) {
		ValaVariableDeclarator* decl = vala_code_generator_get_temp_variable_declarator (self, vala_expression_get_static_type (expr));
		ValaVariableDeclarator* __temp495 = NULL;
		self->priv->temp_vars = g_list_prepend (self->priv->temp_vars, (__temp495 = decl, (__temp495 == NULL ? __temp495 : g_object_ref (__temp495))));
		ValaVariableDeclarator* __temp496 = NULL;
		self->priv->temp_ref_vars = g_list_prepend (self->priv->temp_ref_vars, (__temp496 = decl, (__temp496 == NULL ? __temp496 : g_object_ref (__temp496))));
		ValaCCodeParenthesizedExpression* __temp497 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp497 = vala_ccode_parenthesized_expression_new (VALA_CCODE_EXPRESSION (vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_variable_declarator_get_name (decl))), VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (expr))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))))));
		(__temp497 == NULL ? NULL : (__temp497 = (g_object_unref (__temp497), NULL)));
		(decl == NULL ? NULL : (decl = (g_object_unref (decl), NULL)));
	} else {
		if (vala_expression_get_ref_missing (expr)) {
			ValaCCodeExpression* __temp498 = NULL;
			vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp498 = vala_code_generator_get_ref_expression (self, expr))));
			(__temp498 == NULL ? NULL : (__temp498 = (g_object_unref (__temp498), NULL)));
		}
	}
}


static void vala_code_generator_real_visit_end_object_creation_expression (ValaCodeVisitor* base, ValaObjectCreationExpression* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_OBJECT_CREATION_EXPRESSION (expr));
	if (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)) == NULL) {
		/* no construction method*/
		if (VALA_IS_CLASS (vala_type_reference_get_data_type (vala_object_creation_expression_get_type_reference (expr)))) {
			ValaCCodeFunctionCall* ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_object_new")));
			ValaCCodeConstant* __temp499 = NULL;
			vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp499 = vala_ccode_constant_new (vala_data_type_get_type_id (vala_type_reference_get_data_type (vala_object_creation_expression_get_type_reference (expr)))))));
			(__temp499 == NULL ? NULL : (__temp499 = (g_object_unref (__temp499), NULL)));
			ValaCCodeConstant* __temp500 = NULL;
			vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp500 = vala_ccode_constant_new ("NULL"))));
			(__temp500 == NULL ? NULL : (__temp500 = (g_object_unref (__temp500), NULL)));
			vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE (ccall));
			(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
		} else {
			ValaCCodeFunctionCall* ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_new0")));
			ValaCCodeConstant* __temp501 = NULL;
			vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp501 = vala_ccode_constant_new (vala_data_type_get_cname (vala_type_reference_get_data_type (vala_object_creation_expression_get_type_reference (expr)))))));
			(__temp501 == NULL ? NULL : (__temp501 = (g_object_unref (__temp501), NULL)));
			ValaCCodeConstant* __temp502 = NULL;
			vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp502 = vala_ccode_constant_new ("1"))));
			(__temp502 == NULL ? NULL : (__temp502 = (g_object_unref (__temp502), NULL)));
			vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE (ccall));
			(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
		}
	} else {
		/* use construction method*/
		ValaMethod* m = VALA_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))));
		GList* params = vala_method_get_parameters (m);
		ValaCCodeFunctionCall* ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_method_get_cname (m))));
		int i = 1;
		{
			GList* arg_it;
			for (arg_it = vala_object_creation_expression_get_argument_list (expr); arg_it != NULL; arg_it = arg_it->next) {
				ValaExpression* arg = arg_it->data;
				{
					/* explicitly use strong reference as ccall gets
					 * unrefed at end of inner block
					 */
					ValaCCodeExpression* __temp503 = NULL;
					ValaCCodeExpression* cexpr = (__temp503 = VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (arg))), (__temp503 == NULL ? __temp503 : g_object_ref (__temp503)));
					if (params != NULL) {
						ValaFormalParameter* param = VALA_FORMAL_PARAMETER (params->data);
						if (!vala_formal_parameter_get_ellipsis (param) && vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param)) != NULL && vala_data_type_is_reference_type (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param))) && vala_type_reference_get_data_type (vala_expression_get_static_type (arg)) != NULL && vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param)) != vala_type_reference_get_data_type (vala_expression_get_static_type (arg))) {
							ValaCCodeFunctionCall* ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_data_type_get_upper_case_cname (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param)), NULL))));
							vala_ccode_function_call_add_argument (ccall, cexpr);
							ValaCCodeExpression* __temp505 = NULL;
							ValaCCodeFunctionCall* __temp504 = NULL;
							cexpr = (__temp505 = VALA_CCODE_EXPRESSION ((__temp504 = ccall, (__temp504 == NULL ? __temp504 : g_object_ref (__temp504)))), (cexpr == NULL ? NULL : (cexpr = (g_object_unref (cexpr), NULL))), __temp505);
							(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
						}
					}
					vala_ccode_function_call_add_argument (ccall, cexpr);
					i++;
					if (params != NULL) {
						GList* __temp507 = NULL;
						GList* __temp506 = NULL;
						params = (__temp507 = (__temp506 = params->next, (__temp506 == NULL ? __temp506 : g_list_copy (__temp506))), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp507);
					}
					(cexpr == NULL ? NULL : (cexpr = (g_object_unref (cexpr), NULL)));
				}
			}
		}
		while (params != NULL) {
			ValaFormalParameter* param = VALA_FORMAL_PARAMETER (params->data);
			if (vala_formal_parameter_get_ellipsis (param)) {
				break;
			}
			if (vala_formal_parameter_get_default_expression (param) == NULL) {
				char* __temp508 = NULL;
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp508 = g_strdup_printf ("no default expression for argument %d", i)));
				(__temp508 == NULL ? NULL : (__temp508 = (g_free (__temp508), NULL)));
				(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
				(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
				return;
			}
			/* evaluate default expression here as the code
			 * generator might not have visited the formal
			 * parameter yet */
			vala_code_node_accept (VALA_CODE_NODE (vala_formal_parameter_get_default_expression (param)), VALA_CODE_VISITOR (self));
			vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_formal_parameter_get_default_expression (param)))));
			i++;
			GList* __temp510 = NULL;
			GList* __temp509 = NULL;
			params = (__temp510 = (__temp509 = params->next, (__temp509 == NULL ? __temp509 : g_list_copy (__temp509))), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp510);
		}
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE (ccall));
		(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
		(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
	}
	vala_code_generator_visit_expression (self, VALA_EXPRESSION (expr));
}


static void vala_code_generator_real_visit_typeof_expression (ValaCodeVisitor* base, ValaTypeofExpression* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_TYPEOF_EXPRESSION (expr));
	ValaCCodeIdentifier* __temp511 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp511 = vala_ccode_identifier_new (vala_data_type_get_type_id (vala_type_reference_get_data_type (vala_typeof_expression_get_type_reference (expr)))))));
	(__temp511 == NULL ? NULL : (__temp511 = (g_object_unref (__temp511), NULL)));
}


static void vala_code_generator_real_visit_unary_expression (ValaCodeVisitor* base, ValaUnaryExpression* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_UNARY_EXPRESSION (expr));
	ValaCCodeUnaryOperator op;
	if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_PLUS) {
		op = VALA_CCODE_UNARY_OPERATOR_PLUS;
	} else {
		if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_MINUS) {
			op = VALA_CCODE_UNARY_OPERATOR_MINUS;
		} else {
			if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_LOGICAL_NEGATION) {
				op = VALA_CCODE_UNARY_OPERATOR_LOGICAL_NEGATION;
			} else {
				if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_BITWISE_COMPLEMENT) {
					op = VALA_CCODE_UNARY_OPERATOR_BITWISE_COMPLEMENT;
				} else {
					if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_INCREMENT) {
						op = VALA_CCODE_UNARY_OPERATOR_PREFIX_INCREMENT;
					} else {
						if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_DECREMENT) {
							op = VALA_CCODE_UNARY_OPERATOR_PREFIX_DECREMENT;
						} else {
							if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_REF) {
								op = VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF;
							} else {
								if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_OUT) {
									op = VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF;
								}
							}
						}
					}
				}
			}
		}
	}
	ValaCCodeUnaryExpression* __temp512 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp512 = vala_ccode_unary_expression_new (op, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_unary_expression_get_inner (expr))))))));
	(__temp512 == NULL ? NULL : (__temp512 = (g_object_unref (__temp512), NULL)));
	vala_code_generator_visit_expression (self, VALA_EXPRESSION (expr));
}


static void vala_code_generator_real_visit_cast_expression (ValaCodeVisitor* base, ValaCastExpression* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_CAST_EXPRESSION (expr));
	if (VALA_IS_STRUCT (vala_type_reference_get_data_type (vala_cast_expression_get_type_reference (expr))) || VALA_IS_ENUM (vala_type_reference_get_data_type (vala_cast_expression_get_type_reference (expr))) || VALA_IS_FLAGS (vala_type_reference_get_data_type (vala_cast_expression_get_type_reference (expr)))) {
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_cast_expression_get_inner (expr))));
	} else {
		ValaInstanceCast* __temp513 = NULL;
		vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp513 = vala_instance_cast_new (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_cast_expression_get_inner (expr)))), vala_type_reference_get_data_type (vala_cast_expression_get_type_reference (expr))))));
		(__temp513 == NULL ? NULL : (__temp513 = (g_object_unref (__temp513), NULL)));
	}
	vala_code_generator_visit_expression (self, VALA_EXPRESSION (expr));
}


static void vala_code_generator_real_visit_binary_expression (ValaCodeVisitor* base, ValaBinaryExpression* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_BINARY_EXPRESSION (expr));
	ValaCCodeBinaryOperator op;
	if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_PLUS) {
		op = VALA_CCODE_BINARY_OPERATOR_PLUS;
	} else {
		if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_MINUS) {
			op = VALA_CCODE_BINARY_OPERATOR_MINUS;
		} else {
			if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_MUL) {
				op = VALA_CCODE_BINARY_OPERATOR_MUL;
			} else {
				if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_DIV) {
					op = VALA_CCODE_BINARY_OPERATOR_DIV;
				} else {
					if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_MOD) {
						op = VALA_CCODE_BINARY_OPERATOR_MOD;
					} else {
						if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_SHIFT_LEFT) {
							op = VALA_CCODE_BINARY_OPERATOR_SHIFT_LEFT;
						} else {
							if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_SHIFT_RIGHT) {
								op = VALA_CCODE_BINARY_OPERATOR_SHIFT_RIGHT;
							} else {
								if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_LESS_THAN) {
									op = VALA_CCODE_BINARY_OPERATOR_LESS_THAN;
								} else {
									if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_GREATER_THAN) {
										op = VALA_CCODE_BINARY_OPERATOR_GREATER_THAN;
									} else {
										if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_LESS_THAN_OR_EQUAL) {
											op = VALA_CCODE_BINARY_OPERATOR_LESS_THAN_OR_EQUAL;
										} else {
											if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_GREATER_THAN_OR_EQUAL) {
												op = VALA_CCODE_BINARY_OPERATOR_GREATER_THAN_OR_EQUAL;
											} else {
												if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_EQUALITY) {
													op = VALA_CCODE_BINARY_OPERATOR_EQUALITY;
												} else {
													if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_INEQUALITY) {
														op = VALA_CCODE_BINARY_OPERATOR_INEQUALITY;
													} else {
														if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_BITWISE_AND) {
															op = VALA_CCODE_BINARY_OPERATOR_BITWISE_AND;
														} else {
															if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_BITWISE_OR) {
																op = VALA_CCODE_BINARY_OPERATOR_BITWISE_OR;
															} else {
																if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_BITWISE_XOR) {
																	op = VALA_CCODE_BINARY_OPERATOR_BITWISE_XOR;
																} else {
																	if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_AND) {
																		op = VALA_CCODE_BINARY_OPERATOR_AND;
																	} else {
																		if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_OR) {
																			op = VALA_CCODE_BINARY_OPERATOR_OR;
																		}
																	}
																}
															}
														}
													}
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}
	}
	ValaCCodeBinaryExpression* __temp514 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp514 = vala_ccode_binary_expression_new (op, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_binary_expression_get_left (expr)))), VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_binary_expression_get_right (expr))))))));
	(__temp514 == NULL ? NULL : (__temp514 = (g_object_unref (__temp514), NULL)));
	vala_code_generator_visit_expression (self, VALA_EXPRESSION (expr));
}


static void vala_code_generator_real_visit_type_check (ValaCodeVisitor* base, ValaTypeCheck* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_TYPE_CHECK (expr));
	ValaCCodeFunctionCall* ccheck = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_data_type_get_upper_case_cname (vala_type_reference_get_data_type (vala_type_check_get_type_reference (expr)), "IS_"))));
	vala_ccode_function_call_add_argument (ccheck, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_type_check_get_expression (expr)))));
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE (ccheck));
	(ccheck == NULL ? NULL : (ccheck = (g_object_unref (ccheck), NULL)));
}


static void vala_code_generator_real_visit_conditional_expression (ValaCodeVisitor* base, ValaConditionalExpression* expr)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_CONDITIONAL_EXPRESSION (expr));
	ValaCCodeConditionalExpression* __temp515 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (expr), VALA_CCODE_NODE ((__temp515 = vala_ccode_conditional_expression_new (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_conditional_expression_get_condition (expr)))), VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_conditional_expression_get_true_expression (expr)))), VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_conditional_expression_get_false_expression (expr))))))));
	(__temp515 == NULL ? NULL : (__temp515 = (g_object_unref (__temp515), NULL)));
}


static void vala_code_generator_real_visit_end_lambda_expression (ValaCodeVisitor* base, ValaLambdaExpression* l)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_LAMBDA_EXPRESSION (l));
	ValaCCodeIdentifier* __temp516 = NULL;
	vala_code_node_set_ccodenode (VALA_CODE_NODE (l), VALA_CCODE_NODE ((__temp516 = vala_ccode_identifier_new (vala_method_get_cname (vala_lambda_expression_get_method (l))))));
	(__temp516 == NULL ? NULL : (__temp516 = (g_object_unref (__temp516), NULL)));
}


static void vala_code_generator_real_visit_end_assignment (ValaCodeVisitor* base, ValaAssignment* a)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (base);
	g_return_if_fail (VALA_IS_ASSIGNMENT (a));
	ValaMemberAccess* ma = VALA_MEMBER_ACCESS (vala_assignment_get_left (a));
	if (VALA_IS_PROPERTY (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_assignment_get_left (a))))) {
		ValaProperty* prop = VALA_PROPERTY (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_assignment_get_left (a))));
		ValaClass* cl = VALA_CLASS (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (prop)))));
		if (vala_member_access_get_inner (ma) == NULL && VALA_IS_STATEMENT (vala_code_node_get_parent_node (VALA_CODE_NODE (a))) && vala_statement_get_construction ((VALA_STATEMENT (vala_code_node_get_parent_node (VALA_CODE_NODE (a)))))) {
			/* this property is used as a construction parameter*/
			ValaCCodeIdentifier* cpointer = vala_ccode_identifier_new ("__params_it");
			ValaCCodeCommaExpression* ccomma = g_object_new (VALA_TYPE_CCODE_COMMA_EXPRESSION, NULL);
			/* set name in array for current parameter*/
			ValaCCodeMemberAccess* cnamemember = vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (cpointer), "name");
			ValaCCodeAssignment* cnameassign = vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (cnamemember), VALA_CCODE_EXPRESSION (vala_property_get_canonical_cconstant (prop)), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE);
			vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION (cnameassign));
			ValaCCodeUnaryExpression* gvaluearg = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, VALA_CCODE_EXPRESSION (vala_ccode_member_access_new_pointer (VALA_CCODE_EXPRESSION (cpointer), "value")));
			/* initialize GValue in array for current parameter*/
			ValaCCodeFunctionCall* cvalueinit = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("g_value_init")));
			vala_ccode_function_call_add_argument (cvalueinit, VALA_CCODE_EXPRESSION (gvaluearg));
			ValaCCodeIdentifier* __temp517 = NULL;
			vala_ccode_function_call_add_argument (cvalueinit, VALA_CCODE_EXPRESSION ((__temp517 = vala_ccode_identifier_new (vala_data_type_get_type_id (vala_type_reference_get_data_type (vala_property_get_type_reference (prop)))))));
			(__temp517 == NULL ? NULL : (__temp517 = (g_object_unref (__temp517), NULL)));
			vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION (cvalueinit));
			/* set GValue for current parameter*/
			ValaCCodeFunctionCall* cvalueset = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_code_generator_get_value_setter_function (self, vala_property_get_type_reference (prop))));
			vala_ccode_function_call_add_argument (cvalueset, VALA_CCODE_EXPRESSION (gvaluearg));
			vala_ccode_function_call_add_argument (cvalueset, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_assignment_get_right (a)))));
			vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION (cvalueset));
			/* move pointer to next parameter in array*/
			ValaCCodeUnaryExpression* __temp518 = NULL;
			vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION ((__temp518 = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_POSTFIX_INCREMENT, VALA_CCODE_EXPRESSION (cpointer)))));
			(__temp518 == NULL ? NULL : (__temp518 = (g_object_unref (__temp518), NULL)));
			vala_code_node_set_ccodenode (VALA_CODE_NODE (a), VALA_CCODE_NODE (ccomma));
			(cpointer == NULL ? NULL : (cpointer = (g_object_unref (cpointer), NULL)));
			(ccomma == NULL ? NULL : (ccomma = (g_object_unref (ccomma), NULL)));
			(cnamemember == NULL ? NULL : (cnamemember = (g_object_unref (cnamemember), NULL)));
			(cnameassign == NULL ? NULL : (cnameassign = (g_object_unref (cnameassign), NULL)));
			(gvaluearg == NULL ? NULL : (gvaluearg = (g_object_unref (gvaluearg), NULL)));
			(cvalueinit == NULL ? NULL : (cvalueinit = (g_object_unref (cvalueinit), NULL)));
			(cvalueset == NULL ? NULL : (cvalueset = (g_object_unref (cvalueset), NULL)));
		} else {
			char* set_func = g_strdup ("g_object_set");
			if (!vala_property_get_no_accessor_method (prop)) {
				char* __temp520 = NULL;
				char* __temp519 = NULL;
				set_func = (__temp520 = g_strdup_printf ("%s_set_%s", (__temp519 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (cl), NULL)), vala_property_get_name (prop)), (set_func == NULL ? NULL : (set_func = (g_free (set_func), NULL))), __temp520);
				(__temp519 == NULL ? NULL : (__temp519 = (g_free (__temp519), NULL)));
			}
			ValaCCodeFunctionCall* ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (set_func)));
			/* target instance is first argument */
			ValaCCodeExpression* instance = NULL;
			gboolean req_cast = FALSE;
			if (vala_member_access_get_inner (ma) == NULL) {
				ValaCCodeExpression* __temp521 = NULL;
				instance = (__temp521 = VALA_CCODE_EXPRESSION (vala_ccode_identifier_new ("self")), (instance == NULL ? NULL : (instance = (g_object_unref (instance), NULL))), __temp521);
				/* require casts for inherited properties */
				req_cast = (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (prop))) != self->priv->current_type_symbol);
			} else {
				ValaCCodeExpression* __temp523 = NULL;
				ValaCCodeExpression* __temp522 = NULL;
				instance = (__temp523 = (__temp522 = VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_member_access_get_inner (ma)))), (__temp522 == NULL ? __temp522 : g_object_ref (__temp522))), (instance == NULL ? NULL : (instance = (g_object_unref (instance), NULL))), __temp523);
				/* require casts if the type of the used instance is
				 * different than the type which declared the property */
				req_cast = vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (prop)))) != vala_type_reference_get_data_type (vala_expression_get_static_type (vala_member_access_get_inner (ma)));
			}
			if (req_cast && vala_data_type_is_reference_type ((VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (prop)))))))) {
				ValaCCodeFunctionCall* ccast = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_data_type_get_upper_case_cname ((VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (prop)))))), NULL))));
				vala_ccode_function_call_add_argument (ccast, instance);
				ValaCCodeExpression* __temp525 = NULL;
				ValaCCodeFunctionCall* __temp524 = NULL;
				instance = (__temp525 = VALA_CCODE_EXPRESSION ((__temp524 = ccast, (__temp524 == NULL ? __temp524 : g_object_ref (__temp524)))), (instance == NULL ? NULL : (instance = (g_object_unref (instance), NULL))), __temp525);
				(ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL)));
			}
			vala_ccode_function_call_add_argument (ccall, instance);
			ValaCCodeExpression* __temp526 = NULL;
			ValaCCodeExpression* cexpr = (__temp526 = VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_assignment_get_right (a)))), (__temp526 == NULL ? __temp526 : g_object_ref (__temp526)));
			if (vala_property_get_no_accessor_method (prop)) {
				/* property name is second argument of g_object_set */
				ValaCCodeConstant* __temp527 = NULL;
				vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp527 = vala_property_get_canonical_cconstant (prop))));
				(__temp527 == NULL ? NULL : (__temp527 = (g_object_unref (__temp527), NULL)));
			} else {
				if (vala_type_reference_get_data_type (vala_property_get_type_reference (prop)) != NULL && vala_data_type_is_reference_type (vala_type_reference_get_data_type (vala_property_get_type_reference (prop))) && vala_type_reference_get_data_type (vala_expression_get_static_type (vala_assignment_get_right (a))) != NULL && vala_type_reference_get_data_type (vala_property_get_type_reference (prop)) != vala_type_reference_get_data_type (vala_expression_get_static_type (vala_assignment_get_right (a)))) {
					/* cast is necessary */
					ValaCCodeFunctionCall* ccast = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_data_type_get_upper_case_cname (vala_type_reference_get_data_type (vala_property_get_type_reference (prop)), NULL))));
					vala_ccode_function_call_add_argument (ccast, cexpr);
					ValaCCodeExpression* __temp529 = NULL;
					ValaCCodeFunctionCall* __temp528 = NULL;
					cexpr = (__temp529 = VALA_CCODE_EXPRESSION ((__temp528 = ccast, (__temp528 == NULL ? __temp528 : g_object_ref (__temp528)))), (cexpr == NULL ? NULL : (cexpr = (g_object_unref (cexpr), NULL))), __temp529);
					(ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL)));
				}
			}
			vala_ccode_function_call_add_argument (ccall, cexpr);
			if (vala_property_get_no_accessor_method (prop)) {
				ValaCCodeConstant* __temp530 = NULL;
				vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp530 = vala_ccode_constant_new ("NULL"))));
				(__temp530 == NULL ? NULL : (__temp530 = (g_object_unref (__temp530), NULL)));
			}
			vala_code_node_set_ccodenode (VALA_CODE_NODE (a), VALA_CCODE_NODE (ccall));
			(set_func == NULL ? NULL : (set_func = (g_free (set_func), NULL)));
			(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
			(instance == NULL ? NULL : (instance = (g_object_unref (instance), NULL)));
			(cexpr == NULL ? NULL : (cexpr = (g_object_unref (cexpr), NULL)));
		}
	} else {
		if (VALA_IS_SIGNAL (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_assignment_get_left (a))))) {
			ValaSignal* sig = VALA_SIGNAL (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_assignment_get_left (a))));
			ValaMethod* m = VALA_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_assignment_get_right (a))));
			char* connect_func = "g_signal_connect_object";
			if (!vala_method_get_instance (m)) {
				connect_func = "g_signal_connect";
			}
			ValaCCodeFunctionCall* ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (connect_func)));
			if (vala_member_access_get_inner (ma) != NULL) {
				vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_member_access_get_inner (ma)))));
			} else {
				ValaCCodeIdentifier* __temp531 = NULL;
				vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp531 = vala_ccode_identifier_new ("self"))));
				(__temp531 == NULL ? NULL : (__temp531 = (g_object_unref (__temp531), NULL)));
			}
			ValaCCodeConstant* __temp532 = NULL;
			vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp532 = vala_ccode_constant_new (g_strdup_printf ("\"%s\"", vala_signal_get_name (sig))))));
			(__temp532 == NULL ? NULL : (__temp532 = (g_object_unref (__temp532), NULL)));
			ValaCCodeCastExpression* __temp533 = NULL;
			vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp533 = vala_ccode_cast_expression_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_method_get_cname (m))), "GCallback"))));
			(__temp533 == NULL ? NULL : (__temp533 = (g_object_unref (__temp533), NULL)));
			if (vala_method_get_instance (m)) {
				if (VALA_IS_MEMBER_ACCESS (vala_assignment_get_right (a))) {
					ValaMemberAccess* right_ma = VALA_MEMBER_ACCESS (vala_assignment_get_right (a));
					if (vala_member_access_get_inner (right_ma) != NULL) {
						vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_member_access_get_inner (right_ma)))));
					} else {
						ValaCCodeIdentifier* __temp534 = NULL;
						vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp534 = vala_ccode_identifier_new ("self"))));
						(__temp534 == NULL ? NULL : (__temp534 = (g_object_unref (__temp534), NULL)));
					}
				} else {
					if (VALA_IS_LAMBDA_EXPRESSION (vala_assignment_get_right (a))) {
						ValaCCodeIdentifier* __temp535 = NULL;
						vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp535 = vala_ccode_identifier_new ("self"))));
						(__temp535 == NULL ? NULL : (__temp535 = (g_object_unref (__temp535), NULL)));
					}
				}
				ValaCCodeConstant* __temp536 = NULL;
				vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp536 = vala_ccode_constant_new ("0"))));
				(__temp536 == NULL ? NULL : (__temp536 = (g_object_unref (__temp536), NULL)));
			} else {
				ValaCCodeConstant* __temp537 = NULL;
				vala_ccode_function_call_add_argument (ccall, VALA_CCODE_EXPRESSION ((__temp537 = vala_ccode_constant_new ("NULL"))));
				(__temp537 == NULL ? NULL : (__temp537 = (g_object_unref (__temp537), NULL)));
			}
			vala_code_node_set_ccodenode (VALA_CODE_NODE (a), VALA_CCODE_NODE (ccall));
			(ccall == NULL ? NULL : (ccall = (g_object_unref (ccall), NULL)));
		} else {
			/* explicitly use strong reference as ccast gets
			 * unrefed at end of inner block
			 */
			ValaCCodeExpression* __temp538 = NULL;
			ValaCCodeExpression* rhs = (__temp538 = VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_assignment_get_right (a)))), (__temp538 == NULL ? __temp538 : g_object_ref (__temp538)));
			if (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_assignment_get_left (a))) != NULL && vala_type_reference_get_data_type (vala_expression_get_static_type (vala_assignment_get_right (a))) != NULL && vala_data_type_is_reference_type (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_assignment_get_left (a)))) && vala_type_reference_get_data_type (vala_expression_get_static_type (vala_assignment_get_right (a))) != vala_type_reference_get_data_type (vala_expression_get_static_type (vala_assignment_get_left (a)))) {
				ValaCCodeFunctionCall* ccast = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_data_type_get_upper_case_cname (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_assignment_get_left (a))), NULL))));
				vala_ccode_function_call_add_argument (ccast, rhs);
				ValaCCodeExpression* __temp540 = NULL;
				ValaCCodeFunctionCall* __temp539 = NULL;
				rhs = (__temp540 = VALA_CCODE_EXPRESSION ((__temp539 = ccast, (__temp539 == NULL ? __temp539 : g_object_ref (__temp539)))), (rhs == NULL ? NULL : (rhs = (g_object_unref (rhs), NULL))), __temp540);
				(ccast == NULL ? NULL : (ccast = (g_object_unref (ccast), NULL)));
			}
			if (vala_code_generator_get_memory_management (self) && vala_type_reference_get_takes_ownership (vala_expression_get_static_type (vala_assignment_get_left (a)))) {
				/* unref old value */
				ValaCCodeCommaExpression* ccomma = g_object_new (VALA_TYPE_CCODE_COMMA_EXPRESSION, NULL);
				ValaVariableDeclarator* temp_decl = vala_code_generator_get_temp_variable_declarator (self, vala_expression_get_static_type (vala_assignment_get_left (a)));
				ValaVariableDeclarator* __temp541 = NULL;
				self->priv->temp_vars = g_list_prepend (self->priv->temp_vars, (__temp541 = temp_decl, (__temp541 == NULL ? __temp541 : g_object_ref (__temp541))));
				ValaCCodeAssignment* __temp542 = NULL;
				vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION ((__temp542 = vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_ccode_identifier_new (vala_variable_declarator_get_name (temp_decl))), rhs, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE))));
				(__temp542 == NULL ? NULL : (__temp542 = (g_object_unref (__temp542), NULL)));
				ValaCCodeExpression* __temp543 = NULL;
				vala_ccode_comma_expression_append_expression (ccomma, (__temp543 = vala_code_generator_get_unref_expression (self, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_assignment_get_left (a)))), vala_expression_get_static_type (vala_assignment_get_left (a)))));
				(__temp543 == NULL ? NULL : (__temp543 = (g_object_unref (__temp543), NULL)));
				ValaCCodeIdentifier* __temp544 = NULL;
				vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION ((__temp544 = vala_ccode_identifier_new (vala_variable_declarator_get_name (temp_decl)))));
				(__temp544 == NULL ? NULL : (__temp544 = (g_object_unref (__temp544), NULL)));
				ValaCCodeExpression* __temp546 = NULL;
				ValaCCodeCommaExpression* __temp545 = NULL;
				rhs = (__temp546 = VALA_CCODE_EXPRESSION ((__temp545 = ccomma, (__temp545 == NULL ? __temp545 : g_object_ref (__temp545)))), (rhs == NULL ? NULL : (rhs = (g_object_unref (rhs), NULL))), __temp546);
				(ccomma == NULL ? NULL : (ccomma = (g_object_unref (ccomma), NULL)));
				(temp_decl == NULL ? NULL : (temp_decl = (g_object_unref (temp_decl), NULL)));
			}
			ValaCCodeAssignmentOperator cop = VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE;
			if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_BITWISE_OR) {
				cop = VALA_CCODE_ASSIGNMENT_OPERATOR_BITWISE_OR;
			} else {
				if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_BITWISE_AND) {
					cop = VALA_CCODE_ASSIGNMENT_OPERATOR_BITWISE_AND;
				} else {
					if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_BITWISE_XOR) {
						cop = VALA_CCODE_ASSIGNMENT_OPERATOR_BITWISE_XOR;
					} else {
						if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_ADD) {
							cop = VALA_CCODE_ASSIGNMENT_OPERATOR_ADD;
						} else {
							if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_SUB) {
								cop = VALA_CCODE_ASSIGNMENT_OPERATOR_SUB;
							} else {
								if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_MUL) {
									cop = VALA_CCODE_ASSIGNMENT_OPERATOR_MUL;
								} else {
									if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_DIV) {
										cop = VALA_CCODE_ASSIGNMENT_OPERATOR_DIV;
									} else {
										if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_PERCENT) {
											cop = VALA_CCODE_ASSIGNMENT_OPERATOR_PERCENT;
										} else {
											if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_SHIFT_LEFT) {
												cop = VALA_CCODE_ASSIGNMENT_OPERATOR_SHIFT_LEFT;
											} else {
												if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_SHIFT_RIGHT) {
													cop = VALA_CCODE_ASSIGNMENT_OPERATOR_SHIFT_RIGHT;
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
			ValaCCodeAssignment* __temp547 = NULL;
			vala_code_node_set_ccodenode (VALA_CODE_NODE (a), VALA_CCODE_NODE ((__temp547 = vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode (VALA_CODE_NODE (vala_assignment_get_left (a)))), rhs, cop))));
			(__temp547 == NULL ? NULL : (__temp547 = (g_object_unref (__temp547), NULL)));
			(rhs == NULL ? NULL : (rhs = (g_object_unref (rhs), NULL)));
		}
	}
}


gboolean vala_code_generator_get_memory_management (ValaCodeGenerator* self)
{
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (self), FALSE);
	return self->priv->_memory_management;
}


void vala_code_generator_set_memory_management (ValaCodeGenerator* self, gboolean value)
{
	g_return_if_fail (VALA_IS_CODE_GENERATOR (self));
	self->priv->_memory_management = value;
}


void vala_code_generator_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (object);
	switch (property_id) {
		case VALA_CODE_GENERATOR_MEMORY_MANAGEMENT:
		g_value_set_boolean (value, vala_code_generator_get_memory_management (self));
		break;
	}
}


void vala_code_generator_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (object);
	switch (property_id) {
		case VALA_CODE_GENERATOR_MEMORY_MANAGEMENT:
		vala_code_generator_set_memory_management (self, g_value_get_boolean (value));
		break;
	}
}


static void vala_code_generator_class_init (ValaCodeGeneratorClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaCodeGeneratorPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_code_generator_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_code_generator_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_code_generator_dispose;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_source_file = vala_code_generator_real_visit_begin_source_file;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_source_file = vala_code_generator_real_visit_end_source_file;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_class = vala_code_generator_real_visit_begin_class;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_class = vala_code_generator_real_visit_end_class;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_struct = vala_code_generator_real_visit_begin_struct;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_interface = vala_code_generator_real_visit_begin_interface;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_interface = vala_code_generator_real_visit_end_interface;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_enum = vala_code_generator_real_visit_begin_enum;
	VALA_CODE_VISITOR_CLASS (klass)->visit_enum_value = vala_code_generator_real_visit_enum_value;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_callback = vala_code_generator_real_visit_end_callback;
	VALA_CODE_VISITOR_CLASS (klass)->visit_constant = vala_code_generator_real_visit_constant;
	VALA_CODE_VISITOR_CLASS (klass)->visit_field = vala_code_generator_real_visit_field;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_method = vala_code_generator_real_visit_begin_method;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_method = vala_code_generator_real_visit_end_method;
	VALA_CODE_VISITOR_CLASS (klass)->visit_formal_parameter = vala_code_generator_real_visit_formal_parameter;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_property = vala_code_generator_real_visit_end_property;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_property_accessor = vala_code_generator_real_visit_end_property_accessor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_constructor = vala_code_generator_real_visit_end_constructor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_block = vala_code_generator_real_visit_begin_block;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_block = vala_code_generator_real_visit_end_block;
	VALA_CODE_VISITOR_CLASS (klass)->visit_empty_statement = vala_code_generator_real_visit_empty_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_declaration_statement = vala_code_generator_real_visit_declaration_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_variable_declarator = vala_code_generator_real_visit_variable_declarator;
	VALA_CODE_VISITOR_CLASS (klass)->visit_initializer_list = vala_code_generator_real_visit_initializer_list;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_full_expression = vala_code_generator_real_visit_end_full_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_expression_statement = vala_code_generator_real_visit_expression_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_if_statement = vala_code_generator_real_visit_if_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_while_statement = vala_code_generator_real_visit_while_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_for_statement = vala_code_generator_real_visit_for_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_foreach_statement = vala_code_generator_real_visit_end_foreach_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_break_statement = vala_code_generator_real_visit_break_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_continue_statement = vala_code_generator_real_visit_continue_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_return_statement = vala_code_generator_real_visit_return_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_boolean_literal = vala_code_generator_real_visit_boolean_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_character_literal = vala_code_generator_real_visit_character_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_integer_literal = vala_code_generator_real_visit_integer_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_real_literal = vala_code_generator_real_visit_real_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_string_literal = vala_code_generator_real_visit_string_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_null_literal = vala_code_generator_real_visit_null_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_literal_expression = vala_code_generator_real_visit_literal_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_parenthesized_expression = vala_code_generator_real_visit_parenthesized_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_member_access = vala_code_generator_real_visit_member_access;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_invocation_expression = vala_code_generator_real_visit_end_invocation_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_element_access = vala_code_generator_real_visit_element_access;
	VALA_CODE_VISITOR_CLASS (klass)->visit_postfix_expression = vala_code_generator_real_visit_postfix_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_object_creation_expression = vala_code_generator_real_visit_end_object_creation_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_typeof_expression = vala_code_generator_real_visit_typeof_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_unary_expression = vala_code_generator_real_visit_unary_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_cast_expression = vala_code_generator_real_visit_cast_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_binary_expression = vala_code_generator_real_visit_binary_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_type_check = vala_code_generator_real_visit_type_check;
	VALA_CODE_VISITOR_CLASS (klass)->visit_conditional_expression = vala_code_generator_real_visit_conditional_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_lambda_expression = vala_code_generator_real_visit_end_lambda_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_assignment = vala_code_generator_real_visit_end_assignment;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CODE_GENERATOR_MEMORY_MANAGEMENT, g_param_spec_boolean ("memory-management", "foo", "bar", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void vala_code_generator_init (ValaCodeGenerator * self)
{
	self->priv = VALA_CODE_GENERATOR_GET_PRIVATE (self);
	self->priv->next_temp_var_id = 0;
}


static void vala_code_generator_dispose (GObject * obj)
{
	ValaCodeGenerator * self = VALA_CODE_GENERATOR (obj);
	(self->priv->root_symbol == NULL ? NULL : (self->priv->root_symbol = (g_object_unref (self->priv->root_symbol), NULL)));
	(self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL)));
	(self->priv->current_type_symbol == NULL ? NULL : (self->priv->current_type_symbol = (g_object_unref (self->priv->current_type_symbol), NULL)));
	(self->priv->header_begin == NULL ? NULL : (self->priv->header_begin = (g_object_unref (self->priv->header_begin), NULL)));
	(self->priv->header_type_declaration == NULL ? NULL : (self->priv->header_type_declaration = (g_object_unref (self->priv->header_type_declaration), NULL)));
	(self->priv->header_type_definition == NULL ? NULL : (self->priv->header_type_definition = (g_object_unref (self->priv->header_type_definition), NULL)));
	(self->priv->header_type_member_declaration == NULL ? NULL : (self->priv->header_type_member_declaration = (g_object_unref (self->priv->header_type_member_declaration), NULL)));
	(self->priv->source_begin == NULL ? NULL : (self->priv->source_begin = (g_object_unref (self->priv->source_begin), NULL)));
	(self->priv->source_include_directives == NULL ? NULL : (self->priv->source_include_directives = (g_object_unref (self->priv->source_include_directives), NULL)));
	(self->priv->source_type_member_declaration == NULL ? NULL : (self->priv->source_type_member_declaration = (g_object_unref (self->priv->source_type_member_declaration), NULL)));
	(self->priv->source_type_member_definition == NULL ? NULL : (self->priv->source_type_member_definition = (g_object_unref (self->priv->source_type_member_definition), NULL)));
	(self->priv->instance_struct == NULL ? NULL : (self->priv->instance_struct = (g_object_unref (self->priv->instance_struct), NULL)));
	(self->priv->type_struct == NULL ? NULL : (self->priv->type_struct = (g_object_unref (self->priv->type_struct), NULL)));
	(self->priv->instance_priv_struct == NULL ? NULL : (self->priv->instance_priv_struct = (g_object_unref (self->priv->instance_priv_struct), NULL)));
	(self->priv->prop_enum == NULL ? NULL : (self->priv->prop_enum = (g_object_unref (self->priv->prop_enum), NULL)));
	(self->priv->cenum == NULL ? NULL : (self->priv->cenum = (g_object_unref (self->priv->cenum), NULL)));
	(self->priv->function == NULL ? NULL : (self->priv->function = (g_object_unref (self->priv->function), NULL)));
	(self->priv->block == NULL ? NULL : (self->priv->block = (g_object_unref (self->priv->block), NULL)));
	(self->priv->temp_vars == NULL ? NULL : (self->priv->temp_vars = (g_list_foreach (self->priv->temp_vars, (GFunc) g_object_unref, NULL), g_list_free (self->priv->temp_vars), NULL)));
	(self->priv->temp_ref_vars == NULL ? NULL : (self->priv->temp_ref_vars = (g_list_foreach (self->priv->temp_ref_vars, (GFunc) g_object_unref, NULL), g_list_free (self->priv->temp_ref_vars), NULL)));
	(self->priv->bool_type == NULL ? NULL : (self->priv->bool_type = (g_object_unref (self->priv->bool_type), NULL)));
	(self->priv->int_type == NULL ? NULL : (self->priv->int_type = (g_object_unref (self->priv->int_type), NULL)));
	(self->priv->string_type == NULL ? NULL : (self->priv->string_type = (g_object_unref (self->priv->string_type), NULL)));
	ValaCodeGeneratorClass * klass;
	GObjectClass * parent_class;
	klass = VALA_CODE_GENERATOR_CLASS (g_type_class_peek (VALA_TYPE_CODE_GENERATOR));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_code_generator_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaCodeGeneratorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_code_generator_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaCodeGenerator), 0, (GInstanceInitFunc) vala_code_generator_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_CODE_VISITOR, "ValaCodeGenerator", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



